
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION INTERP3( FNAME, VNAME, CALLER,
     &                            JDATE, JTIME, RSIZE, BUFFER )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: INTERP3
!DEC$ ENDIF
C***********************************************************************
C  INTERP3 function body starts at line  135
C
C  NOTE:  MACHINE-DEPENDENT CODE !!!  depends upon FORTRAN : C
C       coupling between INTERP3() and BUFINT3()
C
C  FUNCTION:  interpolates data from Models-3 data file with logical
C       name FNAME, variable VNAME for all layers to date and time
C       JDATE (coded YYYYDDD) and time JTIME (HHMMSS), putting the
C       result in BUFFER.
C       File type must be CUSTOM, GRIDDED, or BOUNDARY.
C       For time-independent files, JDATE:JTIME are ignored.
C       Encapsulates file opens and reads, and all the book-keeping
C       and data structures necessary for that task.
C
C  USES C routine BUFINT3:  sets up all the data structures for the
C       call so that BUFINT3 can allocate and manage read-buffers
C       internally, call NCVGT and perform the interpolation.
C       (Actually, INTERP3() + BUFINT3() should be considered as
C       two parts of a single routine)
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  FNAME is a Models-3 data file containing
C       variable VNAME for a set of time steps enclosing JDATE:JTIME
C       File type must be CUSTOM3, GRDDED3, or BNDARY3.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       BUFINT3, CURRSTEP, INDEX1, JSTEP3, M3WARN, NEXTIME, OPEN3,
C       SECSDIFF, TIME2SEC
C
C  REVISION  HISTORY:
C       prototype 6/93 by CJC
C       modified  7/94 by CJC for restart (circular-buffer) files
C       modified  9/94 by CJC for new version of OPEN3()
C       modified 10/94 by CJC for new WRITE3 -- granularity at the
C       level of individual variables.  FNAME, VNAME are now CHAR*(*).
C       modified  6/95 by CJC to check VTYPE3( VID,FID )
C       modified 11/95 by CJC to check dates for BUFFERED "files"
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*) FNAME           !  logical file name
        CHARACTER*(*) VNAME           !  variable name, or 'ALL'
        CHARACTER*(*) CALLER          !  name of caller
        INTEGER       JDATE           !  date, formatted YYYYDDD
        INTEGER       JTIME           !  time, formatted HHMMSS
        INTEGER       RSIZE           !  record dim, words, for error-check
        REAL          BUFFER( RSIZE ) !  interpolation-output buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER       BUFVGT3         !  alloc/read for circ. bufs. (C)
        INTEGER       BUFINT3         !  interp from    circ. bufs. (C)
        LOGICAL       CURRSTEP        !  finds start of timestep
        LOGICAL       OPEN3           !  opens M3 files
        INTEGER       INDEX1          !  name-table lookup
        INTEGER       INIT3           !  start up M3 I/O API
        INTEGER       JSTEP3          !  compute time step record numbers
        INTEGER       SECSDIFF        !  computes date&time differences
        INTEGER       TIME2SEC        !  converts time to seconds
        INTEGER       TRIMLEN         !  effective char. string length

        EXTERNAL  BUFVGT3, BUFINT3, CURRSTEP, OPEN3, INIT3, INDEX1,
     &            JSTEP3, SECSDIFF, TIME2SEC, TRIMLEN


C...........   STATE VARIABLES:  circular buffer structures used for
C...........   the interpolation.  Such variables must be SAVEd in order
C...........   to retain their values from invocation to invocation.

        LOGICAL       FIRSTIME
        DATA          FIRSTIME        / .TRUE. /

        CHARACTER*16  TYPNAMES( -3 : 5 )
        DATA          TYPNAMES
     &      / 'UNKNOWN',        !  -3:  file type error
     &        'DGRAPH3',        !  -2:  known file types
     &        'CUSTOM3',        !  -1:    "
     &        'DCTNRY3',        !   0:    "
     &        'GRDDED3',        !   1:    "
     &        'BNDARY3',        !   2:    "
     &        'IDDATA3',        !   3:    "
     &        'PROFIL3',        !   4:    "
     &        'GRNEST3' /       !   5:    "

        SAVE            FIRSTIME, TYPNAMES


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER       IDUM            !  scratch variable
        INTEGER       FID             !  file subscript for STATE3 arrays
        INTEGER       VID             !  variable subscript for STATE3 arrays
        INTEGER       TDIM            !  subscript for time step
        INTEGER       STEP            !  time step record number
        INTEGER       DT              !  seconds in TSTEP3
        INTEGER       MDATE, MTIME    !  date:time args for CURRSTEP
        INTEGER       DTJL            !  difference LDATE:LTIME to JDATE:JTIME
        INTEGER       DTNJ            !  difference JDATE:JTIME to NDATE:NTIME
        REAL          P, Q            !  fractions used for interpolation.
        INTEGER       DELTA           !  d(INDX) / d(NCVGTcall)
        INTEGER       DIMS( 5 )       !  hyperslab corner   for NCVGT()
        INTEGER       DELS( 5 )       !  hyperslab diagonal for NCVGT()
        INTEGER       DIMT( 5 )       !  hyperslab corner   for NCVGT()
        INTEGER       DELT( 5 )       !  hyperslab diagonal for NCVGT()
        INTEGER       FLAGS( 2,2 )    !  timestep-valid flags
        INTEGER       DATE1, DATE2    !  for checking timestep-valid flags
        INTEGER       TIME1, TIME2    !  "
        INTEGER       IERR            !  error flag for NCVGT call
        LOGICAL       RFLAG           !  BUFINT3() read-data flag
        LOGICAL       EFLAG           !  error flag
        CHARACTER*16  FIL16           !  scratch buffer for file-name
        CHARACTER*16  VAR16           !  scratch buffer for vble-name
        CHARACTER*24  NAMBUF          !  buffer for names (CRAY-PORTING HACK)
        CHARACTER*256 MESG            !  buffer for building error messages

!DEC$ IF DEFINED (_WIN32)
!DEC$ ELSE
#ifdef IOAPICPL
        logical interp3v
        external interp3v
#endif
!DEC$ ENDIF


C***********************************************************************
C   begin body of function  INTERP3

!$OMP CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            IDUM = INIT3()
        END IF
!$OMP END CRITICAL( S_LOGDEV )

           
C...........   Check length of name arguments; copy into length=16 buffers
            
        IF ( LEN( FNAME ) .GT. 16 ) THEN
            NAMBUF = CALLER // ':INTERP3'
            MESG = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max file name length 16; actual:', LEN( FNAME )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            INTERP3 = .FALSE.
            RETURN
        ELSE IF ( LEN( VNAME ) .GT. 16 ) THEN
            NAMBUF = CALLER // ':INTERP3'
            MESG = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max vble name length 16; actual:', LEN( VNAME )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            INTERP3 = .FALSE.
            RETURN
        END IF          !  if len( fname ) > 16, or if len( vname ) > 16

        FIL16 = FNAME
        VAR16 = VNAME
        

C.......   Find netCDF index for the file, and open it if necessary

!$OMP CRITICAL( S_NC )

        FID = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN  !  file needs opening

            NAMBUF = CALLER // ':INTERP3'
            IF ( .NOT.OPEN3( FIL16, FSREAD3, NAMBUF ) ) THEN

                CALL M3WARN( NAMBUF, JDATE, JTIME,
     &              'Could not open ' // FIL16( 1:TRIMLEN( FIL16 ) ) )

                INTERP3 = .FALSE.
                RETURN

            END IF      !  if open3() failed

            FID = INDEX1( FIL16, COUNT3, FLIST3 )

            IF ( FID .EQ. 0 ) THEN      !  lookup failed

                NAMBUF = CALLER // ':INTERP3'
                CALL M3WARN( NAMBUF, JDATE, JTIME,
     &              'Error accessing ' // FIL16( 1:TRIMLEN( FIL16 ) ) )

                INTERP3 = .FALSE.
                RETURN

            END IF      !  if file error

        END IF          !  if file needs opening

!$OMP END CRITICAL( S_NC )


        IDUM = FTYPE3( FID )
        IF ( IDUM .EQ. CUSTOM3 ) THEN

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS3( FID )

            DIMS( 2 ) = 1
            DELS( 2 ) = NLAYS3( FID )

            TDIM  = 3

        ELSE IF ( IDUM .EQ. GRDDED3 ) THEN

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS3( FID )

            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS3( FID )

            DIMS( 3 ) = 1
            DELS( 3 ) = NLAYS3( FID )

            TDIM  = 4

        ELSE IF ( IDUM .EQ. BNDARY3 ) THEN

            DELTA = 2 * NTHIK3( FID )
            DELTA = ABS( DELTA ) * ( NCOLS3( FID ) 
     &                             + NROWS3( FID ) + DELTA )

            DIMS( 1 ) = 1
            DELS( 1 ) = DELTA

            DIMS( 2 ) = 1
            DELS( 2 ) = NLAYS3( FID )

            TDIM  = 3

        ELSE        !  bad file type for interp3()

            IF ( IDUM .LT. -2   .OR.  IDUM .GT. 5 ) IDUM = -3
            NAMBUF = CALLER // ':INTERP3'
            CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Can not interpolate:  file' //
     &                  FIL16( 1:TRIMLEN( FIL16 ) ) // ' of type ' //
     &                  TYPNAMES( IDUM ) )

            INTERP3 = .FALSE.
            RETURN

        END IF              !  check file types; set DIMS, DELS, TDIM

        IF ( RSIZE .NE. NLAYS3( FID ) * BSIZE3( FID ) ) THEN

            WRITE( MESG,94010 )
     &      'Size error for ' //VAR16( 1:TRIMLEN( VAR16 ) )//
     &      ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &      '--REQ:', RSIZE, ' ACT:', NLAYS3( FID ) * BSIZE3( FID )

            NAMBUF = CALLER // ':INTERP3'
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )

            INTERP3 = .FALSE.
            RETURN

        END IF          !  requested record-size in error


C.......   Find index for variable, and if necessary allocate circular
C.......   buffers for interpolation, and initialize related data structures.

        VID = INDEX1( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )

        IF ( VID .EQ. 0 ) THEN  !  variable not available in that file

            NAMBUF = CALLER // ':INTERP3'
            CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Variable ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' not in file ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
            INTERP3 = .FALSE.
            RETURN

        ELSE IF ( VTYPE3( VID,FID ) .NE. M3REAL ) THEN  !  variable of wrong type

            NAMBUF = CALLER // ':INTERP3'
            CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                   'Variable ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                   ' not of type REAL in file ' // 
     &                   FIL16( 1:TRIMLEN( FIL16 ) ) )
            INTERP3 = .FALSE.
            RETURN

        END IF    !  if variable not available; else if not already set up

!DEC$ IF DEFINED (_WIN32)
!DEC$ ELSE
#ifdef IOAPICPL
        IF ( CDFID3( FID ) .EQ. VIRFIL3 ) THEN !  virtual file
           INTERP3 = INTERP3V( FID, VID, JDATE, JTIME, BUFFER )
           RETURN
        END IF
#endif
!DEC$ ENDIF

C.......   Interpolate/copy to JDATE:JTIME

        IF ( TSTEP3( FID ) .EQ. 0 ) THEN        !  time-independent file

            RFLAG =  ( LDATE3( VID,FID ) .NE. 0 )
            IF ( RFLAG .AND. ( CDFID3( FID ) .LT. 0 ) ) THEN
                MESG = 'Variable "' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                   '" not yet written to BUFFERED file "' //
     &                   FIL16( 1:TRIMLEN( FIL16 ) ) // '".'
                NAMBUF = CALLER // ':INTERP3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                INTERP3 = .FALSE.
                RETURN
            END IF
            
            P     =  1.0
            Q     =  0.0
            DELTA =  0
            DATE1 =  0
            TIME1 =  0
            DIMS( TDIM ) = 1
            DELS( TDIM ) = 1
            ILAST3( VID,FID ) = 0
            LDATE3( VID,FID ) = 0
            LTIME3( VID,FID ) = 0
            NDATE3( VID,FID ) = 0
            NTIME3( VID,FID ) = 0

        ELSE IF ( CDFID3( FID ) .LT. 0 ) THEN	!  "buffered" virtual file
            
            RFLAG = .FALSE.
            
            IF( LDATE3( VID,FID ) .LT. 0 ) THEN

                IF ( ( NDATE3( VID,FID ) .EQ. JDATE ) .AND.
     &               ( NTIME3( VID,FID ) .EQ. JTIME )    ) THEN

                     Q = 1.0
                     P = 0.0

                ELSE

                    MESG = 'Variable "' // 
     &                       VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                       '" not yet written to BUFFERED file "' //
     &                       FIL16( 1:TRIMLEN( FIL16 ) ) // '".'
                    NAMBUF = CALLER // ':INTERP3'
                    CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                    INTERP3 = .FALSE.
                    RETURN

                END IF

            ELSE

                DT   = TIME2SEC( TSTEP3( FID ) )
                DTJL = SECSDIFF( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                           JDATE,             JTIME )

                IF ( DTJL .LT. 0 ) THEN	!  ldate:ltime not OK for interp

                    NAMBUF = CALLER // ':INTERP3'
                    WRITE( MESG, 94030 )
     &              JDATE, JTIME,
     &              ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &              ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &              ' (',
     &              SDATE3( FID ), STIME3( FID ),
     &              ' by', TSTEP3( FID ), ')'
                    CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                    INTERP3 = .FALSE.
                    RETURN

                ELSE IF ( DTJL .GT. 0 ) THEN !  interp between ld:lt and nd:nt

                    DTNJ = SECSDIFF( JDATE,             JTIME,
     &                        NDATE3( VID,FID ), NTIME3( VID,FID ) )
                    IF ( ( DTNJ .LT. 0 ) .OR. DTNJ .GT. DT ) THEN
                        NAMBUF = CALLER // ':INTERP3'
                        WRITE( MESG, 94030 )
     &                  JDATE, JTIME,
     &                  ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &                  ' (',
     &                  SDATE3( FID ), STIME3( FID ),
     &                  ' by', TSTEP3( FID ), ')'
                        CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                        INTERP3 = .FALSE.
                        RETURN

                    END IF	!  if ndate:ntime not OK for interp

                END IF	!  if dtjl < 0; else if dtjl > 0 (just copy if dtjl=0)

                Q    = FLOAT( DTJL ) / FLOAT( DT )  !  Interpolation coefficients
                P    = 1.0 - Q                      ! = float(dt - dtjl)/float( dt )

            END IF

        ELSE            !  time-stepped file.

C...........   Check to see relationship between currently circular buffer
C...........   start and ending times, and requested time.  Two cases
C...........   require updates:  (1) need new data at both ends; and
C...........   (2) need new data at the futureward end only. (Needing
C...........   data at the pastward end is still treated under (1).)

            DT = TIME2SEC( TSTEP3( FID ) )

            IF ( LDATE3( VID,FID ) .GT. 0 ) THEN
                DTJL = SECSDIFF( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                           JDATE,             JTIME )
            ELSE                !  ldate set to "invalid"
                DTJL = -1       !  dtjl "invalid"
            END IF

            IF ( ( DTJL .LT. 0 ) .OR. ( DTJL .GT. 2*DT ) ) THEN ! read both

                IF ( .NOT. CURRSTEP( JDATE, JTIME,
     &                               SDATE3( FID ), STIME3( FID ),
     &                               TSTEP3( FID ),
     &                               MDATE, MTIME ) ) THEN

                    NAMBUF = CALLER // ':INTERP3'
                    WRITE( MESG, 94030 )
     &                  JDATE, JTIME,
     &                  ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &                  ' (',
     &                  SDATE3( FID ), STIME3( FID ),
     &                  ' by', TSTEP3( FID ), ')'
                    CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                    INTERP3 = .FALSE.
                    RETURN

                END IF          !  if requested date&time before start of file

                RFLAG = .TRUE.
                DELTA =  0                      !  read starts at 2
                ILAST3( VID, FID ) = 0          !  buffer starts at 0
                LDATE3( VID, FID ) = MDATE
                LTIME3( VID, FID ) = MTIME
                DELS( TDIM ) = 2                !  read BOTH time steps
                DATE1 = MDATE
                DATE2 = MDATE
                TIME1 = MTIME
                TIME2 = MTIME
                CALL NEXTIME( DATE2, TIME2, TSTEP3( FID ) )
                NDATE3( VID,FID ) = DATE2
                NTIME3( VID,FID ) = TIME2
                
                STEP = JSTEP3( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                         SDATE3(    FID ),  STIME3(    FID ),
     &                         TSTEP3(    FID ) )

                DTJL = SECSDIFF( LDATE3( VID,FID ),
     &                           LTIME3( VID,FID ),
     &                           JDATE, JTIME )

                IF ( TSTEP3( FID ) .LT. 0 ) THEN
                    DIMS( TDIM ) = 1 + MOD( STEP-1, 2 )
                ELSE
                    DIMS( TDIM ) = STEP
                END IF

            ELSE IF ( DTJL .GT. DT ) THEN      !  advance by one time step

                CALL NEXTIME( LDATE3( VID, FID ),
     &                        LTIME3( VID, FID ),
     &                        TSTEP3( FID ) )
                
                NDATE3( VID,FID ) = LDATE3( VID, FID )
                NTIME3( VID,FID ) = LTIME3( VID, FID )
                CALL NEXTIME( NDATE3( VID,FID ), 
     &                        NTIME3( VID,FID ), 
     &                        TSTEP3( FID ) )

                DELTA = ILAST3( VID,FID )        !  overwrite ILAST entry
                ILAST3( VID,FID ) = 1 - DELTA    !  new ILAST after read:
                                                !  formula swaps 0,1
                DTJL  = DTJL - DT
                RFLAG = .TRUE.
                DELS( TDIM ) = 1                      !  read one time
                STEP = 1 + JSTEP3( LDATE3( VID,FID ),   !  step for the
     &                             LTIME3( VID,FID ),   !  record
     &                             SDATE3( FID ),      !  one step
     &                             STIME3( FID ),      !  BEYOND
     &                             TSTEP3( FID ) )     !  ldate:ltime
                    
                IF ( TSTEP3( FID ) .LT. 0 ) THEN
                    DIMS( TDIM ) = 1 + MOD( STEP-1, 2 )
                ELSE
                    DIMS( TDIM ) = STEP
                END IF

            ELSE        !  do not need to read this variable from this file:

                RFLAG = .FALSE.

            END IF      !  if circular-buffer structure needed updating

            Q    = FLOAT( DTJL ) / FLOAT( DT )  !  Interpolation coefficients
            P    = 1.0 - Q                      ! = float(dt - dtjl)/float( dt )

        END IF          ! time stepped file or not


C..........   If read will be necessary, check timestep flags first:

        IF ( RFLAG ) THEN       !  check record-available flag

            IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN

                MESG = 'BUFFERED virtual-file not yet written to ' 
     &                   // FLIST3( FID )
                NAMBUF = CALLER // ':INTERP3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                INTERP3 = .FALSE.
                RETURN

            END IF

            DIMT( 1 ) = 1               !  field:  date or time
            DELT( 1 ) = 2               !  extent:  both date and time
            DIMT( 2 ) = VID             !  initial variable-number
            DELT( 2 ) = 1               !  extent:  1 variable
            DIMT( 3 ) = DIMS( TDIM )    !  initial time step number
            DELT( 3 ) = DELS( TDIM )    !  time step extent

	    EFLAG = .FALSE.
            
!$OMP       CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ),
     &                  TINDX3( FID ),
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP       END CRITICAL( S_NC )

            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written

                NAMBUF = CALLER // ':INTERP3'
                CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                       'Time step not yet written for ' //
     &                       VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                       ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                EFLAG   = .TRUE.
                GO TO 900

            ELSE IF ( IERR .NE. 0 ) THEN

                WRITE( MESG,94020 )
     &                      'netCDF error', IERR,
     &                      'reading TFLAG for '
     &                      // FIL16( 1:TRIMLEN( FIL16 ) )
                NAMBUF = CALLER // ':INTERP3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                EFLAG   = .TRUE.
                GO TO 900

C...........   Check validity of timestep flag(s)

            ELSE IF ( DELS( TDIM ) .EQ. 1 ) THEN        ! request 1 timestep

                IF ( FLAGS( 1,1 ) .NE. NDATE3( VID,FID )   .OR.
     &               FLAGS( 2,1 ) .NE. NTIME3( VID,FID ) ) THEN

                    NAMBUF = CALLER // ':INTERP3'
                    CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Data not available for ' //
     &                  VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                    EFLAG   = .TRUE.
                    GO TO 900

                END IF

            ELSE        !  dims( tdim ) = 2:  request both time-steps

                IF ( FLAGS( 1,1 ) .NE. DATE1  .OR.
     &               FLAGS( 2,1 ) .NE. TIME1  .OR.
     &               FLAGS( 1,2 ) .NE. DATE2  .OR.
     &               FLAGS( 2,2 ) .NE. TIME2 ) THEN

                    NAMBUF = CALLER // ':INTERP3'
                    CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Time step not available for ' //
     &                  VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                    EFLAG   = .TRUE.
                    GO TO 900

                END IF      !  if this timestep flag bad

            END IF          !  if bad ierr for NCVGT(), or if bad timestep
            
            IF ( 0 .EQ. BUFVGT3( FID, VID, RFLAG, DIMS, DELS,
     &                           NLAYS3( FID ) * BSIZE3( FID ), 
     &                           DELTA, TSTEP3( FID ) ) ) THEN

                WRITE( MESG,94040 ) 'Timesteps:  LAST',  
     &                         LDATE3( VID,FID ), LTIME3( VID,FID ),
     &              '; NEXT',  NDATE3( VID,FID ), NTIME3( VID,FID )
                CALL M3MSG2( MESG )
                MESG = 'Error reading ' // VAR16( 1:TRIMLEN( VAR16 ) )
     &                      // ' from ' // FIL16( 1:TRIMLEN( FIL16 ) )
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                EFLAG   = .TRUE.
                
            END IF              !  if bufvgt() fails

900         CONTINUE

            IF ( EFLAG ) THEN   !  return FALSE if error in critical section

                INTERP3 = .FALSE.
                RETURN

            END IF
        
        ELSE IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN
                
            IF    ( ( Q .EQ. 0.0 .AND. LDATE3( VID,FID ) .LT. 0 ) .OR.  
     &              ( P .EQ. 0.0 .AND. NDATE3( VID,FID ) .LT. 0 ) ) THEN

                NAMBUF = CALLER( 1:TRIMLEN( CALLER ) ) // ':INTERP3'
                WRITE( MESG,94040 ) 
     &              'Time step not available for ' //
     &              VAR16( 1:TRIMLEN( VAR16 ) ) //
     &              ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) // 
     &              '--LAST',  LDATE3( VID,FID ), LTIME3( VID,FID ),
     &              '; NEXT',  NDATE3( VID,FID ), NTIME3( VID,FID )
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
     &                       
            END IF            
            
        END IF          !  if RFLAG:  read will be necessary; else if BUFFERED


C...........   Call BUFINT3 to interpolate; check its success/failure
C...........   status and return accordingly:

        IF ( 0 .NE. BUFINT3( FID, VID, NLAYS3( FID ) * BSIZE3( FID ), 
     &                       ILAST3( VID,FID ), TSTEP3( FID ),
     &                       P, Q, BUFFER ) ) THEN

            INTERP3 = .TRUE.

        ELSE

            INTERP3 = .FALSE.

            NAMBUF = CALLER // ':INTERP3'
            MESG   =  'Could not read ' // VAR16( 1:TRIMLEN( VAR16 ) )
     &                 // ' from file ' // FIL16( 1:TRIMLEN( FIL16 ) )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )

            IF( CDFID3( FID ) .GE. 0 ) THEN	!  not a buffered virtual file
                LDATE3( VID, FID ) = -1         !  "invalid"
            END IF

        END IF          !  if bufint3() succeeded, or failed

        RETURN          !  from INTERP3

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( A , I5 )

C...........   Internal buffering formats............ 94xxx

94010   FORMAT( A, I7, A, I7 )

94020   FORMAT( A, I4, 2X, A )

94030   FORMAT( I7.7, ':', I6.6, A, I7.7, ':', I6.6, A, I8 , A )

94040   FORMAT( A, :, I9.7, ':', I6.6 )


        END             !  logical function interp3

