
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

      LOGICAL FUNCTION RDBUF3( FID, VID, LAYER, JDATE, JTIME, BUFFER )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: RDBUF3
!DEC$ ENDIF
C***********************************************************************
C  function body starts at line  63
C
C  FUNCTION:  reads data from Models-3 BUFFEREd "file" with M3 file
C       index FID for variable with name VNAME and layer LAYER, 
C       for the date and time JDATE (coded YYYYDDD) JTIME (HHMMSS).
C       For time-independent files, JDATE:JTIME are ignored.
C       If VNAME is 'ALL', reads all variables; if LAYER is -1,
C       reads all layers.
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  FNAME is a Models-3 BUFFERED "file" already 
C       opened by CREATE3().  Should only be called via READ3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       JSTEP3, BUFGET3
C
C  REVISION  HISTORY:  
C       Prototype 7/94 by CJC
C	Modified 10/94 by CJC to work with WRITE3() having granularity
C	at the level of individual variables.
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'

C...........   ARGUMENTS and their descriptions:

        INTEGER         FID             !  subscript for STATE3 arrays
        INTEGER         VID             !  subscript for STATE3 arrays
        INTEGER         LAYER           !  layer number, or 0
        INTEGER         JDATE           !  date, formatted YYYYDDD
        INTEGER         JTIME           !  time, formatted HHMMSS
        REAL            BUFFER(*)       !  input buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         JSTEP3     !  compute time step record numbers
        INTEGER         BUFGET3    !  from bufint3.c

        EXTERNAL        JSTEP3, BUFGET3


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         STEP            !  time step record number
        INTEGER         VAR             !  loop counter:  variables


C***********************************************************************
C   begin body of function  RDBUF3


        IF ( VID .GT. 0 ) THEN          !  read just this variable
        
            IF ( TSTEP3( FID ) .EQ. 0 ) THEN
                   
                IF( LDATE3( VID, FID ) .EQ. 0 ) THEN
                    STEP = ILAST3( VID,FID )
                ELSE
                    RDBUF3 = .FALSE.
                    RETURN
                END IF
                
            ELSE IF ( JDATE .EQ. LDATE3( VID,FID ) .AND. 
     &                JTIME .EQ. LTIME3( VID,FID ) ) THEN
                
                STEP = ILAST3( VID,FID )
                
            ELSE IF ( JDATE .EQ. NDATE3( VID,FID ) .AND. 
     &                JTIME .EQ. NTIME3( VID,FID ) ) THEN
                
                STEP = 1 - ILAST3( VID,FID )        !  formula swaps 0 and 1
                
            ELSE
                
                CALL M3WARN( 'READ3/RDBUF3', JDATE, JTIME,                     
     &              'Date and time not available in '// FLIST3( FID ) )
                
                RDBUF3 = .FALSE.
                RETURN
                
            END IF
                
            RDBUF3 = ( 0 .NE. BUFGET3( FID, VID, LAYER, NLAYS3( FID ), 
     &                                 BSIZE3( FID ), STEP,
     &                                 BUFFER ) )
                
        ELSE                            !  read all variables
        
            DO  11  VAR = 1, NVARS3( FID )
                
                IF ( TSTEP3( FID ) .EQ. 0 ) THEN
                       
                    IF( LDATE3( VAR, FID ) .EQ. 0 ) THEN
                        STEP = ILAST3( VAR,FID )
                    ELSE
                        RDBUF3 = .FALSE.
                        RETURN
                    END IF
                    
                ELSE IF ( JDATE .EQ. LDATE3( VAR,FID ) .AND. 
     &                    JTIME .EQ. LTIME3( VAR,FID ) ) THEN
                    
                    STEP = ILAST3( VAR,FID )
                    
                ELSE IF ( JDATE .EQ. NDATE3( VAR,FID ) .AND. 
     &                    JTIME .EQ. NTIME3( VAR,FID ) ) THEN
                    
                    STEP = 1 - ILAST3( VAR,FID )        !  formula swaps 0 and 1
                    
                ELSE
                    
                    CALL M3WARN( 'READ3/RDBUF3', JDATE, JTIME,                     
     &              'Date and time not available in '// FLIST3( FID ) )
                                        
                    RDBUF3 = .FALSE.
                    RETURN
                    
                END IF
                    
                IF ( 0 .EQ. BUFGET3( FID, VAR, LAYER, NLAYS3( FID ), 
     &                               BSIZE3( FID ), STEP,
     &                               BUFFER ) ) THEN
                    
                    RDBUF3 = .FALSE.
                    RETURN
                
                END IF
            
11          CONTINUE
            
            RDBUF3 = .TRUE.             !  (if you get to here)
                
        END IF                          !  read one vble, or read all vbles
        
        RETURN
        END

