
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................


C Version "%W% %P% %G% %U%

        PROGRAM UTMTOOL

C***********************************************************************
C  program body starts at line  97
C
C  DESCRIPTION:
C       Perform coordinate conversions and grid-related computations
C	for Lat-Lon, Lambert, and UTM coordinate systems.
C
C  PRECONDITIONS REQUIRED:
C	"setenv GRIDDESC <pathname>" for using Lambert projections by name.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C	GETMENU, GETNUM, GETREAL, GETSTR, GETYN, LAMBERT
C
C  REVISION  HISTORY:
C	Augmented 6/11/96 by CJC to support Lambert operations.
C
C***********************************************************************

      IMPLICIT NONE


C...........   INCLUDES:

      INCLUDE 'PARMS3.EXT'      ! I/O API constants
      INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
      INCLUDE 'IODECL3.EXT'     ! I/O API function declarations


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         GETMENU
        INTEGER         GETNUM
        REAL            GETREAL
        LOGICAL         GETYN
        LOGICAL         LAMBERT, SETLAM
        LOGICAL         LL2LAM, LAM2LL, UTM2LAM, LAM2UTM

        EXTERNAL        GETMENU, GETNUM, GETREAL, GETYN, LAMBERT,
     &                  SETLAM, LL2LAM, LAM2LL, UTM2LAM, LAM2UTM 


C...........   LOCAL VARIABLES and their descriptions:

        INTEGER         MODE
        INTEGER         ZONE
        INTEGER         NCOL, NROW
        REAL            LAT, LON
        REAL            XX, YY
        REAL            CELL
        REAL            XLL, XUR
        REAL            YLL, YUR
        REAL            A, B, C, X, Y
        DATA            A /  30.0 /
        DATA            B /  60.0 /
        DATA            C / -90.0 /
        DATA            X / -90.0 /
        DATA            Y /  40.0 /

        CHARACTER*16    LAMNAME
        DATA            LAMNAME / CMISS3 /

        CHARACTER*60    MENU( 14 )
        DATA            MENU 
     &      / 'Convert from LAT-LON to UTM',                    !  mode = 1
     &        'Convert from UTM to LAT-LON',                    !  mode = 2
     &        'Convert from UAM/EPS to LAT-LON',                !  mode = 3
     &        'Initialize new LAMBERT projection by name',      !  mode = 4
     &        'Define a new LAMBERT projection by angles',      !  mode = 5
     &        'Convert from LAMBERT to UTM',                    !  mode = 6
     &        'Convert from UTM to LAMBERT',                    !  mode = 7
     &        'Convert from LAMBERT to LAT-LON',                !  mode = 8
     &        'Convert from LAT-LON to LAMBERT',                !  mode = 9
     &        'Get LAT-LON grid corners from UTM specs',        !  mode =10
     &        'Get UTM grid corners from LAT-LON specs',        !  mode =11
     &        'Get LAT-LON grid corners from LAMBERT specs',    !  mode =12
     &        'Get LAMBERT grid corners from LAT-LON specs',    !  mode =13
     &        'Quit the program' /                              !  mode =14

        CHARACTER*60    PROMPT
        DATA            PROMPT / 'What do you want to do next?' /

        LOGICAL         LAMSET
        DATA            LAMSET / .FALSE. /


C***********************************************************************
C   begin body of program UTMTOOL

        WRITE( *,92000 ) ' ', ' ',
     &  'Program UTMTOOL to provide coordinate conversion back and',
     &  'forth among LAT-LON, UTM, and LAMBERT coordinate systems.',
     &  'Note that according to the standard, UTM coordinates should',
     &  'be specified in _meters_ instead of the UAM/EPS bastardized ',
     &  'system which claims to be UTM but in fact uses *kilo*meters;', 
     &  'the latter is a distinct (also-supported) system. ',
     &  ' ',
     &  'Longitudes are specified in _signed_degrees_ (so that for',
     &  'the US longitudes are negative); coordinate system "UAM/EPS"',
     &  'means the thing with the *kilometer* units instead of the',
     &  'UTM standard units, which are meters.',
     &  ' ',
     &  'Grid-corner calculations assume either a UTM-based or a',
     &  'Lambert-based grid, as appropriate (to get UTM or Lambert ',
     &  'corners of a LAT-LON based grid, work one corner at a time',
     &  'using the coordinate conversion facilities). ',
     &  ' '

        IF ( .NOT. GETYN( 'Continue with program?', .TRUE. ) )
     &      CALL M3EXIT( 'UTMTOOL', 0, 0, 'Exit at user request', 0 )

        MODE =    1
        ZONE =   17
        NCOL =   90
        NROW =   60
        LAT  =   30.0
        LON  =  -90.0
        XX   =  350.0E3
        YY   = 3785.0E3
        CELL =    5.0E3

111     CONTINUE

            MODE = GETMENU( 14, MODE, PROMPT, MENU )


            IF ( MODE .EQ. 1 ) THEN             !  convert from LL


                LAT = GETREAL( -90.0, 90.0, LAT,
     &                         'Enter input LATITUDE  (degrees)' )

                LON = GETREAL( -180.0, 180.0, LON,
     &                         'Enter input LONGITUDE (degrees)' )

                ZONE = INT( ( 180.0 + LON ) / 6.0 ) + 1
                ZONE = GETNUM( 1, 60, ZONE,
     &                         'Enter         UTM ZONE (1...60)' )

                CALL LL2UTM( LON, LAT, ZONE, XX, YY )

                WRITE( *,92010 )
     &              'Location LAT=', LAT, ' : LON=', LON, ' (deg)',  
     &              '           X=', XX,  ' :   Y=', YY, ' (meters)'


            ELSE IF ( MODE .EQ. 2 ) THEN        ! convert from UTM


                XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                        'Enter input X  (meters)' )

                YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                        'Enter input Y  (meters)' )

                ZONE = GETNUM( 1, 60, ZONE,
     &                        'Enter UTM ZONE (1...60)' )

                CALL UTM2LL( XX, YY, ZONE, LON, LAT )

                WRITE( *,92010 )
     &              'Location X=', XX,  ' :   Y=', YY,  ' (meters)',
     &              '       LON=', LON, ' : LAT=', LAT, ' (degrees)'


            ELSE IF ( MODE .EQ. 3 ) THEN        ! convert from EPS


                XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                        'Enter input     X  (km)' )

                YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                        'Enter input     Y  (km)' )

                ZONE = GETNUM( 1, 60, ZONE,
     &                        'Enter UTM ZONE (1...60)' )

                CALL UTM2LL( 1000.0 * XX, 1000 * YY, ZONE, LON, LAT )

                WRITE( *,92010 )
     &              'Location X=', XX,  ' :   Y=', YY,  ' (km)',
     &              '       LON=', LON, ' : LAT=', LAT, ' (degrees)'


            ELSE IF ( MODE .EQ. 4 ) THEN        ! new Lambert by name


                CALL GETSTR( 'Enter Lambert projection name',
     &                       '<dummy>',
     &                       LAMNAME )

                LAMSET = LAMBERT( LAMNAME, A, B, C, X, Y )

                IF ( .NOT. LAMSET ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0, 
     &              'Error getting projection; please try again' )
                END IF


            ELSE IF ( MODE .EQ. 5 ) THEN        ! new Lambert by angles


                WRITE( *,92000 )
     &          ' ',
     &          'Lambert projection coordinate systems are defined',
     &          'by two secant (or tangent) latitudes ALPHA <= BETA',
     &          'a central meridian GAMMA, and the longitude XCENT',
     &          'and latitude YCENT of the coordinate origin (where',
     &          'Cartesian coordinates X=Y=0).',
     &          ' '
                A = GETREAL( -90.0, 90.0, A,
     &                        'Enter first  secant latitude ALPHA' )

                B = GETREAL(  -90.0, 90.0, MAX( A,B ),
     &                        'Enter second secant latitude  BETA' )

                C = GETREAL( -180.0, 180.0, C,
     &                        'Enter central longitude      GAMMA' )

                X = GETREAL( -180.0, 180.0, C,
     &                        'Enter origin longitude       XCENT' )


                IF ( Y .LT. A  .OR.  Y .GT. B ) THEN
                    Y = 0.5 * ( A + B )
                END IF

                Y = GETREAL( -90.0, 90.0, Y,
     &                        'Enter origin  latitude       YCENT' )

                LAMSET = SETLAM( A, B, C, X, Y )

                IF ( .NOT. LAMSET ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0, 
     &              'Error getting projection; please try again' )
                END IF


            ELSE IF ( MODE .EQ. 6 ) THEN        ! LAMBERT to UTM


                IF ( LAMSET ) THEN

                    XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                            'Enter input Lambert X  (meters)' )

                    YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                            'Enter input Lambert Y  (meters)' )

                    ZONE = GETNUM( 1, 60, ZONE,
     &                        'Enter output UTM ZONE (1...60)' )

                    IF ( LAM2UTM( XX, YY, ZONE, X, Y ) ) THEN

                        WRITE( *,92010 )
     &                  'Lambert X=', XX, ' :  Y=', YY,  ' (meters)',
     &                  '    UTM X=',  X, ' :  Y=',  Y,  ' (meters)'

                    ELSE

                        CALL M3WARN( 'UTMTOOL', 0, 0,
     &                  'Error converting to UTM; please try again' )

                    END IF      !  if lam2utm succeeded, or not

                ELSE

                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &       'Lambert projection not initialized; please try again' )

                END IF          !  if lamset or not


            ELSE IF ( MODE .EQ. 7 ) THEN        ! UTM to LAMBERT


                IF ( LAMSET ) THEN

                    XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                            'Enter input UTM    X  (meters)' )

                    YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                            'Enter input UTM    Y  (meters)' )

                    ZONE = GETNUM( 1, 60, ZONE,
     &                            'Enter input UTM ZONE  (1...60)' )

                    IF ( UTM2LAM( XX, YY, ZONE, X, Y ) ) THEN

                        WRITE( *,92010 )
     &                  '    UTM X=', XX, ' :  Y=', YY,  ' (meters)',
     &                  'Lambert X=',  X, ' :  Y=',  Y,  ' (meters)'

                    ELSE

                        CALL M3WARN( 'UTMTOOL', 0, 0,
     &              'Error converting to LAMBERT; please try again' )

                    END IF      !  if utm2lam succeeded, or not

                ELSE

                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &              'Lambert projection not initialized; ' //
     &              'please try again' )

                END IF          !  if lamset or not


            ELSE IF ( MODE .EQ. 8 ) THEN        ! LAMBERT to LAT-LON


                IF ( LAMSET ) THEN

                    XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                            'Enter input Lambert  X  (meters)' )

                    YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                            'Enter input Lambert  Y  (meters)' )

                    IF ( LAM2LL( XX, YY, X, Y ) ) THEN

                        WRITE( *,92010 )
     &                  'Lambert X=', XX, ' :  Y=', YY,  ' (meters)',
     &                  'LAT-LON X=',  X, ' :  Y=',  Y,  ' (degrees)'

                    ELSE

                        CALL M3WARN( 'UTMTOOL', 0, 0,
     &              'Error converting to LAT-LON; please try again' )

                    END IF      !  if lam2ll succeeded, or not

                ELSE

                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &          'Lambert projection not initialized; please try again')

                END IF          !  if lamset or not


            ELSE IF ( MODE .EQ. 9 ) THEN        ! LAT-LON to LAMBERT


                IF ( LAMSET ) THEN

                    XX = GETREAL( BADVAL3, -BADVAL3, XX,
     &                            'Enter input Longitude X (degrees)' )

                    YY = GETREAL( BADVAL3, -BADVAL3, YY,
     &                            'Enter input Latitude  Y (degrees)' )


                    IF ( LL2LAM( XX, YY, X, Y ) ) THEN

                        WRITE( *,92010 )
     &                  'LAT-LON X=', XX, ' :  Y=', YY,  ' (degrees)',
     &                  'Lambert X=',  X, ' :  Y=',  Y,  ' (meters)'

                    ELSE

                        CALL M3WARN( 'UTMTOOL', 0, 0,
     &              'Error converting to LAMBERT; please try again' )

                    END IF      !  if ll2lam succeeded, or not

                ELSE

                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &          'Lambert projection not initialized; please try again' )

                END IF          !  if lamset or not


            ELSE IF ( MODE .EQ. 10 ) THEN        ! get LL grid corners from UTM


                XLL = GETREAL( BADVAL3, -BADVAL3, XX,
     &                         'Enter lower-left corner X (meters)' )

                YLL = GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter lower-left corner Y  (meters)' )

                CELL= GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter    cell-size DX(=DY) (meters)' )

                ZONE = GETNUM( 1, 60, ZONE,
     &                        'Enter             UTM ZONE (1...60)' )

                NCOL = GETNUM( 1, 999, NCOL,
     &                        'Enter number of        grid COLUMNS' )

                NROW = GETNUM( 1, 999, NROW,
     &                        'Enter number of        grid    ROWS' )

                XUR  = XLL + FLOAT( NCOL ) * CELL
                YUR  = YLL + FLOAT( NROW ) * CELL

                CALL UTM2LL( XLL, YLL, ZONE, LON, LAT )

                WRITE( *,92000 ) ' '
                WRITE( *,92020 )
     &              'LL corner X=', XLL, ' :   Y=', YLL, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL UTM2LL( XLL, YUR, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'LR corner X=', XLL, ' :   Y=', YUR,' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL UTM2LL( XUR, YUR, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'UR corner X=', XUR, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL UTM2LL( XUR, YLL, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'UL corner X=', XLL, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'
                WRITE( *,92000 ) ' '


            ELSE IF ( MODE .EQ. 11 ) THEN        ! get UTM grid corners from LL


                LON = GETREAL( BADVAL3, -BADVAL3, XX,
     &                         'Enter lower-left corner LON (degrees)' )

                LAT = GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter lower-left corner LAT (degrees)' )

                CALL LL2UTM( LON, LAT, ZONE, XLL, YLL )

                CELL= GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter    cell-size DX(=DY) (meters)' )

                ZONE = GETNUM( 1, 60, ZONE,
     &                        'Enter             UTM ZONE (1...60)' )

                NCOL = GETNUM( 1, 999, NCOL,
     &                        'Enter number of        grid COLUMNS' )

                NROW = GETNUM( 1, 999, NROW,
     &                        'Enter number of        grid    ROWS' )

                XUR  = XLL + FLOAT( NCOL ) * CELL
                YUR  = YLL + FLOAT( NROW ) * CELL

                CALL LL2UTM( XLL, YLL, ZONE, LON, LAT )

                WRITE( *,92000 ) ' '
                WRITE( *,92020 )
     &              'LL corner X=', XLL, ' :   Y=', YLL, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL LL2UTM( XLL, YUR, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'LR corner X=', XLL, ' :   Y=', YUR,' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL LL2UTM( XUR, YUR, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'UR corner X=', XUR, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                CALL LL2UTM( XUR, YLL, ZONE, LON, LAT )

                WRITE( *,92020 )
     &              'UL corner X=', XLL, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'
                WRITE( *,92000 ) ' '


            ELSE IF ( MODE .EQ. 12 ) THEN        ! get LL corners from LAMBERT


                IF ( .NOT. LAMSET ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &          'Lambert projection not initialized; please try again' )
                    GO TO 111
                END IF          !  if lamset or not

                XLL = GETREAL( BADVAL3, -BADVAL3, XX,
     &                         'Enter lower-left corner X (meters)' )

                YLL = GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter lower-left corner Y  (meters)' )

                CELL= GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter    cell-size DX(=DY) (meters)' )

                NCOL = GETNUM( 1, 999, NCOL,
     &                        'Enter number of        grid COLUMNS' )

                NROW = GETNUM( 1, 999, NROW,
     &                        'Enter number of        grid    ROWS' )

                XUR  = XLL + FLOAT( NCOL ) * CELL
                YUR  = YLL + FLOAT( NROW ) * CELL

                IF ( .NOT. LAM2LL( XLL, YLL, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92000 ) ' '
                WRITE( *,92020 )
     &              'LL corner X=', XLL, ' :   Y=', YLL, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XLL, YUR, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'LR corner X=', XLL, ' :   Y=', YUR,' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XUR, YUR, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'UR corner X=', XUR, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XUR, YLL, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'UL corner X=', XLL, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'
                WRITE( *,92000 ) ' '


            ELSE IF ( MODE .EQ. 13 ) THEN        ! get Lambert corners from LL


                IF ( .NOT. LAMSET ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &          'Lambert projection not initialized; please try again' )
                    GO TO 111
                END IF          !  if lamset or not

                LON = GETREAL( BADVAL3, -BADVAL3, XX,
     &                         'Enter lower-left corner LON (degrees)' )

                LAT = GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter lower-left corner LAT (degrees)' )

                IF ( .NOT. LL2LAM( LON, LAT, XLL, YLL ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF


                CELL= GETREAL( BADVAL3, -BADVAL3, YY,
     &                         'Enter    cell-size DX(=DY) (meters)' )

                NCOL = GETNUM( 1, 999, NCOL,
     &                        'Enter number of        grid COLUMNS' )

                NROW = GETNUM( 1, 999, NROW,
     &                        'Enter number of        grid    ROWS' )

                XUR  = XLL + FLOAT( NCOL ) * CELL
                YUR  = YLL + FLOAT( NROW ) * CELL

                IF ( .NOT. LAM2LL( XLL, YLL, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92000 ) ' '
                WRITE( *,92020 )
     &              'LL corner X=', XLL, ' :   Y=', YLL, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XLL, YUR, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'LR corner X=', XLL, ' :   Y=', YUR,' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XUR, YUR, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'UR corner X=', XUR, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'

                IF ( .NOT. LAM2LL( XUR, YLL, LON, LAT ) ) THEN
                    CALL M3WARN( 'UTMTOOL', 0, 0,
     &                           'Coordinate transformation error' )
                    GO TO 111
                END IF

                WRITE( *,92020 )
     &              'UL corner X=', XLL, ' :   Y=', YUR, ' (meters)',
     &              '        LON=', LON, ' : LAT=', LAT, ' (degrees)'
                WRITE( *,92000 ) ' '


            ELSE IF ( MODE .EQ. 14 ) THEN    ! if ( mode .eq. 10 ) then quit:

                GO TO 999       !  to program shut-down

            ELSE !  by construction of menu, should not be able to get to here

                CALL M3EXIT( 'UTMTOOL', 0, 0,
     &                       'Program Error:  mishandled menu item', 2 )

            END IF

            GO TO 111   !  to head of loop


999     CONTINUE
        CALL M3EXIT( 'UTMTOOL', 0, 0, 
     &             'Successful completion of program UTMTOOL', 0 )

C******************  FORMAT  STATEMENTS   ******************************

C...........   Informational (LOG) message formats... 92xxx

92000   FORMAT ( 5X , A )

92010   FORMAT ( /5X , A, F15.4, A, F15.4, A, 
     &           /5X , A, F15.4, A, F15.4, A, / )

92020   FORMAT (  5X , A, F15.4, A, F15.4, A,
     &           /5X , A, F15.4, A, F15.4, A )


        END

