
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MM2DP2( IM, JM, KM, 
     &                      TA, QV, PP, PST, HFX, QFX, SIGH, PTOP,
     &                      NCOLS, NROWS,
     &                      COL0, ROW0,
     &                      DENS, PRES, TV, RH, QFLX, HFLX )

C***********************************************************************
C  subroutine body starts at line  114
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	3D density, pressure, virtual temperature, relative humidity,
C       and potential temperature for the window specified in terms of 
C       grid-window bounds to I/O API order, using TA, QV, Pprime, Pstar, 
C       SIGH, and Ptop from MM5.  If NLAYS=1, also rescales fluxes 
C       QFLX, HFLX by density.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  Oct. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for MM5
        INTEGER         JM      !  J (east-west)   dimension for MM5
        INTEGER         KM      !  K (top-bottom)  dimension for MM5
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )        !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )        !  MM5 P-prime
        REAL            PST( IM, JM )           !  MM5 pstar array
        REAL            HFX( IM, JM )           !  MM5 hfx array
        REAL            QFX( IM, JM )           !  MM5 QFX array
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        REAL            PRES( NCOLS, NROWS ) !  pressure (cb)
        REAL            DENS( NCOLS, NROWS ) !  density
        REAL            TV  ( NCOLS, NROWS ) !  virtual temperature
        REAL            RH  ( NCOLS, NROWS ) !  relative humidity
        REAL            QFLX( NCOLS, NROWS ) !  kinematic-units latent heat flux
        REAL            HFLX( NCOLS, NROWS ) !  kinematic-units sensible heat flux

C...........   PARAMETERs

        REAL	    	CBTOPA
        REAL	    	RDGAS		 !  from param.F, line 1573 & ff
	REAL            CP		 !  "
	REAL            CTOK		 !  Celsius-to-Kelvin
	REAL            ROVCP		 ! RDGAS/CP

        PARAMETER     ( CBTOPA = 1.0E3 , !  Conversion: CB to Pa
     &                  RDGAS =  287.04, !  to preserve MM5 compatibility
     &                  CP    = 1004.0 , !  "
     &                  CTOK  = 273.15 ,
     &                  ROVCP = RDGAS / CP )
	       	
        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R    	        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0                  !  MM5 offsets
        
        REAL            DTA, DQV, DPP		!  decoupled TA,QV,PP
        REAL            ES, DIV, DCP, DDD, RHL

C***********************************************************************
C   begin body of subroutine  MM2DPT

        I0  = ROW0 - 1                   !  offsets for IOAPI grid within MM5
        J0  = COL0 - 1
        K   = KM

        DCP = -1.0 / CP

        DO  R = 1, NROWS
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C
                DDD = 1.0 / PST( I,J )
                DTA = DDD * TA( I,J,K )	!  decoupled
                DQV = DDD * QV( I,J,K )	!   "
                DPP = DDD * PP( I,J,K )     !  ", pa
                TV  ( C,R ) = DTA * ( 1.0 + 0.608 * DQV )
                PRES( C,R ) = CBTOPA * ( PST( I,J ) * SIGH( K )
     &                                   + PTOP ) + DPP
                DENS( C,R ) = PRES( C,R )/( RDGAS*TV( C,R ) )
                IF ( DTA .GE. CTOK ) THEN           !  water is liquid
                    ES = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                             / ( DTA - DLIQ ) )
                ELSE                                !  else below freezing
                    ES = AICE * EXP( ( BICE * DTA - CICE )
     &                             / ( DTA - DICE ) )
                END IF                              !  if liquid or frozen
                RHL = F622*DQV*( PRES( C,R ) - ES ) / ES
                RH( C,R ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                DIV = 1.0 / DENS( C,R )
                HFLX( C,R ) = HFX( I,J ) * DCP * DIV
                QFLX( C,R ) = QFX( I,J ) * DIV
            END DO
        END DO

        RETURN
        END

