
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5 cpyright (C) 1998-2002 MCNC and
C Carlie J. Coats, Jr., and (C) 2003 Baron Advanced Meteorological Systems
C This routine copyright (C) 2003 BAMS.
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2HTR2( IM, JM, KM, NEARSFC, TKICE, TKLIQ, TKFRZ,
     &                      TA, QV, QR, QI, PP, PSTAR, RAINC, RAINNC,
     &                      TA2, UA10, VA10, PBL,
     &                      SIGH, PTOP, P00, TLP, TS0, ROVG,
     &                      NCOLS, NROWS, NLAYS,
     &                      COL0,  ROW0,  LAY0,
     &                      TWSFC, TWBAR, LFRAC, QRTOT, QITOT,
     &                      ZWET0, RAIN,  T2, UX10, VX10,
     &                      CFRACL, CFRACM, CFRACH  )

C***********************************************************************
C  subroutine body starts at line  191
C
C  FUNCTION:
C       Compute layer-1 and near-surface-averaged wet bulb temperatures
C       TWSFC and TWBAR, liquid-water-fraction TWBAR such that
C       freezing <= 0 < mixed < 1 <= liquid, 
C       and 3x3-aggregated layer 1 QR and QI, using values of temperature, 
C       precipitation, etc. extracted/reordered/converted/decoupleed/computed
C       from MM5 2-D and 3-D arrays.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype 3/2003 by Carlie J. Coats, Jr., BAMS
C       Version   5/2003 that does wet-bulb calculation
C       Version   7/2003 that calculates Z-value ZWET0 such that
C       TWET = 0.0 C at elevation ZWET0; accumulated total RAIN(cm),
C       2-meter temperature, 10 meter wind components, high-, mid-,
C       and low-level cloud fraction (0 <= CFRAC* <= 1.  Uses RH
C       formulation borrowed from SFCRAD.
C***********************************************************************

      IMPLICIT NONE

#include "parame.incl"
#include "pmoist.incl"

C...........   ARGUMENTS and their descriptions:

        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension for AM
        INTEGER         NEARSFC !  # of layers for averaging
        REAL            TKICE, TKLIQ, TKFRZ
        REAL		TA( IM, JM, KM )	!  MM5-order input array
        REAL		QV( IM, JM, KM )	!  MM5-order input array
        REAL		QR( IM, JM, KM )	!  MM5-order input array
        REAL		QI( IM, JM, KM )	!  MM5-order input array
        REAL		PP( IM, JM, KM )	!  MM5-order input array
        REAL	     PSTAR( IM, JM )		!  MM5-order pstar array
        REAL	     RAINC( IM, JM )		!  MM5-order convective rain
        REAL	    RAINNC( IM, JM )	!  MM5-order nonconvective rain
        REAL	       TA2( IM, JM )	!  MM5-order nonconvective rain
        REAL	      UA10( IM, JM )	!  MM5-order nonconvective rain
        REAL	      VA10( IM, JM )	!  MM5-order nonconvective rain
        REAL	       PBL( IM, JM )	!  MM5-order nonconvective rain
        REAL		SIGH( KM )		!  sigma half-levels
        REAL            PTOP    !  MM5 ptop (cb)
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            TLP		!  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		!  MM5 ref SL temp (K)
        REAL            ROVG		!  MM5 R/G
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
	REAL		TWSFC( NCOLS, NROWS )
	REAL		TWBAR( NCOLS, NROWS )
	REAL		LFRAC( NCOLS, NROWS )
	REAL		QRTOT( NCOLS, NROWS )
	REAL		QITOT( NCOLS, NROWS )
	REAL		ZWET0( NCOLS, NROWS )    !  height(m) @ TWET=0(C)
	REAL		 RAIN( NCOLS, NROWS )    !  total rain
	REAL		   T2( NCOLS, NROWS )    !  2-meter temperature (K)
	REAL	         UX10( NCOLS, NROWS )    !  10-meter cross point U
	REAL             VX10( NCOLS, NROWS )    !  10-meter cross point V
	REAL           CFRACL( NCOLS, NROWS )    !  high-level cloud fraction
	REAL           CFRACM( NCOLS, NROWS )    !  mid-level cloud fraction
	REAL           CFRACH( NCOLS, NROWS )    !  low-level cloud fraction

C...........   PARAMETERs

        REAL            ICE_T, RAIN_T
        REAL	    	CPA             ! conv facs Pa to cb
        REAL	    	CTOK            ! conv facs Pa to cb
        REAL	    	C1
        REAL	    	C15
        REAL	    	C2
        REAL	    	CP
        REAL	    	D611
        REAL	    	E1
        REAL	    	E2
        REAL	    	EPS
        REAL	    	F
        REAL            PRESBOT         !  Pa
        REAL            PRESMID         !  Pa
        REAL            PRESTOP         !  Pa
        INTEGER         MAXC
        PARAMETER     ( ICE_T  =     9.0E-5 ,
     &                  RAIN_T =     9.0E-7 ,
     &                  CPA    =     1.0E3  ,
     &                  CTOK   =   273.15 ,
     &                  C1     =     0.0091379024 ,
     &                  C15    =    26.66082 ,
     &                  C2     =  6106.396 ,
     &                  CP     =  1004.0 ,
     &                  D611   =     1.0 / 6.11 ,
     &                  E1     =  5418.12 ,
     &                  E2     =    19.84659 ,
     &                  EPS    =     0.622 ,
     &                  F      =     0.0006355 ,
     &                  PRESBOT=   970.0E2 ,
     &                  PRESMID=   800.0E2 ,
     &                  PRESTOP=   450.0E2 ,
     &                  MAXC   =   512 )

        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L, N	        !  I/O API col subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        REAL            DLAY, DFRZ
        REAL            RL, RI, DTA, TW, DZ, LP
        REAL            DP0, DPP, DQV, PR, QQ, WW, EE, TD, ES, ED, S
        REAL            EW, DE, DER
	INTEGER		NL, LTOP
        REAL            ZT, EBL, VAP, RHL, RHTOP, RHMID, RHBOT


#ifdef LANGUAGE_FORTRAN90
        REAL		DFAC( NCOLS )
        REAL		TWET( NCOLS )
        REAL		  ZH( NCOLS )
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL		DFAC( MAXC )
        REAL		TWET( MAXC )
        REAL	  	  ZH( MAXC )

        IF ( NCOLS.GT. MAXC .OR. KM .GT. MAXC ) THEN
            CALL M3EXIT( 'MCPL:MM2HTR2', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAXC > KM', 2 )
        END IF
#endif    /* ifndef LANGUAGE_FORTRAN90  */


C***********************************************************************
C   begin body of subroutine  MM2HTR2

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2
        DLAY = 1.0 / FLOAT( NEARSFC )
        DFRZ = 1.0 / ( TKLIQ - TKICE )
        DP0  = 1.0 / P00

!$OMP       PARALLEL DO DEFAULT( NONE ),
!$OMP&                   SHARED( I0, J0, K0, KM, NROWS, NCOLS, NLAYS,
!$OMP&                           PSTAR, TA, QV, QR, QI, PP, PBL, SIGH,
!$OMP&                           PTOP, ROVG, TLP, TS0,
!$OMP&                           EP2, SVP1, SVP2, SVP3, SVPT0,
!$OMP&                           NEARSFC, DLAY, DFRZ, DP0, RAIN, RAINC,
!$OMP&                           RAINNC, UX10, VX10, UA10, VA10,
!$OMP&                           T2, TA2, ZWET0, TKICE, TKFRZ,
!$OMP&                           TWSFC, TWBAR, QRTOT, QITOT, LFRAC,
!$OMP&                           CFRACL,CFRACM, CFRACH ),
!$OMP&                  PRIVATE( R, L, C, I, J, K, N, DFAC, DTA, DPP,
!$OMP&                           DQV, PR, QQ, WW, EE, TD, ES, ED, S, TW,
!$OMP&                           EW, DE, DER, RL, RI, LP,
!$OMP&                           DZ, ZT, EBL, ZH, TWET, NL, LTOP,
!$OMP&                           VAP, RHL, RHTOP, RHMID, RHBOT )

        DO  R = 1, NROWS

            I = I0 + R

            DO  C = 1, NCOLS
                J = J0 + C
                RAIN( C,R )  = RAINC( I,J ) + RAINNC( I,J )
                T2( C,R )    =   TA2( I,J )
                UX10( C,R ) = 0.25*( UA10( I+1,J   ) + UA10( I,J   ) 
     &                             + UA10( I+1,J+1 ) + UA10( I,J+1 ) )
                VX10( C,R ) = 0.25*( VA10( I+1,J   ) + VA10( I,J   ) 
     &                             + VA10( I+1,J+1 ) + VA10( I,J+1 ) )
            END DO

            K = K0 - 1
            DO  C = 1, NCOLS      !  loop for layer-1 wet-bulb temperature
                J = J0 + C

                DFAC( C ) = 1.0 / PSTAR( I,J )
                
                !!  iteration for wet-bulb temperature:
                
                DTA = DFAC( C ) * TA( I,J,K )
                DPP = DFAC( C ) * PP( I,J,K )
                DQV = DFAC( C ) * QV( I,J,K )
                PR  = 0.01*( CPA*( PSTAR(I,J)*SIGH(K) + PTOP ) + DPP )  !  mb
                QQ  = MAX( DQV, 1.0E-15 )
                WW  = QQ / ( 1.0 - QQ )
                EE  = QQ * PR/( EPS + 0.378*QQ )
                TD  = AMAX1( E1 / (E2 - ALOG( EE*D611 ) ), 200. )
                ES  = EXP (C15 - C1*DTA - C2/DTA)
                ED  = EXP (C15 - C1*TD  - C2/TD)
                S   = (ES - ED) / (DTA - TD)
                TW  = (DTA*F*PR + TD*S) / ( F*PR + S)
                DO  N = 1, 4
                    EW  = EXP (C15 - C1*TW - C2/TW )
                    DE  = F*PR*(DTA - TW ) - (EW - ED)
                    DER = EW*(C1 - C2/TW**2) - F*PR
                    TW  = TW - DE/DER
                END DO

                TWSFC( C,R ) = TW
                TWBAR( C,R ) = TW

                RL = QR(I-1,J-1,K ) + QR(I,J-1,K ) + QR(I+1,J-1,K ) +
     &               QR(I-1,J  ,K ) + QR(I,J  ,K ) + QR(I+1,J  ,K ) +
     &               QR(I-1,J+1,K ) + QR(I,J+1,K ) + QR(I+1,J+1,K )

                RI = QI(I-1,J-1,K ) + QI(I,J-1,K ) + QI(I+1,J-1,K ) +
     &               QI(I-1,J  ,K ) + QI(I,J  ,K ) + QI(I+1,J  ,K ) +
     &               QI(I-1,J+1,K ) + QI(I,J+1,K ) + QI(I+1,J+1,K )

                QRTOT( C,R ) = DFAC( C ) * RL
                QITOT( C,R ) = DFAC( C ) * RI

            END DO              !  end loop for layer-1 wet-bulb temperature

            DO  L = 2, NEARSFC  !  loop for nearsfc wet-bulb temperature

                K = K0 - L

                DO  C = 1, NCOLS

                    J = J0 + C

                    !!  iteration for wet-bulb temperature:

                    DTA = DFAC( C ) * TA( I,J,K )
                    DPP = DFAC( C ) * PP( I,J,K )
                    DQV = DFAC( C ) * QV( I,J,K )
                    PR  = 0.01*( CPA*( PSTAR(I,J)*SIGH(K) + PTOP ) + 
     &  		               DPP )  !  mb
                    QQ  = MAX( DQV, 1.0E-15 )
                    WW  = QQ / ( 1.0 - QQ )
                    EE  = QQ * PR/(EPS + 0.378*QQ)
                    TD  = AMAX1( E1/(E2 - ALOG( EE*D611 ) ), 200. )
                    ES  = EXP (C15 - C1*DTA - C2/DTA)
                    ED  = EXP (C15 - C1*TD  - C2/TD)
                    S   = (ES - ED) / (DTA - TD)
                    TW  = (DTA*F*PR + TD*S) / ( F*PR + S)
                    DO  N = 1, 4
                        EW  = EXP (C15 - C1*TW - C2/TW )
                        DE  = F*PR*(DTA - TW ) - (EW - ED)
                        DER = EW*(C1-C2/TW**2) - F*PR
                        TW  = TW    - DE/DER
                    END DO

                    TWBAR( C,R ) = TWBAR( C,R ) + TW

                END DO          !  end C-loop

            END DO              !  end L-loop for nearsfc wet-bulb temperature

            DO  C = 1, NCOLS

                J = J0 + C

                TWBAR( C,R ) = DLAY *   TWBAR( C,R )
                LFRAC( C,R ) = DFRZ * ( TWBAR( C,R ) - TKICE )

            END DO

            !!     Cloud-fraction calculation:

            DO  C = 1, NCOLS

                J = J0 + C

                RHTOP = 0.0
                RHMID = 0.0
                RHBOT = 0.0
                DO  L = 1, NLAYS
                    K = K0 - L

                    DPP = DFAC( C ) * PP( I,J,K )
                    PR  = CPA*( PSTAR(I,J)*SIGH(K) + PTOP ) + DPP  !  Pa

                    IF ( PR .LE. PRESBOT ) THEN
                        DTA = DFAC( C ) * TA( I,J,K )	        !  decoupled
                        DQV = DFAC( C ) * QV( I,J,K )	        !   "

CJC                     ESL = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
CJC  &                                 / ( DTA - DLIQ ) )
CJC                     RHL = F622*DQV*( PR - ESL )/ESL

                        VAP = CPA*SVP1*EXP(SVP2*(DTA-SVPT0)/(DTA-SVP3))  
C			      !  Pa
                        RHL = ( PR - VAP )*DQV / ( EP2 * VAP )

                        IF      (  PR .LE. PRESTOP ) THEN
                            RHTOP = MAX( RHL, RHTOP )
			    IF( C.EQ.94.and.R.EQ.66.and.L.EQ.28) THEN !! debug rhtop CFRACH calculation
			       PRINT*,'L,RHL,RHTOP,R=66,C=94,L=28: ',
     &         			       L,RHL,RHTOP
			       PRINT*,'DEBUG, PR,VAP,DQV,EP2,DTA=',
     &                         PR,VAP,DQV,EP2,DTA			  
                            ENDIF
                        ELSE IF (  PR .LE. PRESMID ) THEN
                            RHMID = MAX( RHL, RHMID )
                        ELSE   !  pr .le. presbot
                            RHBOT = MAX( RHL, RHBOT )
                        END IF
                    END IF

                END DO          !  end L-loop

                CFRACL( C,R ) = MIN(MAX( 4.0*RHBOT - 3.0, 0.0), 1.0)
                CFRACM( C,R ) = MIN(MAX( 4.0*RHMID - 3.0, 0.0), 1.0)
                CFRACH( C,R ) = MIN(MAX( 2.5*RHTOP - 1.5, 0.0), 1.0)
            
            END DO          !  end C-loop:  cloud-fraction calculation

            !!   Calculate elevation ZWET0 above MSL at which TWET is freezing:

            DO  C = 1, NCOLS

                J   = J0 + C

                PR  = CPA*( PSTAR( I,J ) + PTOP )
                LP  = ALOG( PR * DP0 )
                ZT  = -ROVG*LP*( 0.5*TLP*LP  + TS0 )
                EBL =  MIN( PBL( I,J ), 500.0 )
                NL  = 0

                DO   L = 2, NLAYS
                     K = K0 - L

                     PR  = CPA*( PSTAR( I,J )*SIGH(K) + PTOP)  ! ref press, Pa
                     LP  = ALOG( PR * DP0 )
                     ZH( L ) = -ROVG*LP*( 0.5*TLP*LP + TS0 ) - ZT

                     IF ( ZH( L ) .GT. EBL ) THEN
                         NL  = NL + 1
                         DTA = DFAC( C ) * TA( I,J,K )
                         DPP = DFAC( C ) * PP( I,J,K )
                         DQV = DFAC( C ) * QV( I,J,K )
                         PR  = 0.01*( CPA*( PSTAR(I,J)*SIGH(K) + PTOP )
     &			                  + DPP )  !  mb
                         QQ  = MAX( DQV, 1.0E-15 )
                         WW  = QQ / ( 1.0 - QQ )
                         EE  = QQ * PR/(EPS + 0.378*QQ)
                         TD  = AMAX1( E1/(E2 - ALOG( EE*D611 ) ), 200. )
                         ES  = EXP (C15 - C1*DTA - C2/DTA)
                         ED  = EXP (C15 - C1*TD  - C2/TD)
                         S   = (ES - ED) / (DTA - TD)
                         TW  = (DTA*F*PR + TD*S) / ( F*PR + S)
                         DO  N = 1, 4
                             EW  = EXP (C15 - C1*TW - C2/TW )
                             DE  = F*PR*(DTA - TW ) - (EW - ED)
                             DER = EW*(C1-C2/TW**2) - F*PR
                             TW  = TW - DE/DER
                         END DO
                         TWET( L ) = TW
                         IF ( TW .LT. CTOK ) THEN
                             LTOP = L
                             GO TO  11
                         END IF
                     END IF

                END DO

11              CONTINUE        !  exit from wet-bulb=zero calculation

                IF ( NL .GT. 1 ) THEN
                    L  = LTOP
                    DZ = ZH( L ) - ZH( L-1 )
                    QQ =  ( TWET( L-1 ) - CTOK )
     &                   /( TWET( L-1 ) - TWET( L ) )
                    ZWET0( C,R ) = ZH( L-1 ) + QQ * DZ
                ELSE
                    ZWET0( C,R ) = EBL
                END IF

            END DO

        END DO          !  end parallel loop on R

        RETURN
        END

