C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5 cpyright (C) 1998-2002 MCNC and
C Carlie J. Coats, Jr., and (C) 2003 Baron Advanced Meteorological Systems
C This routine copyright (C) 2003-2008 BAMS.
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2HTR7( DX, IM, JM, KM, NEARSFC, TKICE, TKLIQ,TKFRZ,
     &                      TA, QVA, QRA, QNIA, PPA, U, V, PSTAR0,
     &                      RAINC, RAINNC, TA2, QV2, UA10, VA10, PBL,
     &                      TER, SIGH, PTOP, P00, TLP, TS0, ROVG,
     &                      NCOLS, NROWS, NLAYS, NPDIM, NPVAL, PVALS,
     &                      COL0,  ROW0,  LAY0,
     &                      TWSIG1, TWBAR, LFRAC, QRLAY1, QNILAY1,
     &                      RAIN, TALAY1, TNSFC, T2, TDEW2,
     &                      UX10, VX10, CFRACL, CFRACM, CFRACH, SLP,
     &                      UP, VP, ZP, RHP, TAP, RVORP )

C***********************************************************************
C  subroutine body starts at line  196
C
C  FUNCTION:
C       High-time resolution output subroutine.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype 3/2003 by Carlie J. Coats, Jr., BAMS
C       Version   5/2003 that does wet-bulb calculation
C       Version   7/2003 that calculates Z-value ZWET0 such that
C       TWET = 0.0 C at elevation ZWET0; accumulated total RAIN(cm),
C       2-meter temperature, 10 meter wind components, high-, mid-,
C       and low-level cloud fraction (0 <= CFRAC* <= 1.  Uses RH
C       formulation borrowed from SFCRAD.
C       Version 8/2003 by CJC does surface (2-meter) dewpoint, and
C       pressure-interpolation for  Z, U, V, RH  for a user-specified
C       set of pressure levels.
C       Version 9/2003 by JNM and CJC correcting bugs in the 8/2003 version,
C       implementing more robust ZWET0 and Z calculations, and 
C       introducing MSLP. Reference routines MM2SLP and MMPZ. Improved
C       2-meter dewpoint calculation using metcalc approach.
C       Version 5A: 4/04 by JNM -- removed ZWET0 calculation
C       Version   9/2007 by CJC:  get RH treatment from MM2THERM3()
C       Version   6/2008 by CJC:  pressure level temperature and
C       relative vorticity
C***********************************************************************

      IMPLICIT NONE
      INCLUDE  'PARMS3.EXT'

#include "parame.incl"
#include "pmoist.incl"

C...........   ARGUMENTS and their descriptions:

        REAL            DX      !  cell size
        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension fomcpl.Fr AM
        INTEGER         NEARSFC !  # of layers for averaging
        REAL            TKICE, TKLIQ, TKFRZ
        REAL		TA( IM, JM, KM )	!  MM5-order input array
        REAL		QVA( IM, JM, KM )	!  MM5-order input array
        REAL		QRA( IM, JM, KM )	!  MM5-order input array
        REAL	        QNIA( IM, JM, KM )	!  MM5-order input array
        REAL		PPA( IM, JM, KM )	!  MM5-order input array
        REAL		 U( IM, JM, KM )	!  MM5-order input array
        REAL		 V( IM, JM, KM )        !  MM5-order input array
        REAL	    PSTAR0( IM, JM )		!  MM5-order PSTAR0 array
        REAL	     RAINC( IM, JM )		!  MM5-order convective rain
        REAL	    RAINNC( IM, JM )	!  MM5-order nonconvective rain
        REAL	       TA2( IM, JM )	!  MM5-order 2-meter decoupled temp (K)
        REAL	       QV2( IM, JM )	!  MM5-order 2-meter decoupled mix. rat.
        REAL	      UA10( IM, JM )	!  MM5-order 10-meter decoupled U wind
        REAL	      VA10( IM, JM )	!  MM5-order 10-meter decoupled V wind
        REAL	       PBL( IM, JM )	!  MM5-order PBL height
        REAL           TER( IM, JM )    !  MM5 terrain array == geopotential ht 
C                                       !  (i.e. actual height times g)
        REAL		SIGH( KM )	!  sigma half-levels
        REAL            PTOP            !  MM5 ptop (cb)
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            TLP		!  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		!  MM5 ref SL temp (K)
        REAL            ROVG		!  MM5 R/G
        INTEGER         NCOLS	        !  output dimension
        INTEGER         NROWS	        !  " 
        INTEGER         NLAYS	        !  "
        INTEGER         COL0	        !  starting subscript for output window
        INTEGER         ROW0	        !  "
        INTEGER         LAY0	        !  "
        INTEGER         NPDIM	        !  dim'ed number of pres-interp levels
        INTEGER         NPVAL	        !  actual number of pres-interp levels
	REAL		PVALS( NPVAL )  !  pressure-interp levels (Pa)
	REAL	       TWSIG1( NCOLS, NROWS )
	REAL		TWBAR( NCOLS, NROWS )
	REAL		LFRAC( NCOLS, NROWS )
	REAL	       QRLAY1( NCOLS, NROWS )
	REAL	      QNILAY1( NCOLS, NROWS )
	REAL		 RAIN( NCOLS, NROWS )    !  total rain
	REAL           TALAY1( NCOLS, NROWS )    !  layer-1 temperature (K)
	REAL            TNSFC( NCOLS, NROWS )    !  near-sfc mean temperature (K)
	REAL		   T2( NCOLS, NROWS )    !  2-meter temperature (K)
	REAL            TDEW2( NCOLS, NROWS )    !  2-meter dewpoint (K)
	REAL           TDSIG1( NCOLS, NROWS )    !  layer-1 dewpoint (K)
	REAL	         UX10( NCOLS, NROWS )    !  10-meter cross point U
	REAL             VX10( NCOLS, NROWS )    !  10-meter cross point V
	REAL           CFRACL( NCOLS, NROWS )    !  high-level cloud fraction
	REAL           CFRACM( NCOLS, NROWS )    !  mid-level cloud fraction
	REAL           CFRACH( NCOLS, NROWS )    !  low-level cloud fraction
	REAL              SLP( NCOLS, NROWS )
	REAL               UP( NCOLS, NROWS, NPDIM )    !  pressure-interp U
	REAL               VP( NCOLS, NROWS, NPDIM )    !  pressure-interp V
	REAL               ZP( NCOLS, NROWS, NPDIM )    !  pressure-interp Z
	REAL              RHP( NCOLS, NROWS, NPDIM )    !  pressure-interp RH
	REAL              TAP( NCOLS, NROWS, NPDIM )    !  pressure-interp TA
	REAL            RVORP( NCOLS, NROWS, NPDIM )    !  pressure-interp rel vor

C...........   PARAMETERs

        REAL	    	CPA             ! conv facs Pa to cb
        REAL	    	CTOK            ! temperature conv term, C to K
        REAL	    	C1
        REAL	    	C15
        REAL	    	C2
        REAL	    	CP
        REAL	    	D611
        REAL	    	E1
        REAL	    	E2
        REAL	    	EPS
        REAL	    	F
	REAL            G
	REAL            GAMMA
        REAL	    	ESL
        REAL	    	ESL1
        REAL	    	ESL2
        REAL	    	ESCR
	REAL            MBTOPA
	REAL            PATOMB
	REAL            PCONST
        REAL            PRESBOT         !  Pa
        REAL            PRESMID         !  Pa
        REAL            PRESTOP         !  Pa
	REAL            RDGAS
	REAL            RDG
	REAL            TC
	REAL            TFRZOUT
	REAL            TWODR
	REAL            XTERM
        REAL            F622
        INTEGER         MAXC
	
        PARAMETER     ( CPA     =     1.0E3  ,
     &                  CTOK    =   273.15 ,
     &                  C1      =     0.0091379024 ,
     &                  C15     =    26.66082 ,
     &                  C2      =  6106.396 ,
     &                  CP      =  1004.0 ,
     &                  D611    =     1.0 / 6.112 ,
     &                  E1      =  5418.12 ,
     &                  E2      =    19.84659 ,
     &                  EPS     =     0.622 ,
     &                  F       =     0.0006355 ,
     &                  G       =     9.8 ,
     &                  GAMMA   =     6.5E-3 ,
     &                  ESL1    =    17.67 ,
     &                  ESL2    =    29.65 ,
     &                  ESCR    =    ESL1 * CTOK ,
     &                  MBTOPA  =     1.E2 ,
     &                  PATOMB  =     1.E-2 ,
     &                  PCONST  =   100. ,
     &                  PRESBOT =   970.0E2 ,
     &                  PRESMID =   800.0E2 ,
     &                  PRESTOP =   450.0E2 ,
     &                  RDGAS   =   287.04 ,
     &                  RDG     =   RDGAS/G ,
     &                  TC      =   273.16 + 17.5 ,                 
     &                  TFRZOUT =   263.15 ,
     &                  TWODR   =   2.0/RDGAS ,
     &                  XTERM   =   GAMMA*RDGAS/G,
     &                  MAXC    =   512 )

        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ

        PARAMETER     ( ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  F622   =    1.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L, M, N	        !  I/O API col subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        REAL            DLAY, DFRZ
        REAL            RL, RI, DTA, TW, DZ, LP, DQR, DQN
        REAL            DP0, DPP, DQV, PR, QQ, WW, EE, TD, ES, ED, S
        REAL            EW, DE, DER, PSFC
	INTEGER		LTOP
        REAL            P0DIF        !  (P00 in CB) - PTOP	
        REAL            ZT, VAP, RHL, RHTOP, RHMID, RHBOT
        REAL            ULO, UHI, VLO, VHI, DRH, DTT, DSIGH
	REAL            SIG0, PRS0, SIG, TDSIG
        REAL            VFAC, DUDY, DVDX
	
	INTEGER         KLO, KHI
	REAL            T0, PK, PL, TS, DPHI, PRAT
	REAL            PHI, PLO, THI, TLO, TL
	REAL            TBAR, HL
	
	CHARACTER*256   MESG


#ifdef LANGUAGE_FORTRAN90
        REAL		DFAC( NCOLS )
        REAL		TWET( NCOLS )
        REAL		  ZH( NLAYS )
        REAL		  PH( NLAYS )
        REAL		  RH( NLAYS )
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL		DFAC( MAXC )
        REAL		TWET( MAXC )
        REAL	  	  ZH( MAXC )
        REAL	  	  PH( MAXC )
        REAL	  	  RH( MAXC )

        IF ( NCOLS.GT. MAXC .OR. KM .GT. MAXC ) THEN
            CALL M3EXIT( 'MCPL:MM2HTR5', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAXC > KM', 2 )
        END IF
#endif    /* ifndef LANGUAGE_FORTRAN90  */


C***********************************************************************
C   begin body of subroutine  MM2HTR5

        I0    = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0    = COL0 - 1
        K0    = KM - LAY0 + 2
        DLAY  = 1.0 / FLOAT( NEARSFC )
        DFRZ  = 1.0 / ( TKLIQ - TKICE )
        DP0   = 1.0 / P00
	P0DIF = P00 / CPA  -  PTOP   !! now in centibars
        VFAC  = 0.5 / DX

!$OMP       PARALLEL DO DEFAULT( NONE ),
!$OMP&                   SHARED( I0, J0, K0, KM, NROWS, NCOLS, NLAYS,
!$OMP&                           PSTAR0, TA, QVA, QRA, QNIA, PPA, U, V,
!$OMP&                           PBL, TER, SIGH, PTOP, ROVG, TLP, TS0,
!$OMP&                           NPVAL, PVALS, EP2, SVP1, SVP2, SVP3,
!$OMP&                           SVPT0, NEARSFC, DLAY, DFRZ, DP0, RAIN,
!$OMP&                           RAINC, RAINNC, UX10, VX10, UA10, VA10,
!$OMP&                           T2, TALAY1, TNSFC, TA2, QV2, TDEW2,
!$OMP&                           TDSIG1, TKICE, TKFRZ, TWSIG1,
!$OMP&                           TWBAR, QRLAY1, QNILAY1, LFRAC, CFRACL,
!$OMP&                           CFRACM, CFRACH, ZP, UP, VP, RHP, SLP, 
!$OMP&                           TAP, P00, P0DIF, VFAC, RVORP ),
!$OMP&                  PRIVATE( R, C, I, J, K, L, M, N, DFAC, DTA, DPP,
!$OMP&                           DQV, PR, QQ, WW, EE, TD, ES, ED, S, TW,
!$OMP&                           TDSIG, EW, DE, DER, RL, RI, LP, DZ, ZT,
!$OMP&                           PH, ZH, TWET, LTOP, VAP, RHL, ESL,
!$OMP&                           RHTOP, RHMID, RHBOT, DQR, DQN,
!$OMP&                           ULO, UHI, VLO, VHI, DRH, RH, DTT, PSFC, 
!$OMP&                           DSIGH, KLO, KHI, T0, PK, PL,
!$OMP&                           TS, DPHI, PRAT, PHI, PLO, THI, TLO, 
!$OMP&                           TL, TBAR, HL, MESG, SIG0, PRS0, SIG, 
!$OMP&                           DUDY, DVDX )

        DO  R = 1, NROWS

            I = I0 + R

            DO  C = 1, NCOLS
                J = J0 + C

		RAIN( C,R )  = RAINC( I,J ) + RAINNC( I,J )
                T2( C,R )    =  TA2( I,J )
                UX10( C,R ) = 0.25*( UA10( I+1,J   ) + UA10( I,J   ) 
     &                             + UA10( I+1,J+1 ) + UA10( I,J+1 ) )
                VX10( C,R ) = 0.25*( VA10( I+1,J   ) + VA10( I,J   ) 
     &                             + VA10( I+1,J+1 ) + VA10( I,J+1 ) )                

            END DO

            K = K0 - 1            !  LAY0 *MUST* == 1 on input to subroutine!!
            DO  C = 1, NCOLS      !  loop for 2m Td and layer-1 wet-bulb T
	                          !  Note: K=31 or surface model layer in this loop!!
                J = J0 + C

                DFAC( C ) = 1.0 / PSTAR0( I,J )  ! column decoupling factor
                DTA = DFAC( C ) * TA( I,J,K )    !! scalar local decoupled TA
                DPP = DFAC( C ) * PPA( I,J,K )   !! scalar local decoupled PP
                DQV = DFAC( C ) * QVA( I,J,K )   !! scalar local decoupled QV
                DQR = DFAC( C ) * QRA( I,J,K )   !! scalar local decoupled QR
                DQN = DFAC( C ) * QNIA( I,J,K )  !! scalar local decoupled QNI		
				
		!! store layer-1 temp, QR, QNI
		
		QRLAY1(C,R)  =  DQR
		QNILAY1(C,R) =  DQN
		TALAY1(C,R)  =  DTA
		TNSFC(C,R)   =  TALAY1(C,R)
		
		!! 2-meter and layer-1 dewpt calculations following metcalc approach
		
		PSFC  = 0.01*( CPA*(PSTAR0(I,J) + PTOP) + DPP )  ! mb: at sfc !				
		QQ  = MAX( QV2( I,J ), 1.0E-25 )
                EE  = ALOG( D611 * PSFC * QQ / ( EPS + QQ ) )
		
		IF( T2( C,R ) .GT. TFRZOUT ) THEN 
                    TDEW2( C,R ) = ( ESL2*EE - ESCR ) / ( EE - ESL1 )
		ELSE
		    TDEW2( C,R ) = 6150. / (22.514 - EE)  !  degrees K
		END IF
		
		!!  constrain dewpoint to be at or above ambient 2m temp
		
		TDEW2( C,R ) = MIN( TDEW2( C,R ), T2( C,R ) ) 
		
		PR  = 0.01*( CPA*(PSTAR0(I,J)*SIGH(K) + PTOP) + DPP )  ! mb
		QQ  = MAX( DQV, 1.0E-25 )
                EE  = ALOG( D611 * PR * QQ / ( EPS + QQ ) )		
		
                IF( DTA .GT. TFRZOUT ) THEN 
                    TDSIG1( C,R ) = ( ESL2*EE - ESCR ) / ( EE - ESL1 )
		ELSE
		    TDSIG1( C,R ) = 6150. / (22.514 - EE)  !  degrees K
		END IF
		
		!!  constrain dewpoint to be at or above ambient layer-1 TA
		
		TDSIG1(C,R) = MIN( TDSIG1( C,R ), DTA )		
						
                !!  iteration for layer-1 wet-bulb temperature: 
		!!  PR, QQ calculated just above at layer-1
				                        
                WW  = QQ / ( 1.0 - QQ )
                EE  = QQ * PR/( EPS + 0.378*QQ )
                TD  = AMAX1( E1 / (E2 - ALOG( EE*D611 ) ), 200. )
                ES  = EXP( C15 - C1*DTA - C2/DTA )
                ED  = EXP( C15 - C1*TD  - C2/TD )
                S   = ( ES - ED ) / ( DTA - TD )
                TW  = ( DTA*F*PR + TD*S ) / ( F*PR + S )
                DO  N = 1, 4
                    EW  = EXP ( C15 - C1*TW - C2/TW )
                    DE  = F*PR*( DTA - TW ) - ( EW - ED )
                    DER = EW*( C1 - C2/TW**2 ) - F*PR
                    TW  = TW - DE/DER
                END DO

                !! constrain layer-1 wet-bulbT to be between Td and TA

		TWSIG1( C,R ) = MAX( MIN( TW, DTA ), TDSIG1(C,R) )
                TWBAR( C,R )  = TWSIG1( C,R )

            END DO              !  end loop for layer-1 wet-bulb temperature

            DO  L = 2, NEARSFC  !  loop for nearsfc wet-bulb temperature

                K = K0 - L

                DO  C = 1, NCOLS

                    J = J0 + C

                    DTA = DFAC( C ) * TA( I,J,K )
                    DPP = DFAC( C ) * PPA( I,J,K )
                    DQV = DFAC( C ) * QVA( I,J,K )
                    PR  = 0.01*(CPA*(PSTAR0(I,J)*SIGH(K) + PTOP) + DPP)  !  mb
                    QQ  = MAX( DQV, 1.0E-25 )
                    EE  = ALOG( D611 * PR * QQ / ( EPS + QQ ) )		    
		    
                    IF( DTA .GT. TFRZOUT ) THEN 
                       TDSIG = ( ESL2*EE - ESCR ) / ( EE - ESL1 )
		    ELSE
		       TDSIG = 6150. / (22.514 - EE)  !  degrees K
		    END IF
		    
		    !!  constrain dewpoint to be at or above ambient layer-1 TA
		
		    TDSIG = MIN( TDSIG, DTA )
		    				    		    
                    !!  iteration for wet-bulb temperature from layer-2
		    !!  to NEARSFC: PR, QQ calculated just above		    
		    		    
                    WW  = QQ / ( 1.0 - QQ )
                    EE  = QQ * PR/(EPS + 0.378*QQ)
                    TD  = AMAX1( E1/(E2 - ALOG( EE*D611 ) ), 200. )
                    ES  = EXP( C15 - C1*DTA - C2/DTA )
                    ED  = EXP( C15 - C1*TD  - C2/TD )
                    S   = (ES - ED) / (DTA - TD)
                    TW  = (DTA*F*PR + TD*S) / ( F*PR + S)
                    DO  N = 1, 4
                        EW  = EXP (C15 - C1*TW - C2/TW )
                        DE  = F*PR*(DTA - TW ) - (EW - ED)
                        DER = EW*(C1-C2/TW**2) - F*PR
                        TW  = TW    - DE/DER
                    END DO
                    
		    !! constrain this layer wet-bulbT to be between Td and TA
                    
		    TW = MAX( MIN( TW, DTA ), TDSIG)
		     
                    TWBAR( C,R ) = TWBAR( C,R ) + TW
		    
		    TNSFC( C,R ) = TNSFC( C,R ) + DTA

                END DO          !  end C-loop

            END DO              !  end L-loop for nearsfc wet-bulb T

            DO  C = 1, NCOLS

                J = J0 + C

                TWBAR( C,R ) = DLAY *   TWBAR( C,R )
                LFRAC( C,R ) = DFRZ * ( TWBAR( C,R ) - TKICE )
		TNSFC( C,R ) = DLAY *   TNSFC( C,R )

            END DO

            !!   MSLP, Cloud-fraction, ht of wet-bulb-0, and p-interpolation

            DO  C = 1, NCOLS

                J   = J0 + C
		RHTOP = 0.0
		RHMID = 0.0
		RHBOT = 0.0
		
C ...           terrain height ... divide out G from TER

                ZT  = TER(I,J)/G            

		DO   L = 1, NLAYS            !! L-loop for cloud frac and TWET0
		
                    K = K0 - L

C ...               half-layer sigma height, actual press, and actual rh ... 

                    PR  = CPA*(PSTAR0( I,J )*SIGH(K) + PTOP)     ! ref pres, Pa
                    LP  = ALOG( PR * DP0 )
                    ZH( L ) = -ROVG*LP*( 0.5*TLP*LP + TS0 ) - ZT ! sigmah ht

                    DPP = DFAC( C ) * PPA( I,J,K )               ! decoupled PP
                    PR  = PR + DPP                               ! act pres, Pa
                    PH( L ) = PR                                 ! act pres, Pa

                    DTA = DFAC( C ) *  TA( I,J,K )	         ! decoupled TA
                    DQV = DFAC( C ) * QVA( I,J,K )	         !   "       QV

                    !!  liquid water single-phase RH formulation:

                    ESL = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                     / ( DTA - DLIQ ) )
                    RHL = F622 * DQV * ( PH( L ) - ESL ) / ESL
                    RH( L ) = MIN( MAX( RHL, 0.0 ), 1.0 )
		
                    IF      (  PH(L) .LE. PRESTOP ) THEN
                        RHTOP = MAX( RH( L ), RHTOP )
                    ELSE IF (  PH(L) .LE. PRESMID ) THEN
                        RHMID = MAX( RH( L ), RHMID )
                    ELSE IF ( PH(L) .LE. PRESBOT ) THEN
                        RHBOT = MAX( RH( L ), RHBOT )
                    END IF      !  if OK pressure for cloud calc.
                      
                END DO                !! end of L-loop for cloud frac
		
                CFRACL( C,R ) = MIN( MAX( 4.0*RHBOT - 3.0, 0.0 ), 1.0 )
                CFRACM( C,R ) = MIN( MAX( 4.0*RHMID - 3.0, 0.0 ), 1.0 )
                CFRACH( C,R ) = MIN( MAX( 2.5*RHTOP - 1.5, 0.0 ), 1.0 )
		
		!!  End computing cloud fraction and TWET0

                !!  Begin calculating mean-sea-level pres following mm2slp

C               Compute pressure at the surface -- (following INTERP),
C               this is an estimate with the assumption that PP at
C               the lowest half sigma level is equivalent to PP at 
C               the model surface where full sigma = 1.

                PSFC  = CPA*(PSTAR0(I,J) + PTOP) + DFAC(C)* PPA(I,J,KM)  !! Pa
              
C     ...       Compute pressure at PCONST MB above surface (PL)
C               Note:  must decouple PP...watch units.

                PL  = PSFC - PCONST*MBTOPA       !! in Pa
                KHI = 1
                PHI = CPA*(SIGH(KHI)*PSTAR0(I,J) + PTOP) + 
     &                DFAC(C)*PPA(I,J,KHI)

                KLO = KM
                PLO = CPA*(SIGH(KLO)*PSTAR0(I,J) + PTOP) +
     &                DFAC(C)*PPA(I,J,KLO)
     
                IF( ( PLO .GE. PL ) .AND. ( PHI .LT. PL ) ) THEN
         
31                  CONTINUE    !  binary search for PL
                        K  = ( KLO + KHI ) / 2
                        PK = CPA*(SIGH(K)*PSTAR0(I,J) + PTOP) +
     &                       DFAC(C)*PPA(I,J,K)
                        IF ( PK .LT. PL ) THEN
                            KHI = K
                            PHI = PK
                        ELSE
                            KLO = K
                            PLO = PK
                        END IF
                        IF ( KHI .LT. KLO-1 ) THEN
                            GO TO  31
                        END IF

                ELSE

                    WRITE( MESG ,'( A, 2I5 )' )
     &                    'I,J  = ',I,J
                    CALL M3MSG2( MESG )
                    WRITE( MESG ,'( 2( A, 1PE15.6, 2X ) )' )
     &                    'PLO,PHI= ',PLO, PHI,
     &                    'PSFC,PL= ',PSFC, PL
                    MESG = 'PL not trapped between 1 and KM'
                    CALL M3WARN( 'MCPL:MM2HTR5', 0,0, MESG, 2 )

                END IF     
     
C     ... get TL temperature at pl, extrapolate T at surface (TS)
C         and T at sea level (T0) with 6.5 K/km lapse rate

                DPHI = 1.0 / PHI
                PRAT = PSFC / PL
                TLO  = DFAC(C)*TA(I,J,KLO)  !  decoupled LO temperature
                THI  = DFAC(C)*TA(I,J,KHI)  !  decoupled HI temperature
                TL   = THI - (THI - TLO)*ALOG(PL*DPHI)/ALOG(PLO*DPHI)
                TS   = TL * PRAT**XTERM
                TBAR = (TS + TL)*0.5
                HL   = ZT + RDG*TBAR*ALOG( PRAT )  !! using TER/G = ZT
                T0   = TL + GAMMA*HL               !! Sea-level est. Temp.

C     ...       Correct sea level temperature if too hot

                IF ( T0 .GE. TC ) THEN
                   IF( TS .LE. TC ) THEN
                      T0 = TC
                   ELSE
                      T0 = TC - 0.005*( TS - TC )**2
                   END IF 
                END IF

C     ...       Compute sea level pressure

                SLP(C,R) = PATOMB*PSFC*EXP( TWODR*TER(I,J)/( TS + T0 ) )
                 
		!!  End calculating mean sea-level pressure in mb 

                !!  Now do pressure-interpolations:

                M = 1

                DO   L = 1, NLAYS

11                  CONTINUE    !  loop on pressure levels at this sigma

                    IF ( M .GT. NPVAL ) THEN    !  interp done.

                        GO TO 99

                    ELSE IF ( PH( L ) .LT. PVALS( M ) ) THEN
		    
		        K  = K0 - L

                        IF ( L .EQ. 1 ) THEN    !  level is below model bottom

                            !!  return lowest-level U, V, RH, UX. VX. RVOR, TA

                            UHI = U(I  ,J,K)   + U(I  ,J+1,K)
     &                          + U(I+1,J,K)   + U(I+1,J+1,K)
                            VHI = V(I  ,J,K)   + V(I  ,J+1,K)
     &                          + V(I+1,J,K)   + V(I+1,J+1,K)

                            UP(C,R,M)  = 0.25*DFAC( C )*UHI
                            VP(C,R,M)  = 0.25*DFAC( C )*VHI

                            !!  relative vorticity:

                            UHI = U(I+1,J  ,K) - U(I,J  ,K)
     &                          + U(I+1,J+1,K) - U(I,J+1,K)
                            DUDY = VFAC * DFAC(C) * UHI
                            VHI = V(I+1,J+1,K) - V(I+1,J  ,K)
     &                          + V(I  ,J+1,K) - V(I  ,J  ,K)
                            DVDX = VFAC * DFAC(C) * VHI
                            RVORP(C,R,M) = DVDX - DUDY

                            TAP(C,R,M) = DFAC( C )*TA( I,J,K )

                            RHP(C,R,M) = 100.*
     &			                 MIN( MAX(RH( 1 ),0.0), 1.0)
			    
			    !! Must compute height-w/rt-sea-level of requested 
			    !! pressure surface below ground.
			    
			    !! compute sigma-value SIG0 at sea level;
			    !! use SLP from above
			    			    
			    SIG0 = P0DIF*DFAC(C)			    
			    PRS0 = MBTOPA*SLP(C,R)
			    
			    !! compute pressure-interpolated sigma
			    !! (See mmpz.F)

			    QQ = (PVALS(M) - PRS0)/(PH(L) - PRS0)
			    DSIGH = SIGH(K) - SIG0
			    SIG = SIG0 + QQ*DSIGH 
			    
                        ELSE    !  can pressure-interpolate:

                            QQ = (PVALS(M) - PH(L-1))/(PH(L) - PH(L-1))
			    DSIGH = SIGH(K) - SIGH(K+1)
			    SIG = SIGH(K+1) + QQ*DSIGH

                            UHI = U(I  ,J,K)   + U(I  ,J+1,K)
     &                          + U(I+1,J,K)   + U(I+1,J+1,K)
                            ULO = U(I  ,J,K+1) + U(I  ,J+1,K+1)
     &                          + U(I+1,J,K+1) + U(I+1,J+1,K+1)
                            UP(C,R,M) = 0.25*DFAC(C)
     &                                      *( ULO + QQ*( UHI - ULO ) )

                            VHI = V(I  ,J,K)   + V(I  ,J+1,K)
     &                          + V(I+1,J,K)   + V(I+1,J+1,K)
                            VLO = V(I  ,J,K+1) + V(I  ,J+1,K+1)
     &                          + V(I+1,J,K+1) + V(I+1,J+1,K+1)
                            VP(C,R,M) = 0.25*DFAC(C)
     &                                      *( VLO + QQ*( VHI - VLO ) )

                            !!  relative vorticity:

                            ULO = U(I+1,J  ,K+1) - U(I  ,J  ,K+1)
     &                          + U(I+1,J+1,K+1) - U(I  ,J+1,K+1)
                            UHI = U(I+1,J  ,K  ) - U(I  ,J  ,K  )
     &                          + U(I+1,J+1,K  ) - U(I  ,J+1,K  )
                            DUDY = VFAC * DFAC(C) 
     &                                    * ( ULO+ QQ*( UHI - ULO ) )

                            VLO = V(I+1,J+1,K+1) - V(I+1,J  ,K+1)
     &                          + V(I  ,J+1,K+1) - V(I  ,J  ,K+1)
                            VHI = V(I+1,J+1,K  ) - V(I+1,J  ,K  )
     &                          + V(I  ,J+1,K  ) - V(I  ,J  ,K  )
                            DVDX = VFAC * DFAC(C) 
     &                                    * ( VLO+ QQ*( VHI - VLO ) )

                            RVORP(C,R,M) = DVDX - DUDY

C                            DTT        = DFAC(C)*
C     &                                   ( TA( I,J,K ) - TA( I,J,K+1 ) )
                            DTT        = TA( I,J,K ) - TA( I,J,K+1 )
C                            TAP(C,R,M) = TA( I,J,K+1 ) + QQ*DTT
                            TAP(C,R,M) = DFAC(C) *
     &                                   ( TA( I,J,K+1 ) + QQ*DTT )

                            DRH        = RH( L ) - RH( L-1 )
                            RHP(C,R,M) = RH( L-1 ) + QQ*DRH
			    RHP(C,R,M) = 100.*
     &			                 MIN( MAX(RHP(C,R,M),0.0), 1.0)

                        END IF          !!  if L=1, or not
			
			PR = CPA * ( SIG*PSTAR0( I,J ) + PTOP )  ! ref pres 
			                                         ! at interped 
								 ! sigma, Pa
			LP = ALOG( PR * DP0 )			
			ZP( C,R,M ) = -ROVG*LP*( 0.5*TLP*LP + TS0 ) !! height of 
								    !! requested
                        M  = M + 1

                        GO TO  11       !  next requested pressure level

                    END IF      !  if done; else if pressure level found

                END DO          !  end pressure-interpolation L-loop
                
                !!  if you get to here:
                !!  unresolved requested pressure levels (M:NPVAL)
                
                DO L = M, NPVAL
                    ZP(   C,R,L ) = BADVAL3
                    UP(   C,R,L ) = BADVAL3
                    VP(   C,R,L ) = BADVAL3
                    RHP(  C,R,L ) = BADVAL3
                    TAP(  C,R,L ) = BADVAL3
                    RVORP(C,R,L ) = BADVAL3
                END DO
                
99              CONTINUE    !  exit pressure-interpolation loop

            END DO      !  end loop on cols

        END DO          !  end parallel loop on R

        RETURN
        END
