
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5 cpyright (C) 1998-2002 MCNC and
C Carlie J. Coats, Jr., and (C) 2003 Baron Advanced Meteorological Systems
C This routine copyright (C) 2003 BAMS.
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MM2MASK( IM, JM, KM, NEARSFC, TKICE, TKLIQ, TKFRZ,
     &                       TA, QR, QI, PP, PSTAR,
     &                       NCOLS, NROWS, NLAYS,
     &                       COL0, ROW0, LAY0,
     &                       TWSFC, TWBAR, LFRAC, LMASK, QRTOT, QITOT )

C***********************************************************************
C  subroutine body starts at line  102
C
C  FUNCTION:
C       Compute layer-1 and near-surface-averaged wet bulb temperatures
C       TWSFC and TWBAR, liquid-water-fraction TWBAR such that
C       freezing <= 0 < mixed < 1 <= liquid, precipitation phase
C       index LMASK such that 0--freezing, 1--mixed 2--liquid,
C       and 3x3-aggregated layer 1 QR and QI, using values of temperature, 
C       precipitation, etc. extracted/reordered/converted/decoupleed/computed
C       from MM5 2-D and 3-D arrays.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype 3/2003 by Carlie J. Coats, Jr., BAMS
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension for AM
        INTEGER         NEARSFC !  # of layers for averaging
        REAL            TKICE, TKLIQ, TKFRZ
        REAL		TA( IM, JM, KM )	!  MM5-order input array
        REAL		QR( IM, JM, KM )	!  MM5-order input array
        REAL		QI( IM, JM, KM )	!  MM5-order input array
        REAL		PP( IM, JM, KM )	!  MM5-order input array
        REAL		PSTAR( IM, JM )		!  MM5-order pstar array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
	REAL		TWSFC( NCOLS, NROWS )
	REAL		TWBAR( NCOLS, NROWS )
	REAL		LFRAC( NCOLS, NROWS )
	INTEGER		LMASK( NCOLS, NROWS )
	REAL		QRTOT( NCOLS, NROWS )
	REAL		QITOT( NCOLS, NROWS )

C...........   PARAMETERs

        REAL            ICE_T, RAIN_T
        INTEGER         MAXC
        PARAMETER     ( ICE_T  = 9.0E-5 ,
     &                  RAIN_T = 9.0E-7 ,
     &                  MAXC    = 512 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        REAL            DLAY, DFRZ
        REAL            RL, RI, T, TW

#ifdef LANGUAGE_FORTRAN90
        REAL		DFAC( NCOLS )
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL		DFAC( MAXC )

C***********************************************************************
C   begin body of subroutine  MMXDCPL

        IF ( NCOLS.GT. MAXC ) THEN
            CALL M3EXIT( 'MCPL:MM2MASK', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAXC > NCOLS', 2 )
        END IF
#endif    /* ifndef LANGUAGE_FORTRAN90  */

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2
        DLAY = 1.0 / FLOAT( NEARSFC )
        DFRZ  = 1.0 / ( TKLIQ - TKICE )

!$OMP       PARALLEL DO DEFAULT( NONE ),
!$OMP&                   SHARED( I0, J0, K0, NROWS, NCOLS, PSTAR, TA,
!$OMP&                           QR, QI, NEARSFC, DLAY, DFRZ, TKICE, 
!$OMP&                           TKFRZ, TWSFC, TWBAR, QRTOT, QITOT,
!$OMP&                           LFRAC, LMASK ),
!$OMP&                  PRIVATE( R, L, C, I, J, K, DFAC, T, RL, RI )

        DO  R = 1, NROWS
            I = I0 + R
            K = K0 - 1
            DO  C = 1, NCOLS
                J = J0 + C

                DFAC( C )    = 1.0 / PSTAR( I,J )
                
                T = DFAC( C ) * TA( I,J,K )
                TWSFC( C,R ) = DFAC( C ) * TA( I,J,K )
                TWBAR( C,R ) = TWSFC( C,R )

                RL = QR(I-1,J-1,K ) + QR(I,J-1,K ) + QR(I+1,J-1,K ) +
     &               QR(I-1,J  ,K ) + QR(I,J  ,K ) + QR(I+1,J  ,K ) +
     &               QR(I-1,J+1,K ) + QR(I,J+1,K ) + QR(I+1,J+1,K )

                RI = QI(I-1,J-1,K ) + QI(I,J-1,K ) + QI(I+1,J-1,K ) +
     &               QI(I-1,J  ,K ) + QI(I,J  ,K ) + QI(I+1,J  ,K ) +
     &               QI(I-1,J+1,K ) + QI(I,J+1,K ) + QI(I+1,J+1,K )

                QRTOT( C,R ) = DFAC( C ) * RL
                QITOT( C,R ) = DFAC( C ) * RI
            END DO

            DO  L = 2, NEARSFC
                K = K0 - L
                    DO  C = 1, NCOLS
                    J = J0 + C
                    TWBAR( C,R ) = TWBAR( C,R ) + DFAC(C)*TA( I,J,K )
                END DO
            END DO

            DO  C = 1, NCOLS

                TWBAR( C,R ) = DLAY *   TWBAR( C,R )
                LFRAC( C,R ) = DFRZ * ( TWBAR( C,R ) - TKICE )

                IF ( LFRAC( C,R ) .LT. 0.0 ) THEN
                    IF ( QRTOT( C,R ) .GT. RAIN_T ) THEN
                        LMASK( C,R ) = 1
                    ELSE
                        LMASK( C,R ) = 0
                    END IF
                ELSE IF ( LFRAC( C,R ) .LT. 1.0 ) THEN
                    LMASK( C,R ) = 1
                ELSE IF ( TWSFC( C,R ) .LT. TKFRZ ) THEN
                    LMASK( C,R ) = 1
                ELSE
                    IF ( QITOT( C,R ) .GT. ICE_T ) THEN
                        LMASK( C,R ) = 1
                    ELSE
                        LMASK( C,R ) = 2
                    END IF
                END IF

            END DO

        END DO

        RETURN
        END

