
C Version "%W% %P% %G% %U%

        SUBROUTINE  MM2WST( IM, JM, UST, PBL, MOL,
     &                     NCOLS, NROWS, COL0, ROW0, WST )

C***********************************************************************
C  subroutine body starts at line  61
C
C  FUNCTION:  Extract/reorder/convert WSTAR for the window specified in
C	terms of grid-window bounds to I/O API order from UST and HOL,
C	which are found in standard MM5 subscript order.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        REAL		UST( IM,JM )	!  MM5-order input ustar
        REAL		PBL( IM,JM )	!  MM5-order input pbl
        REAL		MOL( IM,JM )	!  MM5-order input mol
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
	REAL	      	WST( NCOLS, NROWS )	!  output wstar

C...........   PARAMETERS:  Von Karman's constant and its inverse; 1/3
            
        REAL        VKAR, DVKAR, D3
        PARAMETER ( VKAR  = 0.4 , 
     &              DVKAR = 1.0 / VKAR ,
     &              D3    = 1.0 / 3.0 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C      		!  IOAPI col subscript
	INTEGER		R      		!  IOAPI row subscript
	INTEGER		I, J 		!  MM5 subscripts
	INTEGER		I0, J0 		!  MM5 offsets

C***********************************************************************
C   begin body of subroutine  MM2WST

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                IF ( MOL( I,J ) .LT. 0.0 ) THEN
                     WST( C,R ) = UST( I,J )*
     &                     ( -DVKAR*PBL( I,J )*MOL( I,J ) )**D3
                ELSE
                    WST( C,R ) = 1.0E-20
                END IF
11          CONTINUE
22      CONTINUE

        RETURN
        END

