
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMBDPT( IM, JM, KM, TA, QV, PP, PSTAR, SIGH, PTOP,
     &                      NCOLS, NROWS, NLAYS, NTHIK, 
     &                      COL0, ROW0, LAY0, DENS, PRES, TV, RH )

C***********************************************************************
C  subroutine body starts at line  131
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the boudary array
C	for the window specified in terms of grid-window bounds to I/O API
C	order from MM5 TA, QV, Pprime, Pstar, SIGH, and Ptop, where
C       NTHIK > 0 --> external boundary 
C       NTHIK < 0 --> internal boundary
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )        !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )        !  MM5 P-prime
        REAL            PSTAR( IM, JM )         !  MM5 pstar array
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         NTHIK   !  perimeter thickness
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            PRES( * )       !  caller dim( NPERM,NLAYS )
        REAL            DENS( * )       !  caller dim( NPERM,NLAYS )
        REAL            TV  ( * )       !  caller dim( NPERM,NLAYS )
        REAL            RH  ( * )       !  caller dim( NPERM,NLAYS )

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   PARAMETERs

        REAL	    	RDGAS		 !  from param.F, line 1573 & ff
	REAL            CTOK		 !  Celsius-to-Kelvin
        PARAMETER     ( RDGAS =  287.04, !  to preserve MM5 compatibility
     &                  CTOK  = 273.15  )
	       	
        REAL	    	CONVPA
        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( CONVPA = 1.0E3 ,
     &                  ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         C                       !  IOAPI col subscript
        INTEGER         R                       !  IOAPI row subscript
        INTEGER         L                       !  IOAPI lay subscript
        INTEGER         P,Q                     !  IOAPI boundary subscripts
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        INTEGER         NPERM                   !  ioapi dimensions
        INTEGER         R1, R2, R3, R4          !  perimeter-boundary rows
        INTEGER         C1, C2, C3, C4          !  perimeter-boundary cols

C.......   Decoupling-factor, over-dimensioned using the fact that
C.......   NTHIK*NTHIK >= ABS( NTHIK ) to make sure it's big enough

        REAL            DTA, DQV, DPP		!  decoupled TA,QV,PP
        REAL            ES, RHL
        REAL            DFAC( 2*NTHIK*NTHIK*( NCOLS + NROWS
     &                                      + 2*NTHIK*NTHIK ) )

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( PT, DFAC )

	SAVE            PT, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MMBDPT

        NPERM = ABS( 2*NTHIK ) * ( NCOLS + NROWS + 2*NTHIK )

#if __sgi  || __sun || __osf__ || __mips__
	IF ( NPERM .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( PT )
	    PT = MALLOC( 8 * NPERM )
	    IF ( PT .EQ. 0 )
     &           CALL M3EXIT( 'MMBDPT', 0, 0,
     &                       'Memory allocation error', 2 )
	    LSIZE = NPERM
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        IF ( NTHIK .GT. 0 ) THEN        ! external bdy
            R1 = 1 - NTHIK
            R2 = 0
            R3 = NROWS + 1
            R4 = NROWS + NTHIK
            C1 = 1 - NTHIK
            C2 = 0
            C3 = NCOLS + 1
            C4 = NCOLS + NTHIK
        ELSE                            !  else nthik negative:  internal bdy
            R1 = 1
            R2 = NTHIK - 1
            R3 = NROWS - NTHIK + 1
            R4 = NROWS
            C1 = 1
            C2 = NTHIK - 1
            C3 = NCOLS - NTHIK + 1
            C4 = NCOLS
        END IF                          !  if nthik positive or not

        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

C.......   First, compute decoupling-factors

        Q  = 0
           
        DO  22  R = R1, R2          !  south component
            I = I0 + R
            DO  11  C = 1, C4
                J = J0 + C
                Q = Q  + 1
                DFAC( Q ) = 1.0 / PSTAR( I,J )
11          CONTINUE
22      CONTINUE                    !  end south

        DO  44  R = 1, R4           !  east component
            I = I0 + R
            DO  33  C = C3, C4
                J = J0 + C
                Q = Q  + 1
                DFAC( Q ) = 1.0 / PSTAR( I,J )
33          CONTINUE
44      CONTINUE                    !  end east

        DO  66  R = R3, R4          !  north component
            I = I0 + R
            DO  55  C = C1, NCOLS
                J = J0 + C
                Q = Q  + 1
                DFAC( Q ) = 1.0 / PSTAR( I,J )
55          CONTINUE
66      CONTINUE                    !  end north

        DO  88  R = R1, NROWS       !  west component
            I = I0 + R
            DO  77  C = C1, C2
                J = J0 + C
                Q = Q  + 1
                DFAC( Q ) = 1.0 / PSTAR( I,J )
77          CONTINUE
88      CONTINUE                    !  end west


C...........   Extract and apply decoupling-factors

        P  = 0
        K0 = KM - LAY0 + 2

        DO  199  L = 1, NLAYS

            K = K0 - L
            Q = 0

            DO  122  R = R1, R2          !  south component
                I = I0 + R
                DO  111  C = 1, C4
                    J = J0 + C
                    P = P  + 1
                    Q = Q  + 1
                    DTA = DFAC( Q ) * TA( I,J,K )	!  decoupled
                    DQV = DFAC( Q ) * QV( I,J,K )	!  decoupled
                    DPP = DFAC( Q ) * PP( I,J,K )	!  decoupled
                    TV  ( P ) = DTA * ( 1.0 + 0.608 * DQV )
                    PRES( P ) = CONVPA*( PSTAR( I,J )*SIGH( K ) + PTOP ) + DPP
                    DENS( P ) = PRES( P ) / ( RDGAS * TV( P ) )
                    IF ( DTA .GE. CTOK ) THEN           !  water is liquid
                        ES = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                 / ( DTA - DLIQ ) )
                    ELSE                                !  else below freezing
                        ES = AICE * EXP( ( BICE * DTA - CICE )
     &                                 / ( DTA - DICE ) )
                    END IF                              !  if liquid or frozen
                    RHL = F622 * DQV * ( PRES( P ) - ES ) / ES
                    RH( P ) = MIN( MAX( RHL, 0.0 ), 100.0 )
111            CONTINUE
122         CONTINUE                    !  end south

            DO  144  R = 1, R4           !  east component
                I = I0 + R
                DO  133  C = C3, C4
                    J = J0 + C
                    P = P  + 1
                    Q = Q  + 1
                    DTA = DFAC( Q ) * TA( I,J,K )	!  decoupled
                    DQV = DFAC( Q ) * QV( I,J,K )	!  decoupled
                    DPP = DFAC( Q ) * PP( I,J,K )	!  decoupled
                    TV  ( P ) = DTA * ( 1.0 + 0.608 * DQV )
                    PRES( P ) = CONVPA*( PSTAR( I,J )*SIGH( K ) + PTOP ) + DPP
                    DENS( P ) = PRES( P ) / ( RDGAS * TV( P ) )
                    IF ( DTA .GE. CTOK ) THEN           !  water is liquid
                        ES = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                 / ( DTA - DLIQ ) )
                    ELSE                                !  else below freezing
                        ES = AICE * EXP( ( BICE * DTA - CICE )
     &                                 / ( DTA - DICE ) )
                    END IF                              !  if liquid or frozen
                    RHL = F622 * DQV * ( PRES( P ) - ES ) / ES
                    RH( P ) = MIN( MAX( RHL, 0.0 ), 100.0 )
133             CONTINUE
144         CONTINUE                    !  end east

            DO  166  R = R3, R4          !  north component
                I = I0 + R
                DO  155  C = C1, NCOLS
                    J = J0 + C
                    P = P  + 1
                    Q = Q  + 1
                    DTA = DFAC( Q ) * TA( I,J,K )	!  decoupled
                    DQV = DFAC( Q ) * QV( I,J,K )	!  decoupled
                    DPP = DFAC( Q ) * PP( I,J,K )	!  decoupled
                    TV  ( P ) = DTA * ( 1.0 + 0.608 * DQV )
                    PRES( P ) = CONVPA*( PSTAR( I,J )*SIGH( K ) + PTOP ) + DPP
                    DENS( P ) = PRES( P ) / ( RDGAS * TV( P ) )
                    IF ( DTA .GE. CTOK ) THEN           !  water is liquid
                        ES = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                 / ( DTA - DLIQ ) )
                    ELSE                                !  else below freezing
                        ES = AICE * EXP( ( BICE * DTA - CICE )
     &                                 / ( DTA - DICE ) )
                    END IF                              !  if liquid or frozen
                    RHL = F622 * DQV * ( PRES( P ) - ES ) / ES
                    RH( P ) = MIN( MAX( RHL, 0.0 ), 100.0 )
155             CONTINUE
166         CONTINUE                    !  end north

            DO  188  R = R1, NROWS          !  west component
                I = I0 + R
                DO  177  C = C1, C2
                    J = J0 + C
                    P = P  + 1
                    Q = Q  + 1
                    DTA = DFAC( Q ) * TA( I,J,K )	!  decoupled
                    DQV = DFAC( Q ) * QV( I,J,K )	!  decoupled
                    DPP = DFAC( Q ) * PP( I,J,K )	!  decoupled
                    TV  ( P ) = DTA * ( 1.0 + 0.608 * DQV )
                    PRES( P ) = CONVPA*( PSTAR( I,J )*SIGH( K ) + PTOP ) + DPP
                    DENS( P ) = PRES( P ) / ( RDGAS * TV( P ) )
                    IF ( DTA .GE. CTOK ) THEN           !  water is liquid
                        ES = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                 / ( DTA - DLIQ ) )
                    ELSE                                !  else below freezing
                        ES = AICE * EXP( ( BICE * DTA - CICE )
     &                                 / ( DTA - DICE ) )
                    END IF                              !  if liquid or frozen
                    RHL = F622 * DQV * ( PRES( P ) - ES ) / ES
                    RH( P ) = MIN( MAX( RHL, 0.0 ), 100.0 )
177             CONTINUE
188         CONTINUE                    !  end west

199     CONTINUE                        !  end loop on layers

        RETURN
        END

