
C Version "%W% %P% %G% %U%

        SUBROUTINE  MMBPS0( IM, JM, KM, PS0, PTOP,
     &                      NCOLS, NROWS, NLAYS, NTHIK,
     &                      COL0, ROW0, LAY0, AOUT )

C***********************************************************************
C  subroutine body starts at line  70
C
C  FUNCTION:  Extract/reorder/compute PSTAR0 (reference state PSTAR)
C       in terms of grid-window bounds to I/O API order from ref. sfc. pres.
C       PS0, which is found in standard MM5 subscript order, and model
C       top pressure PTOP.
C       NTHIK > 0 --> external boundary 
C       NTHIK < 0 --> internal boundary
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  NOTE also that we have to convert pressures from MM5 centibars
C	to Pascals:  conversion factor is 1000.0
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for PS0
        INTEGER         JM      !  J (east-west)   dimension for PS0
        INTEGER         KM      !  K (top-bottom)  dimension for PS0
        REAL            PS0( IM, JM, KM )        !  MM5-order input array
        REAL            PTOP    !  model-top pressure
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         NTHIK   !  perimeter thickness
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL            AOUT( * )       !  caller dim( NPERM,NLAYS )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         C                       !  IOAPI col subscript
        INTEGER         R                       !  IOAPI row subscript
        INTEGER         L                       !  IOAPI lay subscript
        INTEGER         P                       !  IOAPI boundary subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        INTEGER         NPERM      		!  ioapi dimensions
        INTEGER         R1, R2, R3, R4          !  perimeter-boundary rows
        INTEGER         C1, C2, C3, C4          !  perimeter-boundary cols

C***********************************************************************
C   begin body of subroutine  MMBPS0

        NPERM = ABS( 2*NTHIK ) * ( NCOLS + NROWS + 2*NTHIK )

        IF ( NTHIK .GT. 0 ) THEN        ! external bdy
            R1 = 1 - NTHIK
            R2 = 0
            R3 = NROWS + 1
            R4 = NROWS + NTHIK
            C1 = 1 - NTHIK
            C2 = 0
            C3 = NCOLS + 1
            C4 = NCOLS + NTHIK
        ELSE                            !  else nthik negative:  internal bdy
            R1 = 1
            R2 = NTHIK - 1
            R3 = NROWS - NTHIK + 1
            R4 = NROWS
            C1 = 1
            C2 = NTHIK - 1
            C3 = NCOLS - NTHIK + 1
            C4 = NCOLS
        END IF                          !  if nthik positive or not

        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM   - LAY0 + 2
        P  = 0

        DO  99  L = 1, NLAYS

            K = K0 - L

            DO  22  R = R1, R2          !  south component
                I = I0 + R
                DO  11  C = 1, C4
                    J = J0 + C
                    P = P + 1
                    AOUT( P ) = 1000.0 * ( PS0( I, J, K ) - PTOP )
11              CONTINUE
22          CONTINUE                    !  end south

            DO  44  R = 1, R4           !  east component
                I = I0 + R
                DO  33  C = C3, C4
                    J = J0 + C
                    P = P + 1
                    AOUT( P ) = 1000.0 * ( PS0( I, J, K ) - PTOP )
33              CONTINUE
44          CONTINUE                    !  end east

            DO  66  R = R3, R4          !  north component
                I = I0 + R
                DO  55  C = C1, NCOLS
                    J = J0 + C
                    P = P + 1
                    AOUT( P ) = 1000.0 * ( PS0( I, J, K ) - PTOP )
55              CONTINUE
66          CONTINUE                    !  end north

            DO  88  R = R1, NROWS          !  west component
                I = I0 + R
                DO  77  C = C1, C2
                    J = J0 + C
                    P = P + 1
                    AOUT( P ) = 1000.0 * ( PS0( I, J, K ) - PTOP )
77              CONTINUE
88          CONTINUE                    !  end west

99      CONTINUE                        !  end loop on layers

        RETURN
        END

