
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.
C Copyright (C) 1998-2002 MCNC and Carlie J. Coats, Jr., and
C (C) 2003 Baron Advanced Meteorological Systems
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMPSDVM( DX, IM, JM, KM, UM, VM, PST, FM,
     &                       MSFX, MSFD, LEVS, COEF,
     &                       NCOLS, NROWS, NLAYS,
     &                       COL0, ROW0, LAY0, 
     &                       SPD, DIV, AVOR, RVOR, UWIND, VWIND )

C***********************************************************************
C  subroutine body starts at line  123
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-pt wind components, 
C       wind-speed, divergence and vorticity sub-arrays specified in terms 
C       of grid-window bounds to I/O API order from U and V dot-pt wind 
C       component arrays UM and VM and PSTAR-array PM found in standard 
C       MM5 subscript order.  
C       Returns zero-values when interpolation is out of range (i.e., when
C       requested pressure surface is below terrain).
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       Specified output window fits inside the current MM5 nest.
C       Interpolation index PDEX, coefficients COEF from MMX2P
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype MMPSDV Jan. 26, 1999 by Carlie J. Coats, Jr., MCNC
C
C       Revised Aug. 29, 2002 by CJC, MCNC: F90 alternative to MALLOC()
C
C       Adapted from MMPSDV May 26, 2003 by CJC, BAMS:  corrected speed,
C        derivative formulas using map-factors
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

#include "parame.incl"

C...........   ARGUMENTS and their descriptions:

        REAL            DX      !  cell size
        INTEGER         IM      !  I (north-south)       dimension for UM,VM
        INTEGER         JM      !  J (east-west)         dimension for UM,VM
        INTEGER         KM      !  K (top-bottom, sigma) dimension for UM,VM
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  " (pressure-level) dimension
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL            UM( IM, JM, KM )  !  MM5-order coupled wind component
        REAL            VM( IM, JM, KM )  !  MM5-order coupled wind component
        REAL            PST( IM, JM )     !  MM5-order PSTAR array (CB)
        REAL            FM( IM, JM )       !  MM5-order Coriolis component (1/s)
        REAL            MSFX( IM, JM )          !  X-point inverse-MSF array
        REAL            MSFD( IM, JM )          !  D-point inverse-MSF array
        INTEGER         LEVS( NCOLS, NROWS, NLAYS )  !  dot-pt interp 
        REAL            COEF( NCOLS, NROWS, NLAYS )  !  index & coeffs
        REAL            SPD  ( NCOLS, NROWS, NLAYS )    ! wind speed (m/s)
        REAL            DIV  ( NCOLS, NROWS, NLAYS )	! divergence (1/s)
        REAL            AVOR ( NCOLS, NROWS, NLAYS )	! abs vorticity  (1/s)
        REAL            RVOR ( NCOLS, NROWS, NLAYS )	! rel vorticity  (1/s)
        REAL            UWIND( NCOLS, NROWS, NLAYS )    ! (m/s)
        REAL            VWIND( NCOLS, NROWS, NLAYS )    ! (m/s)

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L, P		!  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets

#ifdef LANGUAGE_FORTRAN90
        REAL            D( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            M( NCOLS+1, NROWS+1 )   !  derivative factor
        REAL            S( NCOLS+1, NROWS+1 )	!  coupled dot-pt spd**2
        REAL            U( NCOLS+1, NROWS+1 )   !  decoupled UM
        REAL            V( NCOLS+1, NROWS+1 )   !  decoupled VM
        REAL            SS( NCOLS, NROWS, KM )  ! wind speed (m/s)
        REAL            DD( NCOLS, NROWS, KM )	! divergence (1/s)
        REAL            VV( NCOLS, NROWS, KM )	! vorticity  (1/s)
        REAL            UW( NCOLS, NROWS, KM )  ! (m/s)
        REAL            VW( NCOLS, NROWS, KM )  ! (m/s)
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL            D( MJX ,MIX )           !  decouple factor
        REAL            M( MJX, MIX )           !  derivative factor
        REAL            S( MJX ,MIX )	        !  coupled dot-pt spd**2
        REAL            U( MJX ,MIX )           !  decoupled UM
        REAL            V( MJX ,MIX )           !  decoupled VM
        REAL            SS( NCOLS, NROWS, KM )  ! wind speed (m/s)
        REAL            DD( MJX ,MIX, KM )	! divergence (1/s)
        REAL            VV( MJX ,MIX, KM )	! vorticity  (1/s)
        REAL            UW( MJX ,MIX, KM )      ! (m/s)
        REAL            VW( MJX ,MIX, KM )      ! (m/s)
#endif    /* ifndef LANGUAGE_FORTRAN90  */

        REAL            DUDX
        REAL            DUDY
        REAL            DVDX
        REAL            DVDY
        REAL            FAC, AA, BB


C***********************************************************************
C   begin body of subroutine  MM2SPD

        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        FAC  = 0.5 / DX

        DO  R = 1, NROWS + 1    !  decoupling-factor, map factor
        DO  C = 1, NCOLS + 1
            I = I0 + R
            J = J0 + C
            D( C,R ) = 4.0 / ( PST( I-1,J-1 )
     &                       + PST( I  ,J-1 )
     &                       + PST( I-1,J   )
     &                       + PST( I  ,J   ) )
            M( C,R ) = FAC / MSFX(I,J)**2
        END DO
        END DO

        K0  = KM - LAY0 + 2
                                             
        DO  77  L = 1, KM       !  loop over sigma-levels

            K = K0 - L

            ! compute decoupled, pressure-interpolated dot point 
            ! wind component and speed**2

            DO  R = 1, NROWS + 1    !  compute dot-point decoupled winds
            DO  C = 1, NCOLS + 1
                I = I0 + R
                J = J0 + C
                U( C,R ) =  D( C,R )*UM( I,J,K )  
                V( C,R ) =  D( C,R )*VM( I,J,K )   
                S( C,R ) =  U( C,R )**2 + V( C,R )**2
            END DO
            END DO

            DO  R = 1, NROWS
            DO  C = 1, NCOLS
                SS( C,R,L ) = 0.5 * SQRT( S( C  , R   )
     &                                  + S( C+1, R   )
     &                                  + S( C,   R+1 )
     &                                  + S( C+1, R+1 ) )
                UW( C,R,L ) = 0.25*( U( C  , R   ) + U( C+1, R   )
     &                             + U( C,   R+1 ) + U( C+1, R+1 ) )
                VW( C,R,L ) = 0.25*( V( C  , R   ) + V( C+1, R   )
     &                             + V( C,   R+1 ) + V( C+1, R+1 ) )
            END DO
            END DO

            DO  R = 1, NROWS + 1    !  re-scale by map-factors
            DO  C = 1, NCOLS + 1
                I = I0 + R
                J = J0 + C
                U( C,R ) =  MSFD( I,J ) * U( C,R )
                V( C,R ) =  MSFD( I,J ) * V( C,R )
            END DO
            END DO

            !  Note that for cross-point derivatives, we have
            !  two versions of dot-point difference calculations
            !  (e.g., du/dx at rows R,R+1) which we average
            !  to get the requested cross-point derivative
                
            DO  R = 1, NROWS  !  cross-point calculations
            DO  C = 1, NCOLS
                DUDX = M( C,R ) * ( U( C+1,R   ) - U( C,R   ) 
     &                            + U( C+1,R+1 ) - U( C,R+1 ) )
                DVDX = M( C,R ) * ( V( C+1,R   ) - V( C,R   ) 
     &                            + V( C+1,R+1 ) - V( C,R+1 ) ) 
                DUDY = M( C,R ) * ( U( C+1,R+1 ) - U( C+1,R ) 
     &                            + U( C  ,R+1 ) - U( C,R   ) ) 
                DVDY = M( C,R ) * ( V( C+1,R+1 ) - V( C+1,R ) 
     &                            + V( C  ,R+1 ) - V( C,R   ) ) 

                DD( C,R,L ) = DUDX + DVDY
                VV( C,R,L ) = DVDX - DUDY

            END DO
            END DO

77      CONTINUE                !  end loop on sigma-layers L

        !  loop: interpolate all these to pressure surfaces

        DO  L = 1, NLAYS
        DO  R = 1, NROWS
        DO  C = 1, NCOLS

            I = I0 + R
            J = J0 + C

            IF ( COEF( C,R,L ) .LE. 1.0 ) THEN
                P  = LEVS( C,R,L )
                AA = COEF( C,R,L )
                BB = 1.0 - AA
                DIV  ( C,R,L ) = AA * DD( C,R,P ) + BB * DD( C,R,P+1 )
                RVOR ( C,R,L ) = AA * VV( C,R,P ) + BB * VV( C,R,P+1 )
                AVOR ( C,R,L ) = RVOR ( C,R,L ) + FM( I,J )
                SPD  ( C,R,L ) = AA * SS( C,R,P ) + BB * SS( C,R,P+1 )
                UWIND( C,R,L ) = AA * UW( C,R,P ) + BB * UW( C,R,P+1 )
                VWIND( C,R,L ) = AA * VW( C,R,P ) + BB * VW( C,R,P+1 )
            ELSE
                DIV  ( C,R,L ) = 0.0
                RVOR ( C,R,L ) = 0.0
                AVOR ( C,R,L ) = FM( I,J )
                SPD  ( C,R,L ) = 0.0
                UWIND( C,R,L ) = 0.0
                VWIND( C,R,L ) = 0.0
            END IF

        END DO
        END DO
        END DO          !  end loop interpolating to pressure surfaces

        RETURN
        END

