
C Version "%W% %P% %G% %U%

        SUBROUTINE  MMPW( IM, JM, KM, 
     &                    PP, WA, PST, SIGF, PTOP, PLEV, 
     &                    NCOLS, NROWS, NLAYS,
     &                    COL0, ROW0, LAY0, W )

C***********************************************************************
C  subroutine body starts at line  84
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	pressure-surface vertical wind components 
C       for the window specified in terms of grid-window bounds to I/O API 
C       order, using Pprime, Pstar, SIGF, and Ptop from MM5
C       Returns zero-values when interpolation is out of range (i.e., 
C       when requested pressure surface is below terrain).
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  Feb 09, 1999 by Carlie J. Coats, Jr., MCNC
C       Validated 2/16/99 CJC, JNM
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            PP( IM, JM, KM )        !  MM5 P-prime (CB-Pa)
        REAL            WA( IM, JM, KM )        !  MM5 coupled W wind (CB*m/s)
        REAL            PST( IM, JM )           !  MM5 pstar array (CB)
        REAL            SIGF( KM )              !  MM5 SIGMA-F
        REAL            PTOP			!  MM5 ptop (CB) 
        REAL            PLEV( NLAYS )           !  requested pressure levels (Pa)
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            W( NCOLS, NROWS, NLAYS ) !  W-wind

C...........   PARAMETERs

        REAL	    	CBTOPA
        INTEGER         MAXK
        PARAMETER     ( CBTOPA =   1.0E3 ,
     &                  MAXK   = 200 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         M, N                    !  loop counters
        INTEGER         I0, J0, K0              !  MM5 offsets
        
        REAL            DPP		        !  decoupled PP
        REAL            DFAC                    !  1/pstar 
        REAL            PRESF( MAXK )           !  pressure    on SIGF (Pa)
        REAL            WF   ( MAXK )           !  decoupled W on SIGF (m/s)
        REAL            AA, BB

C***********************************************************************
C   begin body of subroutine  MM2DPT

        IF ( KM .GT. MAXK ) THEN
            CALL M3EXIT( 'MCPL:MMPW', 0, 0, 
     &      'MAXK too small; increase and recompile', 2 )
        END IF

        I0 = ROW0 - 1                   !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  33  R = 1, NROWS
            I = I0 + R
            DO  22  C = 1, NCOLS
                J = J0 + C
                DFAC = 1.0 / PST( I,J )
                
                DO  L = 1, KM   ! decouple WA and invert subscripts to i/o api 
				! decouple Prime/compute press in inverted 
			        ! subscript order
                    K = K0 - L
                    WF(L)    = DFAC * WA( I,J,K )     !  WWIND, m/s
                    DPP      = DFAC * PP( I,J,K )     !  PPRIME, Pa
                    PRESF(L) = CBTOPA*( PST(I,J)*SIGF(K) + PTOP ) + DPP
                END DO
                
                N = 1
                DO  L = 1, NLAYS        !  loop on output pressure layers

                    IF ( PLEV( L ) .GT. PRESF( 1 ) ) THEN  !  below terrain

                        W( C,R,L ) = 0.0

                    ELSE    !  above terrain:  find PLEV and interpolate W

                        DO  M = N+1, KM !  search remaining i/o api levels

                            IF ( PLEV( L ) .GT. PRESF( M ) ) THEN

                                !  found plev(L)

                                AA = ( PLEV(L)    - PRESF(M) )/ 
     &                               ( PRESF(M-1) - PRESF(M) )
                                BB = 1.0 - AA
                                W( C,R,L ) = AA*WF( M-1 ) + BB*WF( M )
                                N = M-1
                                GO TO  11       !  done with this (C,R,L)

                            END IF

                        END DO          !  end loop on remaining i/o api levels

                        !  if you get to here:  never found PLEV(L); error

                        CALL M3EXIT( 'MCPL:MMPW', 0, 0, 
     &                               'PLEV not found', 2 )

                    END IF          !  if below terrain or not

11                  CONTINUE        !  target of "found plev(L)"

                END DO  !  end loop on output layers L

22          CONTINUE    !  end loop on output cols C
33      CONTINUE        !  end loop on output rows R

        RETURN
        END
