
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMPZ( IM, JM, KM, 
     &              PP, PSTAR, SIGH, PTOP, P00, TLP, TS0, ROVG, 
     &              PVAL, SLPRES, LEVS, COEF,  
     &              NCOLS, NROWS, NLAYS,
     &              COL0, ROW0, LAY0, Z, 
     &              DZ1000_850, DZ1000_500, DZ850_700 )

C***********************************************************************
C  subroutine body starts at line  122
C
C  FUNCTION:  Compute the cross point pressure-level Z (height above
C        sea level) on the requested pressure-levels for the window 
C        specified in terms of grid-window bounds (in I/O API order)
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  Feb 11, 1999 by Carlie J. Coats, Jr., and
C       John N. McHenry, MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        REAL            PP( IM, JM, KM )        !  MM5 coupled P-prime (CB-Pa)
        REAL            PSTAR( IM, JM ) !  MM5 pstar array
        REAL            SIGH( KM )      !  MM5 SIGMA-H
        REAL            PTOP		!  MM5 ptop (cb)
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            TLP		!  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		!  MM5 ref SL temp (K)
        REAL            ROVG		!  MM5 R/G
        REAL            SLPRES( NCOLS, NROWS )       !  sea level press (mb)
        INTEGER         LEVS( NCOLS, NROWS, NLAYS )  !  interp index
        REAL            COEF( NCOLS, NROWS, NLAYS )  !  interp coefficients
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            PVAL( NLAYS )             !  req. pressure levels
        REAL            Z( NCOLS, NROWS, NLAYS )  !  altitude above terrain
        REAL            DZ1000_850( NCOLS, NROWS )!  difference in Z's
        REAL            DZ1000_500( NCOLS, NROWS )!  difference in Z's
        REAL            DZ850_700 ( NCOLS, NROWS )!  difference in Z's

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   PARAMETERs

        REAL	    	CBTOPA        !  conversion factor, Pa to cb
        REAL	    	MBTOPA        !  conversion factor, Pa to mb
        PARAMETER     ( CBTOPA = 1.0E3, 
     &                  MBTOPA = 1.0E2  )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L      !  IOAPI col subscript
        INTEGER         I, J, K      !  MM5 subscripts
        INTEGER         I0, J0, K0   !  MM5 offsets
        REAL            DP0          !  CBTOPA/p0
        REAL            P0DIF        !  (P00 in CB) - PTOP
        REAL            ALP, SIG, AA, BB
        REAL            DFAC( NCOLS, NROWS )    ! 1/pstar decoupling factor
        REAL            PRS1( NCOLS, NROWS )    ! press at lowest half-level
        REAL            SIG0( NCOLS, NROWS )    ! sigma at sea level
        REAL            PRS0( NCOLS, NROWS )    ! sea level pressure (Pa)
        CHARACTER*120   MESG

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( P, DFAC )
	POINTER         ( Q, PRS1 )
	POINTER         ( RR, SIG0 )
	POINTER         ( SS, PRS0 )

	SAVE            P, Q, RR, SS, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C...........   SAVEd VARIABLES and their descriptions:

        LOGICAL FIRSTIME
        DATA    FIRSTIME / .TRUE. /
        INTEGER L1000, L850, L700, L500
        DATA    L1000, L850, L700, L500 / 4 * 0 /

        SAVE    FIRSTIME, L1000, L850, L700, L500

C***********************************************************************
C   begin body of subroutine  MM2DPT

#if __sgi  || __sun || __osf__ || __mips__
	IF ( NCOLS*NROWS .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( P )
	    P = MALLOC( 16 * NCOLS*NROWS )
            Q = P + 4 * NCOLS*NROWS
            RR= Q + 4 * NCOLS*NROWS
            SS= RR+ 4 * NCOLS*NROWS
	    IF ( P .EQ. 0 )
     &           CALL M3EXIT( 'MMPZ', 0, 0,
     &                       'Memory allocation error', 2 )
	    LSIZE = NCOLS*NROWS
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        IF ( FIRSTIME ) THEN
            FIRSTIME = .FALSE.
            DO  L = 1, NLAYS
                IF      ( ABS( PVAL( L )  - 1000.0E2 ) .LT. 1.0 ) THEN
                    L1000 = L
                ELSE IF ( ABS( PVAL( L )  -  850.0E2 ) .LT. 1.0 ) THEN
                    L850  = L
                ELSE IF ( ABS( PVAL( L )  -  700.0E2 ) .LT. 1.0 ) THEN
                    L700  = L
                ELSE IF ( ABS( PVAL( L )  -  500.0E2 ) .LT. 1.0 ) THEN
                    L500  = L
                END IF
            END DO
            IF ( L1000 * L850 * L700 * L500 .EQ. 0 ) THEN
                CALL M3MSG2( 'Model SIGMA-levels' )
                DO  L = 1, NLAYS
                    WRITE( MESG, '( A, I2, A, 1PG13.6, A )' )
     &                       'SIGMA(', L, ')=', PVAL( L ), 'Pa' 
                    CALL M3MSG2( MESG )
                END DO
                CALL M3EXIT( 'MMPZ', 0, 0,
     &                       'Could not find pressure levels for ' // 
     &                       '1000,850,700,500 mb', 2 )
            END IF
        END IF  !  if firstime:  calculate L1000, L850, L500

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2
        P0DIF = P00 / CBTOPA  -  PTOP

        DO  22  R = 1, NROWS        !!  2D computation of "decoupling factor"
            I = I0 + R              !!  DFAC, pressure PRS1 at lowest model
            DO  11  C = 1, NCOLS    !!  level, sigma-value SIG0 at sea level,
                J = J0 + C          !!  and convert sea lvl press PRS0 to Pa

                DFAC( C,R ) = 1.0 / PSTAR( I,J )
                PRS1( C,R ) = CBTOPA*( PSTAR( I,J )*SIGH( KM ) + PTOP )
     &                        + PP( I,J,KM ) * DFAC( C,R )
                SIG0( C,R ) = P0DIF * DFAC( C,R )
                PRS0( C,R ) = MBTOPA * SLPRES( C,R )      !  convert mb to Pa

11          CONTINUE
22      CONTINUE

        DP0 = CBTOPA / P00

        DO  55  L = 1, NLAYS            !  upper-layer processing
            DO  44  R = 1, NROWS
                I = I0 + R
                DO  33  C = 1, NCOLS
                    J   = J0 + C

                    !!  ALGORITHM:  If requested pressure is greater
                    !!  than lowest model-level pressure (i.e., if 
                    !!  altitude of pressure surface is below lowest 
                    !!  SIGH-surface), interpolate/extrapolate SIGMA 
                    !!  to requested pressure-surface using sea-level 
                    !!  surface and lowest SIGH-surface;  
                    !!  else interpolate SIGMA between model SIGH-surfaces.
                    !!  In both cases, interpolation is linear-in-pressure.
                    !! 
                    !!  Having calculated SIGMA, compute the corresponding
                    !!  reference pressure, and its logarithm ALP.
                    !!  Using this ALP, reference surface temperature TS0,
                    !!  and reference temperature lapse rate TLP, solve
                    !!  the reference-atmosphere equation for Z
                    !!
                    !!  NOTE:  this relies on the numerically-stable
                    !!  computation of sea level pressure by MM2SLP;
                    !!  extrapolation to pressure levels below the model
                    !!  from nonhydrostatic pressure velues on the lowest
                    !!  two model levels turns out to be an ill-conditioned
                    !!  computation.

                    IF ( PVAL( L ) .GT. PRS1( C,R ) ) THEN
                        AA  =  ( PVAL( L )   -  PRS1( C,R ) ) 
     &                        /( PRS0( C,R ) -  PRS1( C,R ) )
                        BB  = 1.0 - AA
                        SIG = AA*SIG0( C,R ) + BB*SIGH( KM )
                    ELSE
                        K   = K0 - LEVS( C,R,L )
                        AA  = COEF( C,R,L )
                        BB  = 1.0 - AA
                        SIG = AA*SIGH( K ) + BB*SIGH( K-1 )
                    END IF

                    ALP = ALOG( ( DP0*( SIG*PSTAR( I,J ) + PTOP ) ) )
                    Z( C,R,L ) = -ROVG*ALP*( 0.5*TLP*ALP + TS0 ) 

33              CONTINUE
44          CONTINUE
55      CONTINUE

        DO  77  R = 1, NROWS    !!  2D computation of thicknesses
        DO  66  C = 1, NCOLS    !!  between pressure surfaces

            DZ1000_850( C,R ) = Z( C,R,L850 ) - Z( C,R,L1000 )
            DZ1000_500( C,R ) = Z( C,R,L500 ) - Z( C,R,L1000 )
            DZ850_700 ( C,R ) = Z( C,R,L700 ) - Z( C,R,L850  )

66      CONTINUE
77      CONTINUE

        RETURN
        END

