
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMXUSDVV( FACTOR, DX, IM, JM, KM, UM, VM, PM, FM,
     &                        NCOLS, NROWS, NLAYS,
     &                        COL0, ROW0, LAY0, 
     &                        UX, VX, SPD, DIV, AVOR, RVOR )

C***********************************************************************
C  subroutine body starts at line  113
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-point wind
C       components, wind-speed, divergence and vorticity sub-arrays
C       specified in terms of grid-window bounds to I/O API order
C       from U and V wind component arrays UM and VM and 
C       PSTAR-array PM found in standard MM5 subscript order.
C       Computes both relative vorticity RVOR (with respect to
C       Cartesian wind components only) and also absolute vorticity AVOR
C       which also has a coriolis term.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       starting and ending <column,row,level> ranges <CA,RA,LA> and
C       <CZ,RZ,LZ> are in-range for input array AM.
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C       Adapted 4/2001 by CJC from MMXUSDV() to output both 
C       relative and absolute vorticity
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL            FACTOR  !  conversion factor
        REAL            DX      !  cell size
        INTEGER         IM      !  I (north-south) dimension for UM,VM
        INTEGER         JM      !  J (east-west)   dimension for UM,VM
        INTEGER         KM      !  K (top-bottom)  dimension for UM,VM
        REAL            UM( IM, JM, KM )        !  MM5-order input array
        REAL            VM( IM, JM, KM )        !  MM5-order input array
        REAL            PM( IM, JM )            !  MM5-order PSTAR array
        REAL            FM( IM, JM )            !  MM5-order F array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL            UX( NCOLS, NROWS, NLAYS )
        REAL            VX( NCOLS, NROWS, NLAYS )
        REAL            SPD( NCOLS, NROWS, NLAYS )
        REAL            DIV( NCOLS, NROWS, NLAYS )	!  divergence
        REAL           AVOR( NCOLS, NROWS, NLAYS )	!  abs vorticity
        REAL           RVOR( NCOLS, NROWS, NLAYS )	!  rel vorticity

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
	INTEGER		N			!  single-indexing subscript

        REAL            D( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            S( NCOLS+1, NROWS+1 )	!  coupled dot-pt spd**2
        REAL            U( NCOLS+1, NROWS+1 )   !  decoupled UM
        REAL            V( NCOLS+1, NROWS+1 )   !  decoupled VM
        REAL            DUDX
        REAL            DUDY
        REAL            DVDX
        REAL            DVDY
        REAL            FAC, RFAC

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( PD, D )
	POINTER         ( PS, S )
	POINTER         ( PU, U )
	POINTER         ( PV, V )

	SAVE            PD, PS, PU, PV, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MM2SPD


#if __sgi  || __sun || __osf__ || __mips__
!$OMP   CRITICAL( MCPL_MMXUSDV )
	N = ( NCOLS + 1 ) * ( NROWS + 1 )
        IF ( N .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 ) THEN  
                CALL FREE( PD )
                CALL FREE( PS )
                CALL FREE( PU )
                CALL FREE( PV )
            END IF
	    PD = MALLOC( 4 * N )
	    PS = MALLOC( 4 * N )
	    PU = MALLOC( 4 * N )
	    PV = MALLOC( 4 * N )
	    IF ( PD .EQ. 0  .OR. 
     &           PS .EQ. 0  .OR. 
     &           PU .EQ. 0  .OR. 
     &           PV .EQ. 0  ) THEN
                CALL M3EXIT( 'MMXUSDVV', 0, 0,
     &                       'Memory allocation error', 2 )
	    END IF
	    LSIZE = N
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS + 1        !  (uncoupling-factor)**2
            I = I0 + R
            DO  11  C = 1, NCOLS + 1
                J = J0 + C
                D( C,R ) = 4.0 / ( PM( I-1,J-1 )
     &                           + PM( I  ,J-1 )
     &                           + PM( I-1,J   )
     &                           + PM( I  ,J   ) )
11          CONTINUE
22      CONTINUE

        RFAC = 0.5 * FACTOR
        FAC  = 0.5 / DX
                                             
        DO  77  L = 1, NLAYS

            K = K0 - L
            DO  44  R = 1, NROWS + 1    !  compute dot-point coupled speed**2
                I = I0 + R
                DO  33  C = 1, NCOLS + 1
                    J = J0 + C
                    U( C,R ) =  D( C,R ) * UM( I,J,K )
                    V( C,R ) =  D( C,R ) * VM( I,J,K )
                    S( C,R ) =  U( C,R )**2 + V( C,R )**2
33              CONTINUE
44          CONTINUE

            DO  66  R = 1, NROWS        !  re-scaled decoupled cross-point speed
            DO  55  C = 1, NCOLS
                I = I0 + R
                J = J0 + C
                DUDX = FAC * ( U( C+1,R   ) - U( C,R   ) 
     &                       + U( C+1,R+1 ) - U( C,R+1 ) )
                DVDX = FAC * ( V( C+1,R   ) - V( C,R   ) 
     &                       + V( C+1,R+1 ) - V( C,R+1 ) ) 
                DUDY = FAC * ( U( C+1,R+1 ) - U( C+1,R ) 
     &                       + U( C  ,R+1 ) - U( C,R   ) ) 
                DVDY = FAC * ( V( C+1,R+1 ) - V( C+1,R ) 
     &                       + V( C  ,R+1 ) - V( C,R   ) ) 
                DIV ( C,R,L ) = DUDX + DVDY
                RVOR( C,R,L ) = DVDX - DUDY
                AVOR( C,R,L ) = RVOR( C,R,L ) + FM( I,J )
                SPD ( C,R,L ) = RFAC * SQRT( ( S( C  , R   ) +
     &                                         S( C+1, R   ) +
     &                                         S( C,   R+1 ) +
     &                                         S( C+1, R+1 ) ) )
                UX( C,R,L ) = 0.25 * ( U( C+1,R   ) + U( C,R   ) 
     &                               + U( C+1,R+1 ) + U( C,R+1 ) )
                VX( C,R,L ) = 0.25 * ( V( C+1,R   ) + V( C,R   ) 
     &                               + V( C+1,R+1 ) + V( C,R+1 ) )
55          CONTINUE
66          CONTINUE

77      CONTINUE

#if __sgi  || __sun || __osf__ || __mips__
!$OMP   END CRITICAL( MCPL_MMXUSDV )
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

        RETURN
        END

