
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 2001 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMXUV10( IM, JM, UM, VM,
     &                       NCOLS, NROWS,
     &                       COL0, ROW0,
     &                       UX, VX, SPD )

C***********************************************************************
C  subroutine body starts at line  67
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-point wind
C       components and wind-speed specified in terms of grid-window 
C       bounds to I/O API order from (two-dimensional not-coupled)
C       10-meter-elevation wind component arrays UM and VM found in
C       standard MM5 subscript order.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas I/O API-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       starting and ending <column,row> ranges <CA,RA> and
C       <CZ,RZ> are in-range for input arrays.
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype Oct. 11, 2001 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for UM,VM
        INTEGER         JM      !  J (east-west)   dimension for UM,VM
        REAL            UM( IM, JM )        !  MM5-order input array
        REAL            VM( IM, JM )        !  MM5-order input array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        REAL            UX( NCOLS, NROWS )
        REAL            VX( NCOLS, NROWS )
        REAL           SPD( NCOLS, NROWS )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R		        !  IOAPI col subscript
        INTEGER         I, J                    !  MM5 subscripts
        INTEGER         I0, J0                  !  MM5 offsets


C***********************************************************************
C   begin body of subroutine  MM2SPD

        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        DO  R = 1, NROWS        !  re-scaled decoupled cross-point speed
            I = I0 + R
            DO  C = 1, NCOLS
                 J = J0 + C
                 UX( C,R ) = 0.25 * ( UM( I+1,J   ) + UM( I,J   ) 
     &                              + UM( I+1,J+1 ) + UM( I,J+1 ) )
                 VX( C,R ) = 0.25 * ( VM( I+1,J   ) + VM( I,J   ) 
     &                              + VM( I+1,J+1 ) + VM( I,J+1 ) )
                SPD( C,R ) =  SQRT( UX( C,R   )**2 + VX( C,R   )**2 )
            END DO
        END DO

        RETURN
        END
