C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1998-2002 MCNC and
C Carlie J. Coats, Jr.
C (C) 2003 Baron Advanced Meteorological Systems
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................
C***********************************************************************
C  SUBROUTINE  MCPL--  time-stepped output        starts at line 2734
C  ENTRY MCPL_GRID --  output grid initialization starts at line 3448
C  ENTRY MCPL_OUT  --  processing & output        starts at line 4921
C  ENTRY MCPL_TIME --  processing & output        starts at line 7379
C  ENTRY MCPL_ATIME -- processing & output        starts at line 7397
C  ENTRY MCPL_HTR   --  processing & output       starts at line 7415
C
C  Copyright 1997-2002 MCNC, Inc. and Carlie J. Coats, Jr. and John McHenry
C  Copyright 2003 Baron Advanced Meteorological Systems
C
C  MCPL is an output module for the MM5 meteorology model that produces
C  output via the EDSS/Models-3 I/O API (itself (C) 1992-2001 MCNC).
C
C    This library is free software; you can redistribute it and/or
C    modify it under the terms of the GNU Lesser General Public
C    License (LGPL) as published by the Free Software Foundation;
C    either version 2 of the License, or (at your option) any later
C    version.
C
C    This library is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
C    Lesser General Public License for more details.
C
C    You should have received a copy of the GNU Lesser General Public
C    License along with this library; if not, write to the Free Software
C    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C    The LGPL can also be found at URL http://www.gnu.org/copyleft/lesser.html
C
C  For further information, contact the authors:
C       Carlie J. Coats, Jr.            carlie.coats@baronams.com
C       John N. McHenry                 john.mchenry@baronams.com
C       Baron Advanced Meteorological Systems, LLC.
C       920 Main Campus Drive, Suite 101
C       Raleigh, NC 27606
C***********************************************************************
C
C  FUNCTION:
C       Accesses internal MM5 data structures to generate
C       MAQSIP-compliant output files
C               GRID_BDY_2D non-layered boundary    time-independent
C               GRID_BDY_3D     layered boundary    time-independent
C               GRID_CRO_2D nonlayered cross-point  time-independent
C               GRID_CRO_3D     layered cross-point time-independent
C               GRID_DOT_2D non-layered dot-point   time-independent
C               MET_BDY_2D  non-layered boundary    meteorology
C               MET_BDY_3D      layered boundary    meteorology
C               MET_KF_SCA       scalar  KF-event    meteorology
C               MET_KF_PRO       profile KF-event    meteorology
C               MET_CRO_2D  non-layered or layer 1 cross-point meteorology
C               MET_CRO_3D      layered cross-point meteorology
C               MET_DOT_3D      layered dot-point   meteorology
C               PRS_CRO_3D      pressure-level cross-point meteorology
C               PRS_DOT_3D      pressure-level   dot-point meteorology
C
C       Performs READ3() on virtual synchronization files when used
C       in coupling mode.
C
C  PRECONDITIONS REQUIRED:
C	Compiler allows taking substrings of CHARACTER PARAMETERs
C       (i.e., Sun "f77 4.0" fails Quality-Of-Implementation test)
C
C	Nonhydrostatic only
C
C       Call     MCPL_GRID once to initialize internal data structures
C                          for all MM5 nests and output grids,
C           then MCPL_OUT  for initialization and once per timestep per nest
C           and  MCPL_KF2  once per KF-event per nest
C
C       setenv MCPL_SYNCH_DELAY  <list of record-offsets to use with
C                          synchronization virtual files, by output grid>
C       setenv MCPL_SYNCH_G0 (etc.) <path>, if this file is being read
C
C       setenv EM_OUTGRIDS <list of MM5-nests within which the output
C                          grids are defined (possibly more than one>
C                          MCPL-output grid per MM5 nest, in increasing order>
C       setenv EM_I0       <list of I-offsets for output grids relative to
C                          MM5-nests -- offset 0 means same starting row>
C       setenv EM_J0       <list of J-offsets for output grids relative to
C                          MM5-nests -- offset 0 means same starting col>
C       setenv EM_NCOLS    <list of output grid column dimensions>
C       setenv EM_NROWS    <list of output grid  row   dimensions>
C       setenv EM_NTHIK    <list of output grid bdy-thickness dimensions>
C       setenv CRDNAM      <coordinate-map-projection name>
C       setenv GRIDDESC    <path for GRIDDESC file>
C       setenv MCPL_SDATE  <starting Julian date, format YYYYDDD>
C       setenv MCPL_STIME  <starting time, format HHMMSS>
C       setenv MCPL_TSTEP  <output time step, format HHMMSS>
C       setenv MCPL_EDATE  <ending Julian date, format YYYYDDD>
C       setenv MCPL_ETIME  <ending Julian time, format HHMMSS>
C       setenv GB2_VARS    <path for GRID_BDY_2D-variables list, or NONE>
C       setenv GB3_VARS    <path for GRID_BDY_3D-variables list, or NONE>
C       setenv GC2_VARS    <path for GRID_CRO_2D-variables list, or NONE>
C       setenv GC3_VARS    <path for GRID_CRO_3D-variables list, or NONE>
C       setenv GD2_VARS    <path for GRID_DOT_2D-variables list, or NONE>
C       setenv MB2_VARS    <path for  MET_BDY_2D-variables list, or NONE>
C       setenv MB3_VARS    <path for  MET_BDY_3D-variables list, or NONE>
C       setenv MC2_VARS    <path for  MET_CRO_2D-variables list, or NONE>
C       setenv MC3_VARS    <path for  MET_CRO_3D-variables list, or NONE>
C       setenv MD3_VARS    <path for  MET_DOT_3D-variables list, or NONE>
C       setenv PC3_VARS    <path for PRES_CRO_3D-variables list, or NONE>
C       setenv PD3_VARS    <path for PRES_DOT_3D-variables list, or NONE>
C       setenv GRID_BDY_2D_G0 (etc) <path>, if this file is being produced
C       setenv GRID_BDY_3D_G0 (etc) <path>, if this file is being produced
C       setenv GRID_CRO_2D_G0 (etc) <path>, if this file is being produced
C       setenv GRID_CRO_3D_G0 (etc) <path>, if this file is being produced
C       setenv GRID_DOT_2D_G0 (etc) <path>, if this file is being produced
C       setenv  MET_BDY_2D_G0 (etc) <path>, if this file is being produced
C       setenv  MET_BDY_3D_G0 (etc) <path>, if this file is being produced
C       setenv  MET_CRO_2D_G0 (etc) <path>, if this file is being produced
C       setenv  MET_CRO_3D_G0 (etc) <path>, if this file is being produced
C       setenv  MET_DOT_3D_G0 (etc) <path>, if this file is being produced
C       setenv PRES_CRO_3D_G0 (etc) <path>, if this file is being produced
C       setenv PRES_DOT_3D_G0 (etc) <path>, if this file is being produced
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C	CKVBLE()    check vble-name vs list & update file header for vble.
C	FMTOPTS()   formats MM5 run-time options to file header
C	GETVBLES()  get list of names (1 per line) from file
C       MM2BDY()    windows to output grid bdy  and re-orders to IOAPI order
C       MM2THERM2() windows/reorder/calculate gridded DENS, PRES, TV
C       MM2THERM3() windows/reorder/calculate gridded DENS, PRES, TV
C       MM2IO()     windows to output grid and re-orders to IOAPI order
C       MM2MOL()    " gridded MOL
C       MM2MSQ()    "  cross-pt map-factor-squared
C	MM2RAD()    "  gridded albedo-corr. radiation
C	MM2REF()    "  reference atmosphere
C	MM2SLP()    "  2D sea-level pressure (old algorithm)
C	MM2BMP()    "  2D sea-level pressure (Benjamin-Miller algorithm)
C	MM2WST()    "  WSTAR
C	MMBDCPL()   "  decoupled boundary vbles
C       MMBTHERMO() "  boundary DENS, PRES, TV, RH
C       MMBMSQ()    "  boundary map-factor-squared
C	MMBREF()    "  boundary reference atmosphere
C	MMD2P()     "  dot-pt pressure-level interpolation coeffs
C	MMDDCPL()   "  decoupled dot-pt gridded vbles
C       MMDMSQ()    "  dot-pt map-factor-squared
C	MMPDDCPL()  "  decoupled pressure-level dot-pt gridded vbles
C	MMPTHERMO() "  cross-pt pressure-level DENS, PRES, TV, RH
C	MMPSDVM()   "  cross-pt pressure-level horiz spd, div, and vor
C	MMPW()      "  cross-pt pressure-level vertical wind component
C	MMPXDCPL()  "  decoupled pressure-level cross-pt gridded vbles
C	MMPZ()      "  cross-pt pressure-level elevation above m.s.l.
C	MMX2D()     "  cross-pt to dot-point interpolation
C	MMX2P()     "  cross-pt pressure-level interpolation coeffs
C	MMXDCPL()   "  cross-pt gridded vbles
C	MMXUSDVM()  "  cross-pt wind-related (div, vor) vbles
C	MMPTHETA()  computes 3D potential temperature from PVALH, 3D T
C
C  REVISION  HISTORY:
C       Prototype 10/17/1997 by Carlie J. Coats, Jr.
C       First Operational Version 6/20/1998 by Carlie Coats & John McHenry
C       Sea-Level Pressure,  01/21/1999 by Carlie Coats & John McHenry
C       PC3, PD3 files,      02/01/1999 by Carlie Coats & John McHenry
C       Interp to Pres Pkg.  02/12/1999 by Carlie Coats & John McHenry
C       KF-file Pkg.         04/15/1999 by Carlie Coats & John McHenry
C       Add optional MM5V3 support (compile with "-DMM5_V3=1")
C       10/18/2000 by Carlie Coats
C       New data structure in KF-file Pkg.
C       12/06/1999 by Carlie Coats & John McHenry
C       Add cross-point wind components and Benjamin-Miller SLP option
C       (compile with "-DMCPL_BMP=1")
C       3/16/2000 by Carlie Coats & John McHenry
C       2/2001 by Carlie J. Coats, Jr., MCNC:  now supports
C           three different versions of relative humidity:
C               RHWI has liquid-water based RH above CTOK, ice-based RH
C               below CTOK
C               RHLW has liquid-water based RH only
C               RHKF uses liquid-water based RH above TTFRZ,  ice-based RH
C               below TBFRZ, and linear mixture of the two in between.
C               (as used in KF cloud parameterization and elsewhere)
C           using new routine MM2THERM2, MM2THERM3, MMBTHERMO, MMPTHERMO
C           to replace MM2DP2, MM2DP3, MMBDPT, MMPDTRH, respectively
C       4/2001 by Carlie J. Coats, Jr., MCNC:  MC2, MC3, and PC3
C               now have both relative vorticity variable "RVOR" and
C               absolute vorticity variable "AVOR", instead of just
C               "VOR" (containing relative vorticity), produced by
C               routines MMXUSDVV() and MMPSDVV()
C       6/29/2001 by Carlie J. Coats, Jr. and John N. McHenry, MCNC:
C               Change addmdl3.incl to have MOLINV -- 1/MOL, directly
C               calculated by PBL routines, and output using MM2IO()
C               instead of MM2MOL(); matching change to MM2WST() uses
C               multiplication by MOLINV instead of division by MOL.
C               Support for up through 16 output domains.
C       8/11/2001 and John N. McHenry, MCNC:
C               If MRF PBL, add new MC2 variables QV2, TA2, UX10, VX10, WSPD10
C       11/8/2001 by Carlie J. Coats, Jr., MCNC:
C               New entry MCPL_ATIME takes XTIME-like time (minutes since
C               start of the run) as an argument.
C       11/26/2001 by Carlie J. Coats, Jr., MCNC:
C               new subroutine INT2IO acts like MM2IO but rounds to INTEGER;
C               more variables and validity checks for physics options
C               IMPHYS > 2, ISOIL > 1, Pleim-Xiu LSM/PBL
C       05/27/2003 by Carlie J. Coats, Jr., John McHenry, BAMS: add 3D divergence
C               calculation and compute both divergence and vorticity using
C               proper map-scale factors: do this by replacing MMXUSDVV()
C               and MMPSDVV() with MMXUSDVM() and MMPSDVM() respectively.
C       7/2003 by Carlie J. Coats, Jr., BAMS:
C               Fix calculation and validation of X0( N ), Y0( N )
C               in the case that GRIDDESC already exists;
C               MM2MASK becomes MM2HTR, producing high-temporal-resolution
C               mask, total-precip, and wet-bulb-freezing-level variables.
C               Change terminology to HTR-stuff instead of SC2-stuff.
C               Add HTR variables TA2, UX10, VX10, CFRAC[LMH]
C       9/2003 by John N. McHenry and Carlie J. Coats, Jr., BAMS:
C               Add HTR variables on requested pressure surfaces:
C               Z, U, V, RH, plus SLPRS. Improved
C               2-meter dewpoint calculation using metcalc approach.
C               New subroutine version MM2HTR5 supports this....
C       6/2008 by Carlie J. Coats, Jr. and Don Olerud, BAMS: 
C               MM2HTR7 pressure-level TA and RVOR
C       7/2008 by Carlie J. Coats, Jr., BAMS: TA and AVOR (instead of RVOR)
C       10/2008 by John N. McHenry:
C               If NOAH LSM, add new MC2 variable QSKIN
C***********************************************************************

        BLOCK DATA MCPL_INIT_FLAGS

#include "parame.incl"
#include "mcpl_flags.incl"

        DATA  MB2PSTAR, MB2ZNT                / MXMB2 * .FALSE. /

        DATA  MB3DENS, MB3PRES, MB3TV, MB3RH, MB3RHLW, MB3RHKF
     &                                            / MXMB3 * .FALSE. /

        DATA  MC2FAKE,
     &        MC2QFLX,   MC2HFLX,   MC2PBL,     MC2RN,     MC2RC,
     &        MC2PSTAR,  MC2TGD,    MC2CFRACH,  MC2CFRACM, MC2CFRACL,
     &        MC2USTAR,  MC2MOL,    MC2RGRND,   MC2RNET,   MC2FSOIL,
     &        MC2WSTAR,  MC2TA,     MC2TV,      MC2QV,     MC2QC,
     &        MC2QR,     MC2QI,     MC2QNI,     MC2QG,     MC2QNC,
     &        MC2PRES,   MC2DENS,   MC2UX,      MC2VX,     MC2WSPD,
     &        MC2DIV,    MC2AVOR,   MC2RVOR,    MC2RH,     MC2RHLW,
     &        MC2RHKF,   MC2RSD,    MC2ALB,     MC2CAPG,   MC2CSNROT,
     &        MC2EF,     MC2EMISS,  MC2F,       MC2GLW,    MC2GSW,
     &        MC2HOL,    MC2HT,     MC2MAVAIL,  MC2MSFX,   MC2PRW,
     &        MC2PSA,    MC2PSAINI, MC2QFX,     MC2HFX,    MC2RAINC,
     &        MC2RAINNC, MC2REGIME, MC2SATBRT,  MC2SHC,    MC2SNROT,
     &        MC2SNOWC,  MC2TGA,    MC2THC,     MC2TMN,    MC2UST,
     &        MC2XLAND,  MC2XLAT,   MC2XLONG,   MC2ZNT,    MC2KFTOP,
     &        MC2KFEXP,  MC2SLPRS,  MC2QV2,     MC2TA2,    MC2UX10,
     &        MC2VX10,   MC2WSPD10, MC2MOLENGTH,MC2RADNET, MC2ALBEDO,
     &        MC2GRNFLX, MC2RA,     MC2RS,      MC2TOB1,   MC2RHOBS,
     &        MC2LAI,    MC2VEGFRC, MC2ISLTYP,  MC2CANOM,  MC2SEAICE,
     &        MC2SRNOFF, MC2URNOFF, MC2SNOCVR,  MC2TSEASFC,MC2SNODPT,
     &        MC2SOILT,  MC2SOILM,  MC2QSKIN,
     &        MC2DZ1000_850,  MC2DZ1000_500,    MC2DZ850_700
     &                                            / MXMC2 * .FALSE. /

        DATA  MC3FAKE,
     &        MC3TA,    MC3QV,    MC3QC,    MC3QR,    MC3QI,   MC3QNI,
     &        MC3QG,    MC3QNC,   MC3DENS,  MC3PRES,  MC3TV,   MC3RH,
     &        MC3RHLW,  MC3RHKF,  MC3THETA, MC3WWIND, MC3UX,   MC3VX,
     &        MC3WSPD,  MC3DIV,   MC3AVOR,  MC3RVOR,  MC3QCA,  MC3QIA,
     &        MC3QNIA,  MC3QRA,   MC3QGA,   MC3QNCA,  MC3QVA
     &                                             / MXMC3 * .FALSE. /

        DATA  MD3FAKE,
     &        MD3UWIND, MD3VWIND, MD3UA, MD3VA  / MXMD3 * .FALSE. /

        DATA  PC3PDEX,  PC3CLIN,  PC3CLOG,  PC3CPOW,  PC3Z,     PC3TA, 
     &        PC3QV,    PC3QC,    PC3QR,    PC3QI,    PC3QNI,   PC3QG, 
     &        PC3QNC,   PC3DENS,  PC3TV,    PC3RH,    PC3RHLW,  PC3RHKF, 
     &        PC3THETA, PC3UWIND, PC3VWIND, PC3WWIND, PC3WSPD,  PC3DIV, 
     &        PC3AVOR,  PC3RVOR
     &                                            / MXPC3 * .FALSE. /

        DATA  PD3PDEX,  PD3CLIN,  PD3CLOG,  PD3CPOW, PD3UWIND, PD3VWIND
     &                                            / MXPD3 * .FALSE. /

       END

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_MB2( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, NT, C0, R0 )
        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL, NT          !  window dimensions
        INTEGER         C0, R0                  !  window offsets


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            ABUF( MIX*MJX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

        IF ( MB2PSTAR ) THEN	!  psa from /addr2/, converted from cb to Pa
            CALL MM2BDY( 1000.0, MIX, MJX, 1, PSA,
     &                   NC, NR, 1, NT, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PSTAR',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PSTAR to ' // FNAME
                CALL M3EXIT( 'MCPL_MB2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mb2pstar

        IF ( MB2ZNT ) THEN	!  psa from /addr2/, converted from cb to Pa
            CALL MM2BDY( 1.0, MIX, MJX, 1, ZNT,
     &                   NC, NR, 1, NT,
     &                   C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'ZNT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write ZNT to ' // FNAME
                CALL M3EXIT( 'MCPL_MB2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mb2ZNT

        RETURN
        END SUBROUTINE MCPL_MB2

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_MB3( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, NT, C0, R0, SIGH )
        IMPLICIT NONE

C...........   INCLUDES:

#include "PARMS3.EXT"      ! I/O API constants
#include "FDESC3.EXT"      ! I/O API file description data structure
#include "IODECL3.EXT"     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL, NT          !  window dimensions
        INTEGER         C0, R0                  !  window offsets
        REAL            SIGH( MKX )     !  MM5-convention half-levels


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            RBUF( MIX*MJX*MKX )
        REAL            SBUF( MIX*MJX*MKX )
        REAL            TBUF( MIX*MJX*MKX )
        REAL            UBUF( MIX*MJX*MKX )
        REAL            VBUF( MIX*MJX*MKX )
        REAL            WBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:


        IF ( BDPTFLAG ) THEN

            CALL MMBTHERMO( MIX, MJX, MKX, TA, QVA, PPA, PSA,
     &                      SIGH, PTOP,
     &                      NC, NR, NL, NT, C0, R0, 1,
     &                      RBUF, SBUF, TBUF, UBUF, VBUF, WBUF )
            IF ( MB3DENS ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DENS',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write DENS to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3dens
            IF ( MB3PRES ) THEN	!
                IF ( .NOT. WRITE3( FNAME, 'PRES',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write PRES to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3pres
            IF ( MB3TV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'TV',
     &                             JDATE, JTIME, TBUF ) ) THEN
                    MESG = 'Could not write TV to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3tv
            IF ( MB3RH ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RH',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write RH to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3tv
            IF ( MB3RHLW ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHLW',
     &                             JDATE, JTIME, VBUF ) ) THEN
                    MESG = 'Could not write RHLW to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3tv
            IF ( MB3RHKF ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHKF',
     &                             JDATE, JTIME, WBUF ) ) THEN
                    MESG = 'Could not write RHKF to ' // FNAME
                    CALL M3EXIT( 'MCPL_MB3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mb3tv

        END IF		!  if bdptflag

        RETURN
        END SUBROUTINE MCPL_MB3

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_MC2( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, C0, R0, SIGH, PLAYS, PVALH )
        IMPLICIT NONE

C...........   INCLUDES:

#include "PARMS3.EXT"      ! I/O API constants
#include "FDESC3.EXT"      ! I/O API file description data structure
#include "IODECL3.EXT"     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL, NT          !  window dimensions
        INTEGER         C0, R0                  !  window offsets
        REAL            SIGH( MKX )             !  MM5-convention half-levels
        INTEGER         PLAYS                   !  number of pressure-levels
        REAL            PVALH( MXLAYS3 )        !  pressure half-levels (Pa)


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         I, L
        REAL            T
        REAL            ABUF( MIX*MJX )
        REAL            BBUF( MIX*MJX )
        REAL            CBUF( MIX*MJX )
        REAL            DBUF( MIX*MJX )
        REAL            EBUF( MIX*MJX )
        REAL            FBUF( MIX*MJX )
        REAL            GBUF( MIX*MJX )
        REAL            HBUF( MIX*MJX )
        REAL            RBUF( MIX*MJX*MKX )
        REAL            WBUF( MIX*MJX*MKX )
        REAL            XBUF( MIX*MJX*MKX )
        REAL            YBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

        IF ( MC2FAKE ) THEN	!  fake
            CALL MM2FILL( 1.0E3, NC, NR, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'FAKE',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write FAKE to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2fakes

        IF ( MC2PBL ) THEN	!  pbl from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, PBL,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PBL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PBL to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2pbl

        IF ( MC2RN ) THEN	!  rainnc from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, RAINNC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RN',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RN to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rn

        IF ( MC2RC ) THEN	!  rainc from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, RAINC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rc

        IF ( MC2PSTAR ) THEN	!  psa from /addr2/ ~~> pstar
            CALL MM2IO( 1000.0, MIX, MJX, 1, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PSTAR',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PSTAR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2pstar

        IF ( MC2TGD ) THEN	!  tga from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, TGA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TGD',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TGD to ' //
     &                  FNAME
                CALL M3EXIT( 'MCPL_MC2',
     &                           JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2tgd

        IF ( MC2USTAR ) THEN	!  ust from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, UST,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'USTAR',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write USTAR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2ustar

#ifdef MCPL_ADDMDL3

        IF ( MC2MOL .AND. IBLTYP( NEST ) .NE. 7 ) THEN
            CALL MM2IO( 1.0,  MIX, MJX, 1, MOLINV(1,1,NEST ),
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'MOL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write MOL to ' //FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2mol

        IF ( MC2RNET .AND. IBLTYP( NEST ) .NE. 7  ) THEN    ! see /addmdl3/
            CALL MM2IO( 1.0,  MIX, MJX, 1, RDNET(1,1,NEST ),
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RNET',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RNET to ' //FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rnet

        IF ( MC2FSOIL .AND. IBLTYP( NEST ) .NE. 7  ) THEN    ! see /addmdl3/
            CALL MM2IO( 1.0,  MIX, MJX, 1, FSOIL(1,1,NEST ),
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'FSOIL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write FSOIL to '//FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2fsoil

        IF ( MC2WSTAR ) THEN
             CALL MM2WST( MIX, MJX, UST, PBL, MOLINV( 1,1,NEST ),
     &               NC, NR, C0, R0, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'WSTAR',
     &                     JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write WSTAR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2wstar

#endif   /** ifdef MCPL_ADDMDL3 **/

        IF ( MC2MOL .AND. IBLTYP( NEST ) .EQ. 7 ) THEN
            CALL MM2MOL( MIX, MJX, MOL,
     &                  NC, NR, C0, R0, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'MOL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write MOL to ' //FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2mol and ibltyp = 7

        IF ( MC2RNET .AND. IBLTYP( NEST ) .EQ. 7  ) THEN
            CALL MM2IO( 1.0,  MIX, MJX, 1, RADNET,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RNET',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RNET to ' //FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rnet

        IF ( MC2FSOIL .AND. IBLTYP( NEST ) .EQ. 7  ) THEN
            CALL MM2IO( 1.0,  MIX, MJX, 1, GRNFLX,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'FSOIL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write FSOIL to '//FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2fsoil

         IF ( MC2RGRND ) THEN	! calculated from GSW/(1-ALB)
            CALL MM2RAD( MIX, MJX, GSW, ALB,
     &                   NC, NR, C0, R0, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RGRND',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RGRND to '//FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rgrnd

        IF ( MC2TA ) THEN	!  decouple layer-1 ta from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, TA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TA',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2ta

        IF ( MC2QV .AND. IMPHYS( NEST ).GE.2 ) THEN
                                !  decouple layer-1 qva from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QVA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QV',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QV to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2qv

        IF ( MC2QC .AND. IMPHYS( NEST ).GE.3 ) THEN
                                !  decouple layer-1 QCa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QCA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2qv

        IF ( MC2QR .AND. IMPHYS( NEST ).GE.3 ) THEN
                                !  decouple layer-1 QRa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QRA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QR',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2QR

        IF ( MC2QI .AND. IMPHYS( NEST ).GE.5 ) THEN
                                !  decouple layer-1 QIa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QIA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QI',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2QI

        IF ( MC2QNI .AND. IMPHYS( NEST ).GE.5 ) THEN
                                !  decouple layer-1 QNIa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QNIA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QNI',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QNI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2QNI

        IF ( MC2QG .AND. IMPHYS( NEST ).GE.6 ) THEN
                                !  decouple layer-1 QGa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QGA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QG',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QG to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2QG

        IF ( MC2QNC .AND. IMPHYS( NEST ).GE.6 ) THEN
                                !  decouple layer-1 QNCa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QNCA, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QNC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QNC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2QNC

        IF ( C2DPTFLAG ) THEN	!  dens,pres,tv,rh,qflx,hflx

            CALL MM2THERM2( MIX, MJX, MKX,
     &                   TA, QVA, PPA, PSA, HFX, QFX, SIGH, PTOP,
     &                   NC, NR, C0, R0,
     &                   ABUF, BBUF, CBUF, DBUF,
     &                   EBUF, FBUF, GBUF, HBUF )

            IF ( MC2DENS ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DENS',
     &                             JDATE, JTIME, ABUF ) ) THEN
                    MESG = 'Could not write DENS to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2dens

            IF ( MC2PRES ) THEN
                IF ( .NOT. WRITE3( FNAME, 'PRES',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write PRES to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2pres

            IF ( MC2TV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'TV',
     &                             JDATE, JTIME, CBUF ) ) THEN
                    MESG = 'Could not write TV to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2tv

            IF ( MC2RH ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RH',
     &                             JDATE, JTIME, DBUF ) ) THEN
                    MESG = 'Could not write RH to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2rh

            IF ( MC2RHLW ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHLW',
     &                             JDATE, JTIME, EBUF ) ) THEN
                    MESG = 'Could not write RHLW to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2rhlw

            IF ( MC2RHKF ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHKF',
     &                             JDATE, JTIME, FBUF ) ) THEN
                    MESG = 'Could not write RHKF to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2rhkf

            IF ( MC2QFLX ) THEN	!  qflux is latent heat flux in unit M/S kg/kg
                IF ( IBLTYP( NEST ) .EQ. 7 ) THEN
                    !!  Adjust for P-X MM5-nonstandard W/M^2 units for QFX
                    T = 1.0 / XLV
                    DO  I = 1, NC*NR
                        GBUF( I ) = T * GBUF( I )
                    END DO
                END IF
                IF ( .NOT. WRITE3( FNAME, 'QFLX',
     &                             JDATE, JTIME, GBUF ) ) THEN
                    MESG = 'Could not write QFLX to ' //FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2qflx

            IF ( MC2HFLX ) THEN	!  hflx is sensible heat flux in m K/s
                IF ( .NOT. WRITE3( FNAME, 'HFLX',
     &                             JDATE, JTIME, HBUF ) ) THEN
                    MESG = 'Could not write HFLX to ' //FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc2hfx

        END IF			!  if cdptflag -- compute dens,pres,tv


        IF ( SDV2FLAG .AND. WNDWFLAG( WNDW ) ) THEN	!  calculate wspd,div,vor

            CALL MMXUSDVM(DX, MIX, MJX, MKX, UA, VA, PSA, F,
     &	                  MSFX, MSFD,
     &                    NC, NR, 1, C0, R0, 1,
     &                    EBUF, FBUF, ABUF, BBUF, CBUF, DBUF )

            IF ( MC2UX ) THEN
                IF ( .NOT. WRITE3( FNAME, 'UX',
     &                             JDATE, JTIME, EBUF ) ) THEN
                    MESG = 'Could not write UX to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2ux

            IF ( MC2VX ) THEN
                IF ( .NOT. WRITE3( FNAME, 'VX',
     &                             JDATE, JTIME, FBUF ) ) THEN
                    MESG = 'Could not write VX to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2vx

            IF ( MC2WSPD ) THEN
                IF ( .NOT. WRITE3( FNAME, 'WSPD',
     &                             JDATE, JTIME, ABUF ) ) THEN
                    MESG = 'Could not write WSPD to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2wspd

            IF ( MC2DIV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DIV',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write DIV to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2div

            IF ( MC2AVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'AVOR',
     &                             JDATE, JTIME, CBUF ) ) THEN
                    MESG = 'Could not write AVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2avor

            IF ( MC2RVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RVOR',
     &                             JDATE, JTIME, DBUF ) ) THEN
                    MESG = 'Could not write RVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc2rvor

        END IF		!  if sdv2flag


        IF ( MC2RSD ) THEN	!  gsw/(1-alb)
            CALL MM2RAD( MIX, MJX, GSW, ALB,
     &                   NC, NR, C0, R0, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RSD',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RSD to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc2rsd

        IF ( MC2ALB ) THEN !  ALB from /addr2/
            IF ( ISOIL .NE. 3 ) THEN
                CALL MM2IO( 1.0, MIX, MJX, 1, ALB,
     &                      NC, NR, 1, C0, R0, 1, ABUF )
            ELSE
                CALL MM2IO( 1.0, MIX, MJX, 1, ALBEDO,
     &                      NC, NR, 1, C0, R0, 1, ABUF )
            END IF

            IF ( .NOT. WRITE3( FNAME, 'ALB',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write ALB to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2alb

        IF ( MC2CAPG ) THEN !  CAPG from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, CAPG,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'CAPG',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write CAPG to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2capg

        IF ( MC2CSNROT ) THEN !  COSROT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, COSROT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'CSNROT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write CSNROT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2csnrot

        IF ( MC2EF ) THEN !  EF from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, EF,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'EF',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write EF to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2ef

        IF ( MC2EMISS ) THEN !  EMISS from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, EMISS,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'EMISS',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write EMISS to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2emiss

        IF ( MC2F ) THEN !  F from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, F,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'F',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write F to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2f

        IF ( MC2GLW ) THEN !  GLW from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, GLW,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'GLW',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write GLW to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2glw

        IF ( MC2GSW ) THEN !  GSW from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, GSW,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'GSW',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write GSW to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2gsw

        IF ( MC2HOL ) THEN !  HOL from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, HOL,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'HOL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write HOL to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2hol

        IF ( MC2HT ) THEN !  HT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, HT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'HT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write HT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2ht

        IF ( MC2MAVAIL ) THEN !  MAVAIL from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, MAVAIL,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'MAVAIL',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write MAVAIL to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2mavail

        IF ( MC2MSFX ) THEN !  MSFX from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, MSFX,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'MSFX',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write MSFX to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2msfx

        IF ( MC2PRW ) THEN !  PRW from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, PRW,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PRW',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PRW to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2prw

        IF ( MC2PSA ) THEN   !  PSA from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, PSA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PSA',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PSA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF                 !  if mc2psa

        IF ( MC2PSAINI ) THEN  !  PSAINI from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, PSAINI,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'PSAINI',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write PSAINI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2psaini

        IF ( MC2QFX ) THEN   ! QFX here:  xlv * /addr2/'s qfx
            IF ( IBLTYP( NEST ) .NE. 7 ) THEN
                CALL MM2IO( XLV, MIX, MJX, 1, QFX,
     &                      NC, NR, 1, C0, R0, 1, ABUF )
            ELSE
                !!  Compensate for P-X nonstandard W/M^2 units for QFX
                CALL MM2IO( 1.0, MIX, MJX, 1, QFX,
     &                      NC, NR, 1, 0, R0, 1, ABUF )
            END IF
            IF ( .NOT. WRITE3( FNAME, 'QFX',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QFX to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2qfx

        IF ( MC2HFX ) THEN   !  HFX from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, HFX,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'HFX',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write HFX to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2hfx

        IF ( MC2RAINC ) THEN !  RAINC from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, RAINC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RAINC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RAINC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2rainc

        IF ( MC2RAINNC ) THEN !  RAINNC from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, RAINNC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RAINNC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RAINNC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2rainnc

        IF ( MC2REGIME ) THEN !  REGIME from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, REGIME,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'REGIME',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write REGIME to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2REGIME

        IF ( MC2SATBRT ) THEN !  SATBRT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, SATBRT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SATBRT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SATBRT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2SATBRT

        IF ( MC2SHC ) THEN !  SHC from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, SHC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SHC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SHC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2shc

        IF ( MC2SNROT ) THEN !  SINROT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, SINROT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SNROT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SNROT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2SNROT

        IF ( MC2SNOWC ) THEN !  SNOWC from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, SNOWC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SNOWC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SNOWC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2snowc

        IF ( MC2TGA ) THEN !  TGA from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, TGA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TGA',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TGA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2tga

        IF ( MC2THC ) THEN !  THC from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, THC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'THC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write THC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2thc

        IF ( MC2TMN ) THEN !  TMN from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, TMN,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TMN',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TMN to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2tmn

        IF ( MC2UST ) THEN !  UST from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, UST,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'UST',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write UST to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2ust

        IF ( MC2XLAND ) THEN !  XLAND from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, XLAND,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'XLAND',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write XLAND to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2land

        IF ( MC2XLAT ) THEN !  XLAT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, XLAT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'XLAT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write XLAT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2xlat

        IF ( MC2XLONG ) THEN !  XLONG from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, XLONG,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'XLONG',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write XLONG to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2xlong

        IF ( MC2ZNT ) THEN !  ZNT from /addr2/
            CALL MM2IO( 1.0, MIX, MJX, 1, ZNT,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'ZNT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write ZNT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2znt


        IF ( MC2SLPRS     .OR. MC2DZ1000_850 .OR.
     &       MC2DZ850_700 .OR. MC2DZ1000_850 ) THEN    !  calculated SLPRS and store in MC2

#ifdef      MCPL_BMP
            CALL MM2BMP( MIX, MJX, MKX,
     &                   TA, QVA, PPA, PSA, SIGMA, SIGH, PTOP, HT,
     &                   NC, NR, NL, C0, R0, 1, ABUF )
#endif
#ifndef     MCPL_BMP
            CALL MM2SLP( MIX, MJX, MKX,
     &                   TA, PPA, PSA, SIGH, PTOP, HT,
     &                   NC, NR, NL, C0, R0, 1, ABUF )
#endif
            IF ( MC2SLPRS ) THEN
              IF ( .NOT. WRITE3( FNAME, 'SLPRS',
     &                           JDATE, JTIME, ABUF ) ) THEN
                  MESG = 'Could not write SLPRS to ' // FNAME
                  CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

              END IF
            END IF

            IF ( MC2KFTOP ) THEN    !  calculated KFTOP
                CALL MM2IO( 1.0, MIX, MJX, 1, KFTOP( 1,1,NEST ),
     &                      NC, NR, 1, C0, R0, 1, BBUF )
                IF ( .NOT. WRITE3( FNAME, 'KFTOP',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write KFTOP to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
    
                END IF
            END IF               !  if mc2kftop

            IF ( MC2KFEXP ) THEN    !  calculated KFEXP
                CALL MM2IO( 1.0, MIX, MJX, 1, KFEXP( 1,1,NEST ),
     &                      NC, NR, 1, C0, R0, 1, BBUF )
                IF ( .NOT. WRITE3( FNAME, 'KFEXP',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write KFEXP to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

                END IF
            END IF               !  if mc2kfexp


            IF ( MC2DZ1000_850 .OR.
     &           MC2DZ850_700  .OR. MC2DZ1000_850 ) THEN

                CALL MMX2P( MIX, MJX, MKX,
     &                      PPA, PSA, SIGH, PTOP, PVALH,
     &                      NC, NR, PLAYS, C0, R0, 1,
     &                      WBUF,    !!  interpolation indices
     &                      XBUF,    !!  lin-interpolation coeffs
     &                      YBUF,    !!  log-interpolation coeffs
     &                      RBUF )   !!  pow-interpolation coeffs

                CALL MMPZ( MIX, MJX, MKX,
     &                     PPA, PSA, SIGH, PTOP, P0, TLP, TS0, ROVG,
     &                     PVALH, ABUF, WBUF, XBUF,
     &                     NC, NR, PLAYS, C0, R0, 1,
     &                     RBUF, BBUF, CBUF, DBUF )
            END IF

            IF ( MC2DZ1000_850 ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DZ1000_850',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write DZ1000_850 to ' // FNAME
                    CALL M3EXIT( 'MM5/MCPL', JDATE, JTIME, MESG, 2 )
                END IF
            END IF      !  if mc2dz1000_850

            IF ( MC2DZ850_700 ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DZ1000_500',
     &                             JDATE, JTIME, CBUF ) ) THEN
                    MESG = 'Could not write DZ1000_500 to ' // FNAME
                    CALL M3EXIT( 'MM5/MCPL', JDATE, JTIME, MESG, 2 )
                END IF
            END IF      !  if mc2dz850_700

            IF ( MC2DZ1000_850 ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DZ850_700',
     &                             JDATE, JTIME, DBUF ) ) THEN
                    MESG = 'Could not write DZ850_700 to ' // FNAME
                    CALL M3EXIT( 'MM5/MCPL', JDATE, JTIME, MESG, 2 )
                END IF
            END IF      !  if mc2dz1000_850

        END IF               !  if mc2slprs

                !!  observation-height met variables:

        IF ( OBHTFLAG( WNDW ) .AND. MC2QV2 ) THEN    !  QA2 from /ADDR2/
            CALL MM2IO( 1.0, MIX, MJX, 1, QA2,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'QV2',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write QV2 to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

            END IF
        END IF               !  if MC2QV2

        IF ( OBHTFLAG( WNDW ) .AND. MC2TA2 ) THEN    !  TA2 from /ADDR2/
            CALL MM2IO( 1.0, MIX, MJX, 1, TA2,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TA2',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TA2 to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

            END IF
        END IF               !  if mc2ta2

        IF ( WNDWFLAG( WNDW ) .AND. OBHTFLAG( WNDW ) .AND.
     &       ( MC2UX10 .OR. MC2VX10 .OR. MC2WSPD10 ) ) THEN

            CALL MMXUV10( MIX, MJX, UA10, VA10,
     &                    NC, NR, C0, R0,
     &                    ABUF, BBUF, CBUF )

            IF ( MC2UX10 ) THEN    !  calculated UX10
                IF ( .NOT. WRITE3( FNAME, 'UX10',
     &                             JDATE, JTIME, ABUF ) ) THEN
                    MESG = 'Could not write UX10 to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

                END IF
            END IF               !  if mc2UX10

            IF ( MC2VX10 ) THEN    !  calculated VX10
                IF ( .NOT. WRITE3( FNAME, 'VX10',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write VX10 to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

                END IF
            END IF               !  if mc2VX10

            IF ( MC2WSPD10 ) THEN    !  calculated WSPD10
                IF ( .NOT. WRITE3( FNAME, 'WSPD10',
     &                             JDATE, JTIME, CBUF ) ) THEN
                    MESG = 'Could not write WSPD10 to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )

                END IF
            END IF               !  if mc2wspd10

        END IF          !  if wndwflag and ( mc2ux10 or mcvx10 or mc2wspd10 )

                !!  Pleim-Xiu land-surface variables:

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2MOLENGTH ) THEN !  MOL from /addr2/ in point2d.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, MOL,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'MOLENGTH',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write MOLENGTH to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2molength

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2RADNET ) THEN !  RADNET from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, RADNET,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RADNET',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RADNET to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2radnet

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2ALBEDO ) THEN !  ALBEDO from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, ALBEDO,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'ALBEDO',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write ALBEDO to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2albedo

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2GRNFLX ) THEN !  GRNFLX from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, GRNFLX,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'GRNFLX',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write GRNFLX to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2grnflx

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2RA ) THEN !  RA from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, RA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RA',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2ra

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2RS ) THEN !  RS from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, RS,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RS',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RS to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2rs

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2TOB1 ) THEN !  THZ1OB from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, THZ1OB,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TOB1',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TOB1 to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2tob1

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2RHOBS ) THEN !  RHOBS from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, RHOBS,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'RHOBS',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write RHOBS to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2rhobs

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2LAI ) THEN !  LAI from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, LAI,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'LAI',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write LAI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2lai

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2VEGFRC ) THEN !  VEGFRC from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, VEGFRC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'VEGFRC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write VEGFRC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2vegfrc

        IF ( IBLTYP( NEST ) .EQ. 7  .AND. MC2ISLTYP ) THEN !  ISTYP from /LANDSFC/ in soilp.incl
            CALL MM2INT( 1, MIX, MJX, 1, ISLTYP,
     &                   NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'ISLTYP',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write ISLTYP to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2isltyp

        IF ( MC2SNOCVR ) THEN !  SNOWC from point2d
            CALL MM2INT( 1.0, MIX, MJX, 1, SNOWC,
     &                   NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SNOCVR',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SNOCVR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if MC2SNOCVR

        IF ( MC2TSEASFC ) THEN !  TSS from point2d
            CALL MM2IO( 1.0, MIX, MJX, 1, TSS,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'TSEASFC',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write TSEASFC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if MC2TSEASFC

        IF ( MC2SNODPT ) THEN !  SNOA from
            CALL MM2IO( 1.0, MIX, MJX, 1, SNOA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SNODPT',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SNODPT to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2SNODPT

        IF ( MC2SEAICE ) THEN !  XICE from param2
            CALL MM2IO( 1.0, MIX, MJX, 1, XICE,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SEAICE',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SEAICE to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2SEAICE

        IF ( MC2SRNOFF ) THEN !  SSROFF from
            CALL MM2IO( 1.0, MIX, MJX, 1, SSROFF,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'SRNOFF',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write SRNOFF to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2SRNOFF

        IF ( MC2URNOFF ) THEN !  BGROFF from
            CALL MM2IO( 1.0, MIX, MJX, 1, BGROFF,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'URNOFF',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write URNOFF to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if MC2URNOFF

        IF ( MC2CANOM ) THEN !  CMCA from /LANDSFC/ in soilp.incl
            CALL MM2IO( 1.0, MIX, MJX, 1, CMCA,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
            IF ( .NOT. WRITE3( FNAME, 'CANOM',
     &                         JDATE, JTIME, ABUF ) ) THEN
                MESG = 'Could not write CANOM to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc2canom

        IF ( ISOIL .EQ. 1 ) THEN

           !!  "exotic" selector for starting-layer:  Unlike atmospheric
           !!  variables (which are reversed between MM5 and I/O API),
           !!  for soil variables the I/O API and the MM5 land surface
           !!  submodel use the same subscripting (hence the "MLX-L+1"
           !!  "starting layer" number, to undo the reversal in MM2IO())

           DO L = 1, MLX
               IF ( MC2SOILT(L) ) THEN !  TSLA from param2.incl
                   CALL MM2IO( 1.0, MIX, MJX, MLX, TSLA,
     &                         NC, NR, 1, C0, R0, MLX-L+1, ABUF )
                   IF ( .NOT. WRITE3( FNAME, SOILTNAME( L ),
     &                                 JDATE, JTIME, ABUF ) ) THEN
                       MESG = 'Could not write ' // SOILTNAME( L ) //
     &                         ' to ' // FNAME
                       CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                   END IF
               END IF               !  if mc2soilt(L)
           END DO

        ELSE IF ( ISOIL .EQ. 2 ) THEN

           DO L = 1, MLX
               IF ( MC2SOILT(L) ) THEN !  STCA from /LANDSFC/ in soilp.incl
                   CALL MM2IO( 1.0, MIX, MJX, MLX, STCA,
     &                         NC, NR, 1, C0, R0, MLX-L+1, ABUF )
                   IF ( .NOT. WRITE3( FNAME, SOILTNAME( L ),
     &                                 JDATE, JTIME, ABUF ) ) THEN
                       MESG = 'Could not write ' // SOILTNAME( L ) //
     &                         ' to ' // FNAME
                       CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                   END IF
               END IF               !  if mc2soilt(L)
           END DO

           DO L = 1, MLX
               IF ( MC2SOILM(L) ) THEN !  SMCA from /LANDSFC/ in soilp.incl
                   CALL MM2IO( 1.0, MIX, MJX, MLX, SMCA,
     &                         NC, NR, 1, C0, R0, MLX-L+1, ABUF )
                   IF ( .NOT. WRITE3( FNAME, SOILMNAME( L ),
     &                                 JDATE, JTIME, ABUF ) ) THEN
                       MESG = 'Could not write ' // SOILMNAME( L ) //
     &                         '  to ' // FNAME
                       CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                   END IF
               END IF               !  if mc2soilm(L)
           END DO

           IF ( MC2QSKIN ) THEN !  QSFC from /addr2/ in point2d.incl
               CALL MM2IO( 1.0, MIX, MJX, 1, QSFC,
     &                  NC, NR, 1, C0, R0, 1, ABUF )
               IF ( .NOT. WRITE3( FNAME, 'QSKIN',
     &                         JDATE, JTIME, ABUF ) ) THEN
                   MESG = 'Could not write QSKIN to ' // FNAME
                   CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
               END IF
           END IF               !  if mc2qskin

        ELSE IF ( ISOIL .EQ. 3 ) THEN

           DO L = 1, 2
               IF ( MC2SOILT(L) ) THEN !  STCA from /LANDSFC/ in soilp.incl
                   CALL MM2IO( 1.0, MIX, MJX, MLX, STCA,
     &                         NC, NR, 1, C0, R0, MLX-L+1, ABUF )
                   IF ( .NOT. WRITE3( FNAME, SOILTNAME( L ),
     &                                 JDATE, JTIME, ABUF ) ) THEN
                       MESG = 'Could not write ' // SOILTNAME( L ) //
     &                         ' to ' // FNAME
                       CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                   END IF
               END IF               !  if mc2soilt(L)
           END DO

           DO L = 1, 2
               IF ( MC2SOILM(L) ) THEN !  SMCA from /LANDSFC/ in soilp.incl
                   CALL MM2IO( 1.0, MIX, MJX, MLX, SMCA,
     &                         NC, NR, 1, C0, R0, MLX-L+1, ABUF )
                   IF ( .NOT. WRITE3( FNAME, SOILMNAME( L ),
     &                                 JDATE, JTIME, ABUF ) ) THEN
                       MESG = 'Could not write ' // SOILMNAME( L ) //
     &                         '  to ' // FNAME
                       CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
                   END IF
               END IF               !  if mc2soilm(L)
           END DO

        END IF

        RETURN
        END SUBROUTINE MCPL_MC2

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_MC3( FNAME, GNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, C0, R0, SIGH )
        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  MC3 file name
        CHARACTER*(*)   GNAME                   !  MC2 file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL              !  window dimensions
        INTEGER         C0, R0                  !  window offsets
        REAL            SIGH( MKX )     !  MM5-convention half-levels


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            ABUF( MIX*MJX )
        REAL            BBUF( MIX*MJX )
        REAL            CBUF( MIX*MJX )

        REAL            RBUF( MIX*MJX*MKX )
        REAL            SBUF( MIX*MJX*MKX )
        REAL            TBUF( MIX*MJX*MKX )
        REAL            UBUF( MIX*MJX*MKX )
        REAL            VBUF( MIX*MJX*MKX )
        REAL            WBUF( MIX*MJX*MKX )
        REAL            XBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

        IF ( MC3FAKE ) THEN	!  fake
            CALL MM2FILL( 1.0E3, NC, NR, NL, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'FAKE',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write FAKE to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc3fake

        IF ( MC3TA ) THEN       !  decouple ta from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, TA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'TA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write TA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc3ta

        IF ( MC3QV .AND. IMPHYS( NEST ).GE.2 ) THEN
                                !  decouple layer-1 qva from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QVA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QV',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QV to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3qv

        IF ( MC3QC .AND. IMPHYS( NEST ).GE.3 ) THEN
                                !  decouple layer-1 QCa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QCA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QC',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3qv

        IF ( MC3QR .AND. IMPHYS( NEST ).GE.3 ) THEN
                                !  decouple layer-1 QRa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QRA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QR',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QR to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3QR

        IF ( MC3QI .AND. IMPHYS( NEST ).GE.5 ) THEN
                                !  decouple layer-1 QIa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QIA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QI',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3QI

        IF ( MC3QNI .AND. IMPHYS( NEST ).GE.5 ) THEN
                                !  decouple layer-1 QNIa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QNIA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNI',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNI to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3QNI

        IF ( MC3QG .AND. IMPHYS( NEST ).GE.6 ) THEN
                                !  decouple layer-1 QGa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QGA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QG',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QG to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3QG

        IF ( MC3QNC .AND. IMPHYS( NEST ).GE.6 ) THEN
                                !  decouple layer-1 QNCa from /addr1/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX, QNCA, PSA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNC',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNC to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if MC3QNC

        IF ( MC3DPTFLAG ) THEN	!  dens,pres,tv,rh,theta

            !!  Also calculates PRES in SBUF and RHLW in VBUF
            !!  for use by SUBROUTINE CLDFRAC below:

            CALL MM2THERM3( MIX, MJX, MKX, TA, QVA, PPA, PSA,
     &                   SIGH, PTOP,
     &                   NC, NR, NL, C0, R0, 1,
     &                   RBUF, SBUF, TBUF, UBUF, VBUF, WBUF, XBUF )

            IF ( MC3DENS ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DENS',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write DENS to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3dens

            IF ( MC3PRES ) THEN
                IF ( .NOT. WRITE3( FNAME, 'PRES',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write PRES to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3pres

            IF ( MC3TV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'TV',
     &                             JDATE, JTIME, TBUF ) ) THEN
                    MESG = 'Could not write TV to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3tv

            IF ( MC3RH ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RH',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write RH to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3rh

            IF ( MC3RHLW ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHLW',
     &                             JDATE, JTIME, VBUF ) ) THEN
                    MESG = 'Could not write RHLW to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3rhlw

            IF ( MC3RHKF ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHKF',
     &                             JDATE, JTIME, WBUF ) ) THEN
                    MESG = 'Could not write RHKF to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if mc3rhkf

            IF ( MC3THETA ) THEN
                IF ( .NOT. WRITE3( FNAME, 'THETA',
     &                             JDATE, JTIME, XBUF ) ) THEN
                    MESG = 'Could not write THETA to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if mc3THETA

            IF ( MC2CFRACH .OR. MC2CFRACM .OR. MC2CFRACL ) THEN

                !!  Uses PRES in SBUF and RHLW in VBUF
                !!  from SUBROUTINE MM2THERM3 above:

C                CALL CLDFRAC(  NC, NR, NL,
C     &                         SBUF, VBUF, ABUF, BBUF, CBUF )
                CALL CLDFRAC2( MIX, MJX, MKX, DX,
     &                         PSA, PBL, SIGH, SIGMA, PTOP,
     &                         P0, TLP, TS0, ROVG, R,
     &                         NC, NR, NL,
     &                         SBUF, WBUF, ABUF, BBUF, CBUF )
            END IF		!  if calculating cloud fractional coverage

            IF ( MC2CFRACL ) THEN
                IF ( .NOT. WRITE3( GNAME, 'CFRACL',
     &                             JDATE, JTIME, ABUF ) ) THEN
                    MESG = 'Could not write CFRACL to ' // GNAME
                    CALL M3EXIT( 'MCPL_MC3',
     &                           JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if MC2CFRACL

            IF ( MC2CFRACM ) THEN
                IF ( .NOT. WRITE3( GNAME, 'CFRACM',
     &                             JDATE, JTIME, BBUF ) ) THEN
                    MESG = 'Could not write CFRACM to ' // GNAME
                    CALL M3EXIT( 'MCPL_MC3',
     &                           JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if MC2CFRACM

            IF ( MC2CFRACH ) THEN
                IF ( .NOT. WRITE3( GNAME, 'CFRACH',
     &                             JDATE, JTIME, CBUF ) ) THEN
                    MESG = 'Could not write CFRACH to ' // GNAME
                    CALL M3EXIT( 'MCPL_MC3',
     &                           JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if MC2CFRACH

       END IF			!  if cdptflag -- compute dens,pres,tv,rh,theta


        IF ( MC3WWIND ) THEN	!  decouple wa from /nonhyd/
            CALL MMXDCPL( 1.0, MIX, MJX, MKX+1, WA, PSA,
     &                  NC, NR, NL, C0, R0, 2, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'WWIND',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write WWIND to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if mc3wwind


        IF ( SDV3FLAG .AND. WNDWFLAG( WNDW )  ) THEN !  calculate wspd,div,vor

            CALL MMXUSDVM(DX, MIX, MJX, MKX, UA, VA, PSA, F,
     &	                  MSFX, MSFD,
     &                    NC, NR, NL, C0, R0, 1,
     &                    UBUF, VBUF, RBUF, SBUF, TBUF, WBUF )

            IF ( MC3UX ) THEN
                IF ( .NOT. WRITE3( FNAME, 'UX',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write UX to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3ux

            IF ( MC3VX ) THEN
                IF ( .NOT. WRITE3( FNAME, 'VX',
     &                             JDATE, JTIME, VBUF ) ) THEN
                    MESG = 'Could not write VX to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3vx

            IF ( MC3WSPD ) THEN
                IF ( .NOT. WRITE3( FNAME, 'WSPD',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write WSPD to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3wspd

            IF ( MC3DIV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DIV',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write DIV to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3div

            IF ( MC3AVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'AVOR',
     &                             JDATE, JTIME, TBUF ) ) THEN
                    MESG = 'Could not write AVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3avor

            IF ( MC3RVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RVOR',
     &                             JDATE, JTIME, WBUF ) ) THEN
                    MESG = 'Could not write RVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF              !  if mc3rvor

        END IF		!  if sdv2flag

        IF ( MC3QVA .AND. IMPHYS(NEST).GE.2 ) THEN !  QVA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QVA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QVA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QVA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qva

        IF ( MC3QCA .AND. IMPHYS(NEST).GE.3 ) THEN !  QCA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QCA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QCA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QCA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qca

        IF ( MC3QRA .AND. IMPHYS(NEST).GE.3 ) THEN !  QRA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QRA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QRA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QRA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qra

        IF ( MC3QIA .AND. IMPHYS(NEST).GE.5 ) THEN !  QIA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QIA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QIA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QIA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qia

        IF ( MC3QNIA .AND. IMPHYS(NEST).GE.5 ) THEN !  QNIA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QNIA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNIA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNIA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qnia

        IF ( MC3QGA .AND. IMPHYS(NEST).GE.6 ) THEN !  QGA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QGA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QGA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QGA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3qga

        IF ( MC3QNCA .AND. IMPHYS(NEST).GE.6 ) THEN !  QNCA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, QNCA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNCA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNCA to ' // FNAME
                CALL M3EXIT( 'MCPL_MC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if mc3QNCa


        RETURN
        END SUBROUTINE MCPL_MC3

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_MD3( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, C0, R0 )
        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL              !  window dimensions
        INTEGER         C0, R0                  !  window offsets


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            RBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

        IF ( MD3FAKE ) THEN	!  fake
            CALL MM2FILL( 1.0E3, NC, NR, NL, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'FAKE',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write FAKE to ' // FNAME
                CALL M3EXIT( 'MCPL_MC2', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if md3fake

        IF ( MD3UWIND .AND. WNDWFLAG( WNDW )  ) THEN	!  decouple ua
            CALL MMDDCPL( 1.0, MIX, MJX, MKX, UA, PSA,
     &                    NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'UWIND',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write UWIND to ' // FNAME
                CALL M3EXIT( 'MCPL_MD3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if md3uwind

        IF ( MD3VWIND .AND. WNDWFLAG( WNDW )  ) THEN	!  decouple va
            CALL MMDDCPL( 1.0, MIX, MJX, MKX, VA, PSA,
     &                    NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'VWIND',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write VWIND to ' // FNAME
                CALL M3EXIT( 'MCPL_MD3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if md3vwind

        IF ( MD3UA ) THEN !  UA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, UA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'UA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write UA to ' // FNAME
                CALL M3EXIT( 'MCPL_MD3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if md3ua

        IF ( MD3VA ) THEN !  VA from /addr1/
            CALL MM2IO( 1.0, MIX, MJX, MKX, VA,
     &                  NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'VA',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write VA to ' // FNAME
                CALL M3EXIT( 'MCPL_MD3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF               !  if md3va

        RETURN
        END SUBROUTINE MCPL_MD3

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_PC3( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, C0, R0, SIGH, PVALH )
        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL              !  window dimensions
        INTEGER         C0, R0                  !  window offsets
        REAL            SIGH( MKX )     !  MM5-convention half-levels
        REAL            PVALH( MXLAYS3 )


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            ABUF( MIX*MJX )
        REAL            BBUF( MIX*MJX )
        REAL            CBUF( MIX*MJX )
        REAL            DBUF( MIX*MJX )

        INTEGER         PDEX( MIX*MJX*MKX )
        REAL            PLIN( MIX*MJX*MKX )
        REAL            PLOG( MIX*MJX*MKX )

        REAL            RBUF( MIX*MJX*MKX )
        REAL            SBUF( MIX*MJX*MKX )
        REAL            TBUF( MIX*MJX*MKX )
        REAL            UBUF( MIX*MJX*MKX )
        REAL            VBUF( MIX*MJX*MKX )
        REAL            WBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

        CALL MMX2P( MIX, MJX, MKX,
     &              PPA, PSA, SIGH, PTOP, PVALH,
     &              NC, NR, NL, C0, R0, 1,
     &              PDEX,    !!  interpolation indices
     &              PLIN,    !!  lin-interpolation coeffs
     &              PLOG,    !!  log-interpolation coeffs
     &              RBUF )   !!  pow-interpolation coeffs

        IF ( PC3PDEX ) THEN
            IF ( .NOT. WRITE3( FNAME, 'PDEX',
     &                         JDATE, JTIME, PDEX ) ) THEN
                MESG = 'Could not write PDEX to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3pdex

        IF ( PC3CLIN ) THEN
            IF ( .NOT. WRITE3( FNAME, 'CLIN',
     &                         JDATE, JTIME, PLIN ) ) THEN
                MESG = 'Could not write CLIN to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3clin

        IF ( PC3CLOG ) THEN
            IF ( .NOT. WRITE3( FNAME, 'CLOG',
     &                         JDATE, JTIME, PLOG ) ) THEN
                MESG = 'Could not write CLOG to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3clog

        IF ( PC3CPOW ) THEN
            IF ( .NOT. WRITE3( FNAME, 'CPOW',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write CPOW to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3cpow

        IF ( PC3Z ) THEN    !  calculated SLPRS and store in MC2

#ifdef      MCPL_BMP
            CALL MM2BMP( MIX, MJX, MKX,
     &                   TA, QVA, PPA, PSA, SIGMA, SIGH, PTOP, HT,
     &                   NC, NR, NL, C0, R0, 1, ABUF )
#endif
#ifndef     MCPL_BMP
            CALL MM2SLP( MIX, MJX, MKX,
     &                   TA, PPA, PSA, SIGH, PTOP, HT,
     &                   NC, NR, NL, C0, R0, 1, ABUF )
#endif

            CALL MMPZ( MIX, MJX, MKX,
     &                 PPA, PSA, SIGH, PTOP, P0, TLP, TS0, ROVG,
     &                 PVALH, ABUF, PDEX, PLIN,
     &                 NC, NR, NL, C0, R0, 1,
     &                 RBUF, BBUF, CBUF, DBUF )

            IF ( .NOT. WRITE3( FNAME, 'Z',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write Z to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF

        END IF          !  if pc3z

        IF ( PC3QV .AND. IMPHYS( NEST ).GE.2 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QVA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QV',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QV to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QV

        IF ( PC3QC .AND. IMPHYS( NEST ).GE.3 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QCA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QC',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QC to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QC

        IF ( PC3QR .AND. IMPHYS( NEST ).GE.3 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QRA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QR',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QR to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QR

        IF ( PC3QI .AND. IMPHYS( NEST ).GE.5 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QIA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QI',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QI to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QI

        IF ( PC3QNI .AND. IMPHYS( NEST ).GE.5 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QNIA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNI',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNI to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QNI

        IF ( PC3QG .AND. IMPHYS( NEST ).GE.6 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QGA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QG',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QG to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QG

        IF ( PC3QNC .AND. IMPHYS( NEST ).GE.6 ) THEN
            CALL MMPXDCPL( 1.0, MIX, MJX, MKX, QNCA, PSA,
     &                     NC, NR, NL, C0, R0, 1, PDEX, PLIN, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'QNC',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write QNC to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3QNC

        IF ( PC3DPTFLAG ) THEN  !  calculating ta, dens, tv, rh:

            CALL MMPTHERMO( MIX, MJX, MKX, TA, QVA, PPA, PSA,
     &                    SIGH, PTOP, P0,
     &                    PVALH, PDEX, PLIN, PLOG,
     &                    NC, NR, NL, C0, R0, 1,
     &                    RBUF, SBUF, TBUF, UBUF, VBUF, WBUF )       ! =ta,dens,tv,rh

            IF ( PC3TA ) THEN
                IF ( .NOT. WRITE3( FNAME, 'TA',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write TA to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3ta

            IF ( PC3DENS ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DENS',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write DENS to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3dens

            IF ( PC3TV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'TV',
     &                             JDATE, JTIME, TBUF ) ) THEN
                    MESG = 'Could not write TV to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3tv

            IF ( PC3RH ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RH',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write RH to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3rh

            IF ( PC3RHLW ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHLW',
     &                             JDATE, JTIME, VBUF ) ) THEN
                    MESG = 'Could not write RHLW to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3rhlw

            IF ( PC3RHKF ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RHKF',
     &                             JDATE, JTIME, WBUF ) ) THEN
                    MESG = 'Could not write RHKF to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
               END IF
            END IF		!  if pc3rhkf

            IF ( PC3THETA ) THEN            !!  TA(C,R,L) is in RBUF
                CALL MMPTHETA( NC, NR, NL,
     &                         PVALH, RBUF, UBUF )
                IF ( .NOT. WRITE3( FNAME, 'THETA',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write THETA to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3theta

        END IF		!  if pc3dpt

        IF ( PC3WWIND ) THEN
            CALL MMPW( MIX, MJX, MKX+1, PPA, WA, PSA,
     &                 SIGMA, PTOP, PVALH,
     &                 NC, NR, NL, C0, R0, 1, RBUF )
            IF ( .NOT. WRITE3( FNAME, 'WWIND',
     &                         JDATE, JTIME, RBUF ) ) THEN
                MESG = 'Could not write WWIND to ' // FNAME
                CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
            END IF
        END IF		!  if pc3WWIND

        IF ( WNDWFLAG( WNDW ) .AND. SDVPFLAG ) THEN

            CALL MMPSDVM( DX, MIX, MJX, MKX, UA, VA, PSA, F,
     &                    MSFX, MSFD, PDEX, PLIN,
     &                    NC, NR, NL, C0, R0, 1,
     &                    RBUF, SBUF, TBUF, WBUF, UBUF, VBUF )

            IF ( PC3WSPD ) THEN
                IF ( .NOT. WRITE3( FNAME, 'WSPD',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write WSPD to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3wspd

            IF ( PC3DIV ) THEN
                IF ( .NOT. WRITE3( FNAME, 'DIV',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write DIV to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3div

            IF ( PC3AVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'AVOR',
     &                             JDATE, JTIME, TBUF ) ) THEN
                    MESG = 'Could not write AVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3avor

            IF ( PC3RVOR ) THEN
                IF ( .NOT. WRITE3( FNAME, 'RVOR',
     &                             JDATE, JTIME, WBUF ) ) THEN
                    MESG = 'Could not write RVOR to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3rvor

            IF ( PC3UWIND ) THEN
                IF ( .NOT. WRITE3( FNAME, 'UWIND',
     &                             JDATE, JTIME, UBUF ) ) THEN
                    MESG = 'Could not write UWIND to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3UWIND

            IF ( PC3VWIND ) THEN
                IF ( .NOT. WRITE3( FNAME, 'VWIND',
     &                             JDATE, JTIME, VBUF ) ) THEN
                    MESG = 'Could not write VWIND to ' // FNAME
                    CALL M3EXIT( 'MCPL_PC3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pc3VWIND

        END IF !wndwflag and sdvpflag: interp&compute uwind,vwind,wspd,div,vor


        RETURN
        END SUBROUTINE MCPL_PC3


C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        SUBROUTINE MCPL_PD3( FNAME, JDATE, JTIME, NEST, WNDW,
     &                       NC, NR, NL, C0, R0, SIGH, PVALH )
        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME                   !  file name
        INTEGER         JDATE                   !  date, YYYDDD
        INTEGER         JTIME                   !  time, HHMMSS
        INTEGER         NEST                    !  MM5 nest number
        INTEGER         WNDW                    !  MCPL window number
        INTEGER         NC, NR, NL              !  window dimensions
        INTEGER         C0, R0                  !  window offsets
        REAL            SIGH( MKX )     !  MM5-convention half-levels
        REAL            PVALH( MXLAYS3 )!  standard pressure half-levels (Pa)


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        REAL            PDEX( MIX*MJX*MKX )
        REAL            PLIN( MIX*MJX*MKX )
        REAL            RBUF( MIX*MJX*MKX )
        REAL            SBUF( MIX*MJX*MKX )
        CHARACTER*256   MESG

C..................................................................
C...........   Subroutine body:

                ! Compute pressure-level interp index & coeffs:
                ! PDEX is the index array
                ! PLIN is coefficients for pressure-interpolation
                ! YBUF is coefficients for log(pressure)-interpolation
                ! ZBUF is coefficients for pressure**K-interpolation
                ! NOTE:  PC3 outputs winds at cross-points for interface
                ! to packages like gempack, but needs dot-point vertical
                ! interpolation tables to compute them.

            CALL MMD2P( MIX, MJX, MKX,
     &                  PPA, PSA, SIGH, PTOP, PVALH,
     &                  NC, NR, NL, C0, R0, 1,
     &                  PDEX, PLIN, RBUF, SBUF )

            IF ( PD3PDEX ) THEN
                IF ( .NOT. WRITE3( FNAME, 'PDEX',
     &                             JDATE, JTIME, PDEX ) ) THEN
                    MESG = 'Could not write PDEX to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pd3pdex

            IF ( PD3CLIN ) THEN
                IF ( .NOT. WRITE3( FNAME, 'CLIN',
     &                             JDATE, JTIME, PLIN ) ) THEN
                    MESG = 'Could not write CLIN to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pd3clin

            IF ( PD3CLOG ) THEN
                IF ( .NOT. WRITE3( FNAME, 'CLOG',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write CLOG to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pd3clog

            IF ( PD3CPOW ) THEN
                IF ( .NOT. WRITE3( FNAME, 'CPOW',
     &                             JDATE, JTIME, SBUF ) ) THEN
                    MESG = 'Could not write CPOW to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if pd3cpow

            IF ( PD3UWIND ) THEN	!  decouple ua
                CALL MMPDDCPL( 1.0, MIX, MJX, MKX, UA, PSA,
     &                         NC,NR,NL, C0, R0, 1,
     &                         PDEX, PLIN, RBUF )
                IF ( .NOT. WRITE3( FNAME, 'UWIND',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write UWIND to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if PD3uwind

            IF ( PD3VWIND ) THEN	!  decouple va
                CALL MMPDDCPL( 1.0, MIX, MJX, MKX, VA, PSA,
     &                         NC, NR, NL, C0, R0, 1,
     &                         PDEX, PLIN, RBUF )
                IF ( .NOT. WRITE3( FNAME, 'VWIND',
     &                             JDATE, JTIME, RBUF ) ) THEN
                    MESG = 'Could not write VWIND to ' // FNAME
                    CALL M3EXIT( 'MCPL_PD3', JDATE, JTIME, MESG, 2 )
                END IF
            END IF		!  if PD3vwind

        RETURN
        END SUBROUTINE MCPL_PD3

C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-



        SUBROUTINE  MCPL
#ifdef  MM5_V3
        IMPLICIT NONE
#endif   /** ifdef MM5_V3 **/



C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations

#include "parame.incl"
#include "addr0.incl"
#include "point3d.incl"
#include "navypb.incl"
#include "point2d.incl"
#include "various.incl"
#include "variousn.incl"
#include "nestl.incl"
#include "nonhyd.incl"
#include "nhcnst.incl"
#include "fddagd.incl"
#include "pmoist.incl"
#include "varia.incl"
#include "param2.incl"
#include "param3.incl"
#include "soil.incl"

#ifdef MM5_V3
#include "radiat.incl"       ! RTTEN
#include "soilp.incl"        ! variables for soil and veg for LSM
#include "parpx.incl"        ! switches for PX LSM

#include "chardate.incl"     ! for CDATE....
#include "hdrv3.incl"        ! for MIF array and MRF arrays....
#endif                  /** ifdef MM5_V3 **/

#ifndef MM5_V3          /** ifndef MM5_V3 -- MM5_V2 **/
#include "header.incl"       ! for MIF array and MRF arrays....
#endif                  /** ifndef MM5_V3 -- MM5_V2 **/

#ifdef MCPL_ADDMDL3
#include "addmdl3.incl"    ! for add"l Models-3/MAQSIP specific arrays....
#endif   /** ifdef MCPL_ADDMDL3 **/

        INCLUDE 'mcpl_flags.incl'

C...........   ARGUMENTS and their descriptions:

        INTEGER         NEST  ! input:  the current nest number, 1=coarse domain
        INTEGER         IDATE, ITIME   !  mcpl_[a]time() output date&time
        REAL            ATIME          !  mcpl_atime() input time

C.......   Arguments for MCPL_KF2:

        INTEGER         II, JJ  !  MM5 I and J values
        REAL            P300KF  !  pressure 300 mb above sfc at cloud inception
        REAL            TCKF    !  duration of this cloud in seconds
        REAL            RKF     !  Precip originating as liquid water in cloud
        REAL            ICEKF   !  Precip originating as ice in cloud
        REAL            TADVEC  !  advection transit time, seconds
        INTEGER         NIC     !  number of MM5 advection steps for TADVEC
        REAL            CLDTP   !  KF cloud top, meters a.g.l.
        REAL            TKF0( MKX )     !  Temperature, K
        REAL            QKF0( MKX )     !  Water Vapor, kg/kg
        REAL            UKF0( MKX )     !  U-component of wind velocity, M/S
        REAL            VKF0( MKX )     !  V-", M/S
        REAL            WAV0( MKX )     !  averaged W-", M/S
        REAL            PKF0( MKX )     !  Presssure, PA
        REAL            DZQKF( MKX )    !  full-sigma layer thickness, m
        REAL            DPKF0( MKX )    !  full-sigma pressure thickess, PA


C...........   PARAMETERS and their descriptions:

        CHARACTER*1     QUOTE           !  quote-character
        CHARACTER*16    BLANK           !  blank-string

        PARAMETER     ( QUOTE = '''' ,
     &                  BLANK = ' ' )

        CHARACTER*16    GB2VARS         !  logical name for vbles-list file
        CHARACTER*16    GB3VARS         !  logical name for vbles-list file
        CHARACTER*16    GC2VARS         !  logical name for vbles-list file
        CHARACTER*16    GC3VARS         !  logical name for vbles-list file
        CHARACTER*16    GD2VARS         !  logical name for vbles-list file
        CHARACTER*16    MB2VARS         !  logical name for vbles-list file
        CHARACTER*16    MB3VARS         !  logical name for vbles-list file
        CHARACTER*16    MC2VARS         !  logical name for vbles-list file
        CHARACTER*16    MC3VARS         !  logical name for vbles-list file
        CHARACTER*16    MD3VARS         !  logical name for vbles-list file
        CHARACTER*16    PC3VARS         !  logical name for vbles-list file
        CHARACTER*16    PD3VARS         !  logical name for vbles-list file

        PARAMETER (     GB2VARS = 'GB2_VARS' ,
     &                  GB3VARS = 'GB3_VARS' ,
     &                  GC2VARS = 'GC2_VARS' ,
     &                  GC3VARS = 'GC3_VARS' ,
     &                  GD2VARS = 'GD2_VARS' ,
     &                  MB2VARS = 'MB2_VARS' ,
     &                  MB3VARS = 'MB3_VARS' ,
     &                  MC2VARS = 'MC2_VARS' ,
     &                  MC3VARS = 'MC3_VARS' ,
     &                  MD3VARS = 'MD3_VARS' ,
     &                  PC3VARS = 'PC3_VARS' ,
     &                  PD3VARS = 'PD3_VARS' )

        CHARACTER*16    GB2BASE         !  root for output-file logical name
        CHARACTER*16    GB3BASE         !  root for output-file logical name
        CHARACTER*16    GC2BASE         !  root for output-file logical name
        CHARACTER*16    GC3BASE         !  root for output-file logical name
        CHARACTER*16    GD2BASE         !  root for output-file logical name
        CHARACTER*16    MB2BASE         !  root for output-file logical name
        CHARACTER*16    MB3BASE         !  root for output-file logical name
        CHARACTER*16    MC2BASE         !  root for output-file logical name
        CHARACTER*16    MC3BASE         !  root for output-file logical name
        CHARACTER*16    MD3BASE         !  root for output-file logical name
        CHARACTER*16    PC3BASE         !  root for output-file logical name
        CHARACTER*16    PD3BASE         !  root for output-file logical name
        CHARACTER*16    MKABASE         !  root for output-file logical name
        CHARACTER*16    MKPBASE         !  root for output-file logical name

        DATA GB2BASE / 'GRID_BDY_2D' /
        DATA GB3BASE / 'GRID_BDY_3D' /
        DATA GC2BASE / 'GRID_CRO_2D' /
        DATA GC3BASE / 'GRID_CRO_3D' /
        DATA GD2BASE / 'GRID_DOT_2D' /
        DATA MB2BASE / 'MET_BDY_2D'  /
        DATA MB3BASE / 'MET_BDY_3D'  /
        DATA MC2BASE / 'MET_CRO_2D'  /
        DATA MC3BASE / 'MET_CRO_3D'  /
        DATA MD3BASE / 'MET_DOT_3D'  /
        DATA PC3BASE / 'PRES_CRO_3D' /
        DATA PD3BASE / 'PRES_DOT_3D' /
        DATA MKABASE / 'MET_KF_SCA'   /
        DATA MKPBASE / 'MET_KF_PRO'   /

        REAL	D60    	        !  conversion factor:  min::sec
        REAL	CMPHR2MPS	!  conversion factor:  cm/hr::M/S
        REAL	SEC2DAY	        !  conversion factor:  seconds::day
        REAL    TWOTHDS         !  fraction:  2/3
        REAL    DWAT            !  molecular diffusivity of water vapor, cm^2/s
        REAL    KVIS            !  kinematic viscosity of air, cm^2/s
        REAL    PR              !  Prandtl number (dimensionless)
        REAL    RBIFAC          !  coeff for laminar BL resistance calculation

        PARAMETER ( D60       = 1.0 / 60.0,
     &              CMPHR2MPS = 1.0E-2 * 3600.0,
     &              SEC2DAY   = 86400.0,
     &              TWOTHDS   = 2.0 / 3.0 ,
     &              DWAT      = 0.244,
     &              KVIS      = 0.1461,
     &              PR        = 0.709 )
     
        REAL            MBTOPA, PATOMB  !  conv factor, millibars <~~> Pascals
        PARAMETER     ( MBTOPA = 1.0E2,
     &                  PATOMB = 1.0 / MBTOPA )     

C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         CURREC
        LOGICAL         DSCGRID
        INTEGER		ENVINT
        REAL 		ENVREAL
        LOGICAL         INTLIST, REALIST, STRLIST
        INTEGER		GETEFILE
        INTEGER		INDEX1, JULIAN
        INTEGER		SECSDIFF, SEC2TIME, TIME2SEC
        INTEGER		LBLANK, TRIMLEN
        CHARACTER*24    DT2STR

        EXTERNAL CURREC, DSCGRID, INTLIST, REALIST, STRLIST,
     &           GETEFILE, INDEX1, JULIAN, SECSDIFF, SEC2TIME, TIME2SEC,
     &           LBLANK, TRIMLEN, DT2STR, ENVINT, ENVREAL

        LOGICAL         KFOPEN
        INTEGER		KFWRITE
        EXTERNAL        KFOPEN, KFWRITE


C...........   SAVED LOCAL VARIABLES and their descriptions:
C...........   NOTE:  the ANSI standard requires the use of SAVE statements
C...........   for variables which must retain their values from call to call.

        !!  Scratch variables:  subscripts and loop counters.
        !!  Needed HERE for Intel "ifc" to accept the implied DO loop
        !!  of the DATA statement for MC2NAMES

        INTEGER	        I, J, K, L, M, N, MMN, V
        REAL*8          DFAC, DINV

        LOGICAL         FIRSTIMEG, FIRSTIMEK
        DATA            FIRSTIMEG, FIRSTIMEK / 2 * .TRUE. /
        INTEGER         LOGDEV
        DATA            LOGDEV / -1 /

        LOGICAL         LNOVARS   !  no "standard" I/O API output for
                                  !  dot-, cross- or bdy-pt vbles.
        DATA            LNOVARS   / .FALSE. /

        LOGICAL         FIRSTIME( MXFIL )
        DATA            FIRSTIME / MXFIL * .TRUE. /

        LOGICAL         FIRSTK( MXFIL )
        DATA            FIRSTK / MXFIL * .TRUE. /

        LOGICAL         FIRSTSC( MXFIL )
        DATA            FIRSTSC / MXFIL * .TRUE. /

        LOGICAL         TURNOFF( MXFIL )
        DATA            TURNOFF / MXFIL * .FALSE. /

        INTEGER         LASTREC( MXFIL )    !  logical rec # of last output
        DATA            LASTREC / MXFIL * -1 /

        INTEGER         MM5DOMN( MXFIL )    ! MM5 nest for this output grid
        INTEGER         GRIDCUM( 0:MXFIL )  ! # of cum out-grids per MM5 nest
        INTEGER         GRIDCNT             ! total # of out-nests

        CHARACTER*3     GTAIL( MXFIL )
C        DATA            GTAIL / '_G0', '_G1', '_G2', '_G3', '_G4',
C     &                          '_G5', '_G6', '_G7', '_G8', '_G9',
C     &                          '_GA', '_GB', '_GC', '_GD', '_GE',
C     &                          '_GF' /
        DATA            GTAIL / '_G0', '_G1', '_G2', '_G3', '_G4',
     &                          '_G5', '_G6', '_G7', '_G8', '_G9',
     &                          '_GA', '_GB', '_GC', '_GD', '_GE',
     &                          '_GF', '_GG', '_GH', '_GI', '_GJ',
     &                          '_GK', '_GL', '_GM', '_GN' /

        CHARACTER*16    PROJNAME( 0:6 )
        DATA            PROJNAME
     &                  / '<invalid/NA>',
     &                    'Lat-Lon',
     &                    'Lambert',
     &                    'Mercator',
     &                    'Stereographic',
     &                    'UTM',
     &                    '<invalid/NA>' /

        CHARACTER*16    GB2FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    GB3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    GC2FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    GC3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    GD2FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MB2FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MB3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MC2FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MC3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MD3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    PC3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    PD3FILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MKAFILE( MXFIL ) !  logical name for output file
        CHARACTER*16    MKPFILE( MXFIL ) !  logical name for output file

        CHARACTER*16    GB2NAMES( MXGB2 ) !  lists of actual output variables
        CHARACTER*16    GB3NAMES( MXGB3 )
        CHARACTER*16    GC2NAMES( MXGC2 )
        CHARACTER*16    GC3NAMES( MXGC3 )
        CHARACTER*16    GD2NAMES( MXGD2 )
        CHARACTER*16    MB2NAMES( MXMB2 )
        CHARACTER*16    MB3NAMES( MXMB3 )
        CHARACTER*16    MC2NAMES( MXMC2 )
        CHARACTER*16    MC3NAMES( MXMC3 )
        CHARACTER*16    MD3NAMES( MXMD3 )
        CHARACTER*16    PC3NAMES( MXPC3 )
        CHARACTER*16    PD3NAMES( MXPD3 )

        INTEGER		MDATE0	        !  model starting date, YYYYDDD
        INTEGER		MTIME0	        !  model starting time, HHMMSS
        INTEGER		SDATE( MXFIL )	!  output starting date, YYYYDDD
        INTEGER		STIME( MXFIL )	!  output starting time, HHMMSS
        INTEGER		TSTEP( MXFIL )	!  output time step, HHMMSS
        INTEGER		EDATE( MXFIL )	!  output ending date, YYYYDDD
        INTEGER		ETIME( MXFIL )	!  output end time, HHMMSS
        INTEGER		ENDREC( MXFIL )	!  output end record number

        CHARACTER*12    GNAME( MXFIL )  !  output grid names
        CHARACTER*16    CNAME( MXFIL )  !  cross-point grid names
        CHARACTER*16    DNAME( MXFIL )  !  dot  -point grid names
        REAL            X0( MXFIL )     !  x at dotpt (1,1)
        REAL            Y0( MXFIL )     !  y at dotpt (1,1)
        REAL            DN( MXFIL )     !  dx(grid)
        INTEGER		RCRO0( MXFIL )	!  starting row for OUTPT cro-grid in mm5
        INTEGER		CCRO0( MXFIL )	!  starting col for OUTPT cro-grid in mm5
        INTEGER		RDOT0( MXFIL )	!  starting row for OUTPT dot-grid in mm5
        INTEGER		CDOT0( MXFIL )	!  starting col for OUTPT dot-grid in mm5
        INTEGER		NCCRO( MXFIL )	!  number of cols, OUTPT cro-grid
        INTEGER		NRCRO( MXFIL )	!  number of rows, OUTPT cro-grid
        INTEGER		NCDOT( MXFIL )	!  number of cols, OUTPT dot-grid
        INTEGER		NRDOT( MXFIL )	!  number of rows, OUTPT dot-grid
        INTEGER		NTHIK( MXFIL )	!  boundary thickness
        INTEGER		NLAYS           !  number of sigma-levels, 3-D output grids
        INTEGER         PLAYS           !  # of pressure levels, PC3 & PD3 files

        LOGICAL         WNDWERR  !  error generating proper-window variables
        LOGICAL         BNDYERR  !  generating bdy structure variables

        LOGICAL         GB2FLAG( MXFIL ) !  generating GB2 file, or not
        LOGICAL         GB3FLAG( MXFIL ) !  generating GB3 file
        LOGICAL         GC2FLAG( MXFIL ) !  generating GC2 file
        LOGICAL         GC3FLAG( MXFIL ) !  generating GC3 file
        LOGICAL         GD2FLAG( MXFIL ) !  generating GD2 file
        LOGICAL         MB2FLAG( MXFIL ) !  generating MB2 file
        LOGICAL         MB3FLAG( MXFIL ) !  generating MB3 file
        LOGICAL         MC2FLAG( MXFIL ) !  generating MC2 file
        LOGICAL         MC3FLAG( MXFIL ) !  generating MC3 file
        LOGICAL         MD3FLAG( MXFIL ) !  generating MD3 file
        LOGICAL         PC3FLAG( MXFIL ) !  generating PC3 file
        LOGICAL         PD3FLAG( MXFIL ) !  generating PD3 file
        LOGICAL         MKAFLAG( MXFIL ) !  generating MKA file
        LOGICAL         MKPFLAG( MXFIL ) !  generating MKP file

        LOGICAL         GB2LAT
        LOGICAL         GB2LON
        LOGICAL         GB2MSFX2
        LOGICAL         GB2MSFX
        LOGICAL         GB2HT
        LOGICAL         GB2PSTAR0
        LOGICAL         GB2TSFC0
        LOGICAL         GB2PSFC0
        LOGICAL         GB2ZZERO

        DATA     GB2LAT,    GB2LON,   GB2MSFX2, GB2MSFX, GB2HT,
     &           GB2PSTAR0, GB2TSFC0, GB2PSFC0, GB2ZZERO
     &                                            / MXGB2 * .FALSE. /

        LOGICAL         GB3DENS0
        LOGICAL         GB3ZH
        LOGICAL         GB3ZF
        LOGICAL         GB3DZZ
        LOGICAL         GB3TH0
        LOGICAL         GB3TF0
        LOGICAL         GB3PRH0
        LOGICAL         GB3PRF0

        DATA     GB3DENS0, GB3ZH, GB3ZF, GB3DZZ, GB3TH0, GB3TF0,
     &           GB3PRH0,  GB3PRF0
     &                                            / MXGB3 * .FALSE. /

        LOGICAL         GC2LAT
        LOGICAL         GC2LON
        LOGICAL         GC2MSFX2
        LOGICAL         GC2HT
        LOGICAL         GC2PSTAR0
        LOGICAL         GC2MSFX
        LOGICAL         GC2TSFC0
        LOGICAL         GC2PSFC0
        LOGICAL         GC2ZZERO
        LOGICAL         GC2ALB
        LOGICAL         GC2CAPG
        LOGICAL         GC2CSNROT
        LOGICAL         GC2EF
        LOGICAL         GC2EMISS
        LOGICAL         GC2F
        LOGICAL         GC2MAVAIL
        LOGICAL         GC2SATBRT
        LOGICAL         GC2SHC
        LOGICAL         GC2SNROT
        LOGICAL         GC2SNOWC
        LOGICAL         GC2THC
        LOGICAL         GC2TMN

        DATA     GC2LAT,   GC2LON,   GC2MSFX2, GC2HT,     GC2PSTAR0,
     &           GC2MSFX,  GC2TSFC0, GC2PSFC0, GC2ZZERO,  GC2ALB,
     &           GC2CAPG,  GC2CSNROT,GC2EF,    GC2EMISS,  GC2F,
     &           GC2MAVAIL,GC2SATBRT,GC2SHC,   GC2SNROT, GC2SNOWC,
     &           GC2THC,   GC2TMN
     &                                            / MXGC2 * .FALSE. /

        LOGICAL         GC3DENS0
        LOGICAL         GC3ZH
        LOGICAL         GC3ZF
        LOGICAL         GC3DZZ
        LOGICAL         GC3TH0
        LOGICAL         GC3TF0
        LOGICAL         GC3PRH0
        LOGICAL         GC3PRF0

        DATA     GC3DENS0, GC3ZH, GC3ZF, GC3DZZ, GC3TH0, GC3TF0,
     &           GC3PRH0, GC3PRF0
     &                                            / MXGC3 * .FALSE. /

        LOGICAL         GD2LAT
        LOGICAL         GD2LON
        LOGICAL         GD2MSFD2
        LOGICAL         GD2MSFD

        DATA     GD2LAT, GD2LON, GD2MSFD2, GD2MSFD / MXGD2 * .FALSE. /

        CHARACTER*16    GB2LIST( MXGB2 )
        DATA            GB2LIST /
     &  'LAT',  'LON',   'MSFX2', 'HT',  'PSTAR0',
     &  'MSFX', 'TSFC0', 'PSFC0', 'ZZERO'
     &   /

        CHARACTER*16    GB3LIST( MXGB3 )
        DATA            GB3LIST /
     &  'DENS0', 'ZH', 'ZF', 'DZZ', 'TH0', 'TF0', 'PRH0', 'PRF0' /

        CHARACTER*16    GC2LIST( MXGC2 )
        DATA            GC2LIST /
     &  'LAT',   'LON',   'MSFX2',  'HT',    'PSTAR0',
     &  'MSFX',  'TSFC0', 'PSFC0',  'ZZERO', 'ALB',
     &  'CAPG',  'CSNROT','EF',     'EMISS', 'F',
     &  'MAVAIL','SATBRT','SHC',   'SNROT',  'SNOWC',
     &  'THC',   'TMN'  /

        CHARACTER*16    GC3LIST( MXGC3 )
        DATA            GC3LIST /
     &  'DENS0', 'ZH', 'ZF', 'DZZ', 'TH0', 'TF0', 'PRH0', 'PRF0' /

        CHARACTER*16    GD2LIST( MXGD2 )
        DATA            GD2LIST / 'LAT', 'LON', 'MSFD2', 'MSFD' /

        CHARACTER*16    MB2LIST( MXMB2 )
        DATA            MB2LIST / 'PSTAR', 'ZNT' /

        CHARACTER*16    MB3LIST( MXMB3 )
        DATA            MB3LIST /
     &  'DENS', 'PRES', 'TV', 'RH', 'RHLW', 'RHKF'  /

        CHARACTER*16    OSUSOILEXT( 4 )
        DATA            OSUSOILEXT /  '010', '040', '100', '200' /

        CHARACTER*16    MC2LIST( MXMC2 )
        DATA          ( MC2LIST(L),  L = 1,  MXMC2 - 2*MLX ) /
     &  'FAKE',
     &  'QFLX',   'HFLX',   'PBL',    'RN',     'RC',     'PSTAR',
     &  'TGD',    'CFRACH', 'CFRACM', 'CFRACL', 'USTAR',  'MOL',
     &  'RGRND',  'RNET',   'FSOIL',  'WSTAR',  'TA',     'QV',
     &  'QC',     'QR',     'QI',     'QNI',    'QG',     'QNC',
     &  'DENS',   'PRES',   'TV',     'RH',     'RHLW',   'RHKF',
     &  'UX',     'VX',     'WSPD',   'DIV',    'AVOR',   'RVOR',
     &  'RSD',    'ALB',    'CAPG',   'CSNROT', 'EF',     'EMISS',
     &  'F',      'GLW',    'GSW',    'HOL',    'HT',     'MAVAIL',
     &  'MSFX',   'PRW',    'PSA',    'PSAINI', 'QFX',    'HFX',
     &  'RAINC',  'RAINNC', 'REGIME', 'SATBRT', 'SHC',    'SNROT',
     &  'SNOWC',  'TGA',    'THC',    'TMN',    'UST',    'XLAND',
     &  'XLAT',   'XLONG',  'ZNT',    'KFTOP',  'KFEXP',  'SLPRS',
     &  'DZ1000_850',       'DZ1000_500',       'DZ850_700',
     &  'QV2',    'TA2',    'UX10',   'VX10',   'WSPD10', 'MOLENGTH',
     &  'RADNET', 'ALBEDO', 'GRNFLX', 'RA',     'RS',     'TOB1',
     &  'RHOBS',  'LAI',    'VEGFRC', 'ISLTYP', 'CANOM',  'SRNOFF',
     &  'URNOFF', 'SEAICE', 'SNOCVR', 'TSEASFC','SNODPT', 'QSKIN'
     &  /

        CHARACTER*16    MC3LIST( MXMC3 )
        DATA            MC3LIST /
     &  'FAKE',
     &  'TA',   'QV',  'QC',   'QR',  'QI',   'QNI',  'QG',   'QNC',
     &  'DENS', 'PRES','TV',   'RH',  'RHLW', 'RHKF', 'THETA','WWIND',
     &  'UX',   'VX',  'WSPD', 'DIV', 'AVOR', 'RVOR', 'QCA',  'QIA',
     &  'QNIA', 'QRA', 'QVA',  'QGA', 'QNCA' /

        CHARACTER*16    MD3LIST( MXMD3 )
        DATA            MD3LIST  /
     &  'FAKE','UWIND', 'VWIND', 'UA', 'VA'  /

        CHARACTER*16    PC3LIST( MXPC3 )
        DATA            PC3LIST /
     &  'PDEX',  'CLIN',  'CLOG',  'CPOW',  'Z',    'TA',   'QV',
     &  'QC',    'QR',    'QI',    'QNI',   'QG',   'QNC',  'DENS',
     &  'TV',    'RH',    'RHLW',  'RHKF',  'THETA','UWIND','VWIND',
     &  'WWIND', 'WSPD',  'DIV',   'AVOR',  'RVOR'
     &          /

        CHARACTER*16    PD3LIST( MXPD3 )
        DATA            PD3LIST  /
     &  'PDEX', 'CLIN', 'CLOG', 'CPOW', 'UWIND', 'VWIND' /

        INTEGER         NGB2    !  number of vbles-list file entries
        INTEGER         NGB3    !  number of vbles-list file entries
        INTEGER         NGC2    !  number of vbles-list file entries
        INTEGER         NGC3    !  number of vbles-list file entries
        INTEGER         NGD2    !  number of vbles-list file entries
        INTEGER         NMB2    !  number of vbles-list file entries
        INTEGER         NMB3    !  number of vbles-list file entries
        INTEGER         NMC2    !  number of vbles-list file entries
        INTEGER         NMC3    !  number of vbles-list file entries
        INTEGER         NMD3    !  number of vbles-list file entries
        INTEGER         NPC3    !  number of vbles-list file entries
        INTEGER         NPD3    !  number of vbles-list file entries


C...........   State Variables for computing KFTOP:

        INTEGER         KFDIM
        PARAMETER     ( KFDIM = MIX * MJX * MXFIL )
C        REAL		KFTOP( MIX, MJX, MXFIL )  !  cloud top, or 0.0
C        REAL		KFTIM( MIX, MJX, MXFIL )  !  cloud expiration time
C        REAL            KFEXP( MIX, MJX, MXFIL )  !  time remaining till expiration
C        DATA            KFTOP / KFDIM * 0.0 /
C        DATA            KFTIM / KFDIM * 0.0 /
C        DATA            KFEXP / KFDIM * 0.0 /
        INTEGER         KFTHIK( MXFIL )    ! max number of KF events per run

C        SAVE            KFTOP, KFTIM, KFTHIK, KFEXP
        SAVE            KFTHIK

C...........   Data structures for High Temporal Resolution
C...........    (Snow/Mixed/Liquid Mask + 5-minute-met) Files

        CHARACTER*16    HTRBASE         !  root for HTR-file logical name
        DATA            HTRBASE / 'HTR_CRO_2D'  /
        CHARACTER*16    HTRFILE( MXFIL ) !  logical name for output HTR file
        LOGICAL         HTRFLAG( MXFIL ) !  generating HTR file, or not
        INTEGER         HTRDATE( MXFIL ) !  starting dates
        INTEGER         HTRTIME( MXFIL ) !  starting times
        INTEGER         HTRSTEP( MXFIL ) !  time steps
        INTEGER         LASTHTR( MXFIL ) !  last output time step
        INTEGER         NEARSFC
        REAL            TKICE, TKLIQ, TKFRZ
        INTEGER         NPHTR, NPDIM
        REAL            PVHTR( MXLAYS3 )  !  standard pressure half-levels (Pa)
        CHARACTER*16    UNAMHTR( MXLAYS3 )  !  pressure level U  variables
        CHARACTER*16    VNAMHTR( MXLAYS3 )  !  pressure level V  variables
        CHARACTER*16    ZNAMHTR( MXLAYS3 )  !  pressure level Z  variables
        CHARACTER*16    HNAMHTR( MXLAYS3 )  !  pressure level RH variables
        CHARACTER*16    TNAMHTR( MXLAYS3 )  !  pressure level TA variables
        CHARACTER*16    ANAMHTR( MXLAYS3 )  !  pressure level AVOR variables	

        SAVE    HTRFILE, HTRFLAG, HTRDATE, HTRTIME, HTRSTEP, LASTHTR,
     &          NEARSFC, TKICE, TKLIQ, TKFRZ, NPHTR, NPDIM, PVHTR,
     &          UNAMHTR, VNAMHTR, ZNAMHTR, HNAMHTR, TNAMHTR, ANAMHTR


C.......   Vertical grid structures:

        REAL            SIGH( MKX )     !  MM5-convention half-levels
        REAL            VGTOP	        !  model top

        REAL            PVALS( MXLAYS3+1 )!  standard pressure full-levels (Pa)
        REAL            PVALH( MXLAYS3 )  !  standard pressure half-levels (Pa)
        REAL            PVALT( 18 )       !  default pressure  full-levels (Pa)
        DATA            PVALT
     &          /
     &  1025.0E2, 975.0E2, 925.0E2, 875.0E2, 825.0E2, 775.0E2,   !   1- 6
     &   725.0E2, 675.0E2, 625.0E2, 575.0E2, 525.0E2, 475.0E2,   !   7-12
     &   425.0E2, 375.0E2, 325.0E2, 275.0E2, 225.0E2, 175.0E2    !  13-18
     &          /

C...........   MODEL-SYNCHRONIZATION data structures:
C.......   SYNCHFLAG:  do we synchronize on output grid N?
C.......   SYNCHINIT:  is output grid N's synch file open yet?
C.......   SYNCHFILE:  output grid N's synch file delay (in timesteps)
C.......   SYNCHDATE:  output grid N's synch file starting date
C.......   SYNCHTIME:  output grid N's synch file starting time
C.......   SYNCHSTEP:  output grid N's synch file time step
C.......   SYNCHDIFF:  time-delta between MM5 and synch-read times
C.......   SYNCHLAST:  last call's synch record number

        LOGICAL         SYNCHFLAG( MXFIL )
        DATA            SYNCHFLAG / MXFIL * .FALSE. /

        LOGICAL         SYNCHINIT( MXFIL )
        DATA            SYNCHINIT / MXFIL * .FALSE. /

        CHARACTER*16    SYNCHFILE ( MXFIL )
        INTEGER         SYNCHDELAY( MXFIL )
        INTEGER         SYNCHDATE ( MXFIL )
        INTEGER         SYNCHTIME ( MXFIL )
        INTEGER         SYNCHSTEP ( MXFIL )
        INTEGER         SYNCHLAST ( MXFIL )
        INTEGER         SYNCHDIFF ( MXFIL )
        INTEGER         SJDATE, SJTIME        !  synch-read date&time

C.......   SAVE statements:

        SAVE   LOGDEV,    FIRSTIME,  MM5DOMN,   GRIDCUM,   GRIDCNT,
     &         GB2FILE,   GB3FILE,   GC2FILE,   GC3FILE,   GD2FILE,
     &         MB2FILE,   MB3FILE,   MC2FILE,   MC3FILE,   MD3FILE,
     &         PC3FILE,   PD3FILE,   MKAFILE,   MKPFILE,
     &         GB2NAMES,  GB3NAMES,  GC2NAMES,  GC3NAMES,  GD2NAMES,
     &         MB2NAMES,  MB3NAMES,  MC2NAMES,  MC3NAMES,  MD3NAMES,
     &         PC3NAMES,  PD3NAMES,  LASTREC,   PVALH,     PVALS,
     &         NGB2,      NGB3,      NGC2,      NGC3,      NGD2,
     &         NMB2,      NMB3,      NMC2,      NMC3,      NMD3,
     &         NPC3,      NPD3,      SIGH,      VGTOP,
     &         MDATE0,    MTIME0,    SDATE,     STIME,     TSTEP ,
     &         EDATE,     ETIME,     ENDREC

        SAVE   WNDWERR,   BNDYERR,
     &         GNAME,     CNAME,     DNAME,     X0,        Y0,
     &         DN,        RCRO0,     CCRO0,     RDOT0,     CDOT0,
     &         NCCRO,     NRCRO,     NCDOT,     NRDOT,     NTHIK,
     &         NLAYS,     PLAYS,
     &         GB2FLAG,   GB3FLAG,   GC2FLAG,   GC3FLAG,   GD2FLAG,
     &         MB2FLAG,   MB3FLAG,   MC2FLAG,   MC3FLAG,   MD3FLAG,
     &         PC3FLAG,   PD3FLAG,   MKAFLAG,   MKPFLAG

        SAVE   GB2LAT,    GB2LON,    GB2MSFX2,  GB2MSFX,   GB2HT,
     &         GB2PSTAR0, GB2TSFC0,  GB2PSFC0,  GB2ZZERO,
     &         GB3DENS0,  GB3ZH,     GB3ZF,     GB3DZZ,    GB3TH0,
     &         GB3TF0,    GB3PRH0,   GB3PRF0,
     &         GC2LAT,    GC2LON,    GC2MSFX2,  GC2HT,     GC2PSTAR0,
     &         GC2MSFX,   GC2TSFC0,  GC2PSFC0,  GC2ZZERO,  GC2ALB,
     &         GC2CAPG,   GC2CSNROT, GC2EF,     GC2EMISS,  GC2F,
     &         GC2MAVAIL, GC2SATBRT, GC2SHC,    GC2SNROT,  GC2SNOWC,
     &         GC2THC,    GC2TMN

        SAVE   GC3DENS0,  GC3ZH,     GC3ZF,     GC3DZZ,    GC3TH0,
     &         GC3TF0,    GC3PRH0,   GC3PRF0,
     &         GD2LAT,    GD2LON,    GD2MSFD2,  GD2MSFD

        SAVE   OSUSOILEXT

        SAVE   SYNCHFLAG, SYNCHINIT, SYNCHFILE, SYNCHDELAY,
     &         SYNCHDATE, SYNCHTIME, SYNCHSTEP, SYNCHLAST, SYNCHDIFF


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        CHARACTER*16    CLOUDTYPE  ! Cloud parameterization used in MM5

        INTEGER		IMAX 	!  coarse domain dimension I
        INTEGER		JMAX	!  ", J
        INTEGER         I0( MXFIL )   !  output domain offsets wrt MM5
        INTEGER         J0( MXFIL )   !  nests (0 means same (1,1))
        INTEGER         NNG     !  number of MM5DOMN-list entries
        INTEGER         NI0     !  number of I0-list entries
        INTEGER         NJ0     !  number of J0-list entries
        INTEGER         NNR     !  number of NROWS-list entries
        INTEGER         NNC     !  number of NCOLS-list entries
        INTEGER         NNT     !  number of NTHIK-list entries
        INTEGER         NNO     !  number of OUTGNAMES-list entries
        INTEGER         NNDL    !  number of DOTNAMES-list entries
        INTEGER         NNCL    !  number of CRONAMES-list entries
        INTEGER         GUNIT   !  unit number for GRIDDESC

        LOGICAL         EFLAG   !  for detecting/accumulating errors

        INTEGER         IREC    !  from CURREC()
        INTEGER         JSTAT   !  status return

        INTEGER         JDATE   !  current simulation date, YYYDDD
        INTEGER         JTIME   !  current simulation time, HHMMSS
        INTEGER         KDATE   !  scratch date, YYYDDD
        INTEGER         KTIME   !  scratch time, HHMMSS
        INTEGER         KSTEP   !  scratch HHMMSS
        INTEGER         ICC, IYY, IMON, MDAY, IHH

        LOGICAL         DOTFILES        !  will output some dot-pt file
        LOGICAL         CROFILES        !  will output some cro-pt file
        LOGICAL         KFFILES         !  will output some kf-event file
        LOGICAL         SCFILES         !  will output some HTR file
        SAVE            DOTFILES, CROFILES, KFFILES, SCFILES

        LOGICAL         GRDDFLAG        !  GRIDDESC file exists
        LOGICAL         DOTNFLAG        !  dot-point-output-grid-names list exists
        LOGICAL         CRONFLAG        !  cro- "

        REAL		ALP, BTA, GAM, XC, YC !  grid-calc scr-vbles
        REAL		XX0             !  X for coarse-dot-grid (1,1)
        REAL		YY0             !  Y for coarse-dot-grid (1,1)
        REAL		XXC             !  X for coarse-dot-grid (1,1)
        REAL		YYC             !  Y for coarse-dot-grid (1,1)
        REAL		XXN             !  X for current window
        REAL		YYN             !  Y for current window
        REAL		GGC             !  1/nest-rato for coarse-grid
        REAL		DX0             !  coarse-grid DX (m)
        REAL		RIRAX           !  float( nest ratio )

        REAL		T, GG, XX, YY, DXX !  grid-calculation scr-vbles

        REAL		ABUF( MIX * MJX )		!  output buffer
        REAL		BBUF( MIX * MJX )		!  output buffer
        REAL		CBUF( MIX * MJX )		!  output buffer
        REAL		DBUF( MIX * MJX )		!  output buffer
        REAL		EBUF( MIX * MJX )		!  output buffer

        REAL		QBUF( MIX * MJX * MKX )		!  output buffer
        REAL		RBUF( MIX * MJX , MKX )		!  output buffer
        REAL		SBUF( MIX * MJX * MKX )		!  output buffer
        REAL		TBUF( MIX * MJX * MKX )		!  output buffer
        REAL		UBUF( MIX * MJX * MKX )		!  output buffer
        REAL		VBUF( MIX * MJX * MKX )		!  output buffer
        REAL		WBUF( MIX * MJX * MKX )		!  output buffer
        REAL		XBUF( MIX * MJX * MKX )		!  output buffer
        REAL		YBUF( MIX * MJX * MKX )		!  output buffer
        REAL		ZBUF( MIX * MJX * MKX )		!  output buffer

        CHARACTER*16    CRDNAM  !  coordinate system name
        CHARACTER*16    NAMBUF  !  scratch name variable
        CHARACTER*16    SCRBUF  !  scratch name variable

        CHARACTER*80    DESC    !  for ckvble()
        CHARACTER*80    DESCE   !  for HTR-file envstr()
        CHARACTER*256   MESG    !  for m3msg2(), m3exit(), etc.
        CHARACTER*256   EQNAME  !  for nameval()


C...........   Floating-point "not-approximately-equal" function:

      REAL*8		P
      REAL		Q
      LOGICAL   	FLTERR
      FLTERR( P, Q ) = ( ( P - Q )**2 .GT. 2.0E-10*( P*P + Q*Q ) )


C***********************************************************************
C   begin body of subroutine  MCPL
C...............   Should never call MCPL() directly:

        MESG = 'Incorrect direct call to MCPL():  ' //
     &  'call MCPL_GRID(), then MCPL_OUT(NEST) instead'
        CALL M3EXIT( 'MM5:MCPL', 0, 0, MESG, 2 )


C......................................................................
C...............   Grid Initialization Segment:

        ENTRY MCPL_GRID

        IF ( FIRSTIMEG ) THEN                  !  if very first call:

            LOGDEV = INIT3()

            CALL M3MSG2(  ' ' )
            CALL M3MSG2(
     &'This program uses the MCPL output module for MM5 (C) 2003 BAMS')
            CALL M3MSG2(
     &'This work is distributed under version 2.1 of the')
            CALL M3MSG2(
     &'GNU Lesser General Public License, itself ' //
     &'copyright (C) 1991,1999 )' )
            CALL M3MSG2(
     &'by the Free Software Foundation, Inc.  ' //
     &'The LGPL may be found at' )
            CALL M3MSG2(  'http://www.gnu.org/copyleft/lesser.html' )
            CALL M3MSG2(  ' ' )

            IF ( INHYD .NE. 1 ) THEN
                CALL M3EXIT( 'MM5:MCPL', 0, 0,
     &          'Nonhyrostatic required for MCPL() module.', 2 )
            ELSE IF ( MAXNES .GT. MXFIL ) THEN
                WRITE( MESG, '( A, I3 )' )
     &          'Number MAXNES=', MAXNES,
     &          'of MM5 nests exceeds MCPL dimension MXFIL=', MXFIL
                CALL M3WARN( 'MM5:MCPL_GRID', 0, 0,  MESG )
            END IF

            FIRSTIMEG = .FALSE.

C.......   Get vertical grid structure (recall that for MM5,
C.......   layer K goes _down_ whereas for IOAPI, layer L goes _up_)

            VGTOP = 10.0*PTOP           !  from /param3/ cb ~~> mb
            NLAYS = MKX

            DO K = 1, MKX
                SIGH( K ) = 0.5 * ( SIGMA( K ) + SIGMA( K+1 ) )
            END DO

C.......   Get time-step structure:  defaults come from MM5 MDATE
C.......   (which is missing the century, minutes, and seconds fields),
C.......   TAPFRC, and the wall-clock JDATE:JTIME from  GETDTTIME().
C.......   These may be overriden by environment variables
C.......   MCPL_SDATE, MCPL_STIME, MCPL_TSTEP, MCPL_EDATE, and MCPL_ETIME.
C.......   MDATE0:MTIME0 is the starting date&time for the MM5 run
C.......   (after century-field fixup)

            DESC = '-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-' //
     &             '=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-'
            CALL M3MSG2( DESC )

#ifdef   MM5_V3
            READ( CDATE(  1: 4 ), '( I4 )' ) IYY
            READ( CDATE(  6: 7 ), '( I2 )' ) IMON
            READ( CDATE(  9:10 ), '( I2 )' ) MDAY
            READ( CDATE( 12:13 ), '( I2 )' ) IHH
            READ( CDATE( 15:16 ), '( I2 )' ) K
            READ( CDATE( 18:19 ), '( I2 )' ) L
            MDATE0 = 1000 * IYY + JULIAN( IYY, IMON, MDAY )     ! yyyyddd
            MTIME0 = 10000 * IHH  +  100*K  +  L                ! hhmmss
            WRITE( MESG, '( A , I9.7, A, I6.6 )' )
     &      'MCPL:  MM5 starting date and time', MDATE0, ':', MTIME0
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &      '(', DT2STR( MDATE0, MTIME0 ), '); mm5 date=',  CDATE
            CALL M3MSG2( MESG )
#endif   /**  ifdef MM5_V3  **/

#ifndef  MM5_V3   /**  Assume MM5_V2:  **/
            CALL GETDTTIME( JDATE, JTIME )
            ICC = JDATE / 100000		!  starting century
            IYY  =      MDATE            / 1000000
            IMON = MOD( MDATE, 1000000 ) / 10000
            MDAY = MOD( MDATE, 10000   ) / 100
            IHH  = MOD( MDATE, 100     )

            IYY = 100 * ICC + IYY

            MDATE0 = 1000 * IYY + JULIAN( IYY, IMON, MDAY )     ! yyyyddd
            MTIME0 = 10000 * IHH   !  + 100*0  +  0  as hhmmss
            WRITE( MESG, '( A , I9.7, A, I6.6 )' )
     &      'MCPL:  MM5 starting date and time', MDATE0, ':', MTIME0
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 3 A, I10 )' )
     &      '(', DT2STR( MDATE0, MTIME0 ), '); mm5 date=',  MDATE
            CALL M3MSG2( MESG )
#endif   /**  ifndef MM5_V3 -- MM5_V2 **/

            CALL M3MSG2( DESC )
            DESC = ' '

            TSTEP( 1 ) = SEC2TIME( NINT( 60.0 * TAPFRQ ) )

            IF ( INTLIST( 'MCPL_SDATE','Starting DATEs (YYYYDDD)',
     &                    MXFIL, NNG, SDATE ) ) THEN
                DO  L = NNG+1, MXFIL
                    SDATE( L ) = SDATE( NNG )
                END DO
            ELSE
                WRITE( MESG, '( A, I10 )' )
     &            'No environment variable "MCPL_SDATE"--using default',
     &            MDATE0
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                DO  L = 1, MXFIL
                    SDATE( L ) = MDATE0
                END DO
            END IF              !  if INTLIST failed

            IF ( INTLIST( 'MCPL_STIME','Starting TIMEs (HHMMSS)',
     &                    MXFIL, NNG, STIME ) ) THEN
                DO  L = NNG+1, MXFIL
                    STIME( L ) = STIME( NNG )
                END DO
            ELSE
                WRITE( MESG, '( A, I10 )' )
     &            'No environment variable "MCPL_STIME"--using default',
     &            MTIME0
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                DO  L = 1, MXFIL
                    STIME( L ) = MTIME0
                END DO
            END IF              !  if INTLIST failed


            IF ( INTLIST( 'MCPL_TSTEP','TIME STEPs (HHMMSS)',
     &                    MXFIL, NNG, TSTEP ) ) THEN
                DO  L = NNG+1, MXFIL
                    TSTEP( L ) = TSTEP( NNG )
                END DO
            ELSE
                WRITE( MESG, '( A, I10 )' )
     &          'No environment variable "MCPL_TSTEP"--using default',
     &          TSTEP( 1 )
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                DO  L = 2, MXFIL
                    TSTEP( L ) = TSTEP( 1 )
                END DO
            END IF              !  if INTLIST failed

            IF ( INTLIST( 'MCPL_EDATE','Ending DATEs (YYYYDDD)',
     &                    MXFIL, NNG, EDATE ) ) THEN
                DO  L = NNG+1, MXFIL
                    EDATE( L ) = EDATE( NNG )
                END DO
            ELSE
                MESG ='No environment variable "MCPL_EDATE"'
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                DO  L = 1, MXFIL
                    EDATE( L ) = 9999999
                END DO
            END IF              !  if INTLIST failed

            IF ( INTLIST( 'MCPL_ETIME','Ending TIMEs (HHMMSS)',
     &                    MXFIL, NNG, ETIME ) ) THEN
                DO  L = NNG+1, MXFIL
                    ETIME( L ) = ETIME( NNG )
                END DO
            ELSE
                WRITE( MESG, '( A, I10 )' )
     &            'No environment variable "MCPL_ETIME"--using default'
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                DO  L = 1, MXFIL
                    ETIME( L ) = 235959
                END DO
            END IF              !  if INTLIST failed

            !  "Normalize" the SDATE:STIME, EDATE:ETIME values.
            !  e.g., this turns 1999123:253000 into 1999124:013000

            DO  L = 1, MXFIL
                CALL NEXTIME( SDATE( L ), STIME( L ), 0 )
                IF ( EDATE( L ) .NE. 9999999 ) THEN
                    CALL NEXTIME( EDATE( L ), ETIME( L ), 0 )
                    ENDREC( L ) = CURREC( EDATE( L ), ETIME( L ),
     &                                    SDATE( L ), STIME( L ),
     &                                    TSTEP( L ), JDATE, JTIME )
                ELSE
                    ENDREC( L ) = 2 147 400 000      !  almost 2**31, is huge
                END IF
            END DO
            K = SDATE( 1 ) / 100000  !  century field from SDATE
            IF ( ICC .NE. K ) THEN
                MDATE0 = 100000 * K + MOD( MDATE0, 100000 )
            END IF


C...............   MM5-nest to Output-File-Grid Structure:
C.......   Check/Get output dot and cross-pt grid names, offsets, dims, etc.

            IF ( .NOT. INTLIST( 'EM_OUTGRIDS',
     &                          'MM5 domains for MCPL output grids',
     &                          MXFIL, NNG, MM5DOMN ) ) THEN
                MESG = 'No environment variable "EM_OUTGRIDS"' //
     &                 '--using one output grid per MM5 nest'
                CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                NNG = 0
                GRIDCUM( 0 ) = 0
                DO  L = 1, MXFIL
                    MM5DOMN( L ) = L
                    GRIDCUM( L ) = L
                END DO
            ELSE
                GRIDCNT = NNG
                GRIDCUM( 0 ) = 0
                GRIDCUM( 1 ) = 1
                L = MM5DOMN( 1 )
                J = 1
                DO  I = 2, NNG
                    IF ( L .EQ. MM5DOMN( I ) ) THEN
                        GRIDCUM( J ) = GRIDCUM( J )  + 1
                    ELSE
                        J = J + 1
                        L = MM5DOMN( I )
                        GRIDCUM( J ) = GRIDCUM( J-1 )  + 1
                    END IF
                END DO
            END IF              !  if INTLIST failed or not

C...............   Horizontal Grid Structure:
C.......   Check/Get output dot and cross-pt grid names, offsets, dims

            DO  L = 1, MXFIL
                WRITE( CNAME( L ), '( A, I2.2 )' ) 'MCPL_CRO', L
                WRITE( DNAME( L ), '( A, I2.2 )' ) 'MCPL_DOT', L
            END DO

            IF ( .NOT. INTLIST( 'EM_I0','Row offsets of OUTPT grids',
     &                          MXFIL, NI0, I0 ) ) THEN
                CALL M3EXIT( 'MM5:MCPL', 0, 0,
     &          'Bad environment variable "EM_IO"', 2 )
            END IF              !  if INTLIST failed

            IF ( .NOT. INTLIST( 'EM_J0','Row offsets of OUTPT grids',
     &                          MXFIL, NJ0, J0 ) ) THEN
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0,
     &          'Bad environment variable "EM_JO"', 2 )
            END IF              !  if INTLIST failed

            IF ( .NOT. INTLIST( 'EM_NROWS',
     &                          'Row dimension of cross-pt OUTPT grids',
     &                          MXFIL, NNR, NRCRO ) ) THEN
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0,
     &          'Bad environment variable "EM_NROWS"', 2 )
            END IF              !  if INTLIST failed

            IF ( .NOT. INTLIST( 'EM_NCOLS',
     &                          'Col dimension of cross-pt OUTPT grids',
     &                          MXFIL, NNC, NCCRO ) ) THEN
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0,
     &          'Bad environment variable "EM_NCOLS"', 2 )
            END IF              !  if INTLIST failed

            IF ( .NOT. INTLIST( 'EM_NTHIK',
     &                          'Col dimension of cross-pt OUTPT grids',
     &                          MXFIL, NNT, NTHIK ) ) THEN
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0,
     &          'Bad environment variable "EM_NTHIK"', 2 )
            END IF              !  if INTLIST failed

            CRONFLAG = STRLIST( 'CRONAMES',
     &                 'CHAR*16 names for the cross-pt OUTPT grids',
     &                 MXFIL, NNCL, CNAME )

            DOTNFLAG = STRLIST( 'DOTNAMES',
     &                 'CHAR*16 names for the dot-pt OUTPT grids',
     &                 MXFIL, NNDL, DNAME )

            IF ( CRONFLAG .AND. DOTNFLAG ) THEN

                IF ( NI0 .NE. NNDL  .OR.  NI0 .NE. NNCL ) THEN
                    WRITE( MESG, '( 3( A, I3, 2X ) )' )
     &              'Environment list-count mismatch:  EM_IO', NI0,
     &              'DOTNAMES', NNDL,
     &              'CRONAMES', NNCL
                    CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
                END IF

            ELSE

                IF ( .NOT. STRLIST( 'OUTGNAMES',
     &                              'CHAR*12 root names: OUTPT grids',
     &                              MXFIL, NNO, GNAME ) ) THEN
                    CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0,
     &              'Bad environment variable "OUTGNAMES"', 2 )
                END IF              !  if strlist failed
                IF ( DOTNFLAG  .AND. ( NI0 .NE. NNDL ) ) THEN
                    WRITE( MESG, '( 2( A, I3, 2X ) )' )
     &              'Environment list-count mismatch:  EM_IO', NI0,
     &              'DOTNAMES', NNDL
                    CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
                ELSE IF ( CRONFLAG  .AND. ( NI0 .NE. NNCL ) ) THEN
                    WRITE( MESG, '( 2( A, I3, 2X ) )' )
     &              'Environment list-count mismatch:  EM_IO', NI0,
     &              'CRONAMES', NNCL
                    CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
                END IF                  !  if dot or cross count mismatch

            END IF                      !  if dotnflag and cronflag, or not

            IF ( NNG .EQ. 0 ) THEN
                NNG = NI0
                GRIDCNT = NI0
            END IF
            IF ( NNG .NE. NI0  .OR.
     &           NI0 .NE. NJ0  .OR.
     &           NI0 .NE. NNR  .OR.
     &           NI0 .NE. NNC  .OR.
     &           NI0 .NE. NNT  .OR.
     &           NI0 .NE. NNO ) THEN
                WRITE( MESG, '( 6( A, I3, 2X ) )' )
     &              'Environment list-count mismatch:  EM_IO', NI0,
     &              'EM_J0',    NJ0,
     &              'EM_NROWS', NNR,
     &              'EM_NCOLS', NNC,
     &              'EM_NTHIK', NNT,
     &              'OUTGNAMES', NNO
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
            END IF

            DO  N = 1, NI0
                IF ( CRONFLAG ) THEN
                ELSE
                    WRITE( CNAME( N ), '( 2 A )' )
     &              GNAME( N )( 1:TRIMLEN( GNAME( N ) ) ), '_CRO'
                END IF
                IF ( DOTNFLAG ) THEN
                ELSE
                    WRITE( DNAME( N ), '( 2 A )' )
     &              GNAME( N )( 1:TRIMLEN( GNAME( N ) ) ), '_DOT'
                END IF
                TURNOFF( N ) = ( ( CNAME( N ) .EQ. 'NONE ' ) .OR.
     &                           ( DNAME( N ) .EQ. 'NONE ' ) .OR.
     &                           ( GNAME( N ) .EQ. 'NONE ' ) )
                IF ( TURNOFF( N ) ) THEN
                    WRITE( MESG, '( A, I2, 2X, A )' )
     &              'MCPL():  Nest', N, 'turned off'
                END IF
            END DO

            DO  N = NI0 + 1, MXFIL
                CNAME( N ) = BLANK
                DNAME( N ) = BLANK
                GNAME( N ) = BLANK
                TURNOFF( N ) = .TRUE.
            END DO


C...........   Check/Get horizontal output grid names and structure
C...........   Two cases:  GRIDDESC already exists, or not:

            CALL NAMEVAL( 'GRIDDESC', EQNAME )
            INQUIRE( FILE = EQNAME, EXIST = GRDDFLAG, IOSTAT = JSTAT )
            IF ( JSTAT .NE. 0 ) THEN
                WRITE( MESG, '( A, I7 )' )
     &          'INQUIRE failure:  GRIDDESC.  Status = ', JSTAT
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
            END IF

C.......   Coarse domain structure:

#ifdef MM5_V3
            DX0  = BHR( 1,1 ) ! coarse dx: mm5v3 already uses m.
            JMAX = BHI(6,1)   ! coarse domain J (x) dimension
            IMAX = BHI(5,1)   ! coarse domain I (y) dimension
            XX0  = -0.5 * DX0 * FLOAT( JMAX-1 )
            YY0  = -0.5 * DX0 * FLOAT( IMAX-1 )
            XXC  = BHR(11,1) - 1
            YYC  = BHR(10,1) - 1
            GGC  = 1.0 / FLOAT( BHI(20,1) )

            IF ( BHI(7,1) .EQ. 1 ) THEN !  Lambert

                GDTYP3D = LAMGRD3    ! from PARMS3.EXT
                XC  = BHR(3,1)       ! central longitude
                YC  = BHR(2,1)       ! central latitude
                GAM = XC             ! same as XCENT3D for MM5
                IF ( BHR(5,1) .GT. BHR(6,1) ) THEN
                    BTA = BHR(5,1)   ! true latitude 1 (degrees)
                    ALP = BHR(6,1)   ! true latitude 2 (degrees)
                ELSE
                    ALP = BHR(5,1)   ! true latitude 2 (degrees)
                    BTA = BHR(6,1)   ! true latitude 1 (degrees)
                END IF

            ELSE IF ( BHI(7,1) .EQ. 2 ) THEN    !  Polar Stereographic

                GDTYP3D = POLGRD3    ! from PARMS3.EXT
                XC  = BHR(3,1)       ! central longitude
                YC  = BHR(2,1)       ! central latitude
                IF ( BHR(7,1) .GT. 0.0 ) THEN   !  North polar
                   ALP = 1.0D0
                ELSE
                   ALP = -1.0D0
                END IF
                BTA = BHR(5,1)       ! true latitude 1 (degrees)
                GAM = XC             ! same as XCENT3D for MM5

            ELSE IF ( BHI(7,1) .EQ. 3 ) THEN    !  Equatorial Mercator

                GDTYP3D = EQMGRD3    ! from PARMS3.EXT
                XC  = BHR(3,1)       ! central longitude
                YC  = BHR(2,1)       ! central latitude
                ALP = 0.0
                BTA = 0.0
                GAM = XC

            ELSE

                WRITE( MESG, '( A, I5, 2X, A )' )
     &          'MM5 map projection type', BHI(7,1),'not yet supported'
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )

            END IF
#endif                                          /** ifdef MM5_V3 **/

#ifndef MM5_V3                                  /**  MM5V2:  **/
            DX0  = 1000.0 * MRF( 1,1 ) ! coarse dx: convert from km. to m.
            JMAX = MIF(3,1)   ! coarse domain J (x) dimension
            IMAX = MIF(2,1)   ! coarse domain I (y) dimension
            XX0  = -0.5 * DX0 * FLOAT( JMAX-1 )
            YY0  = -0.5 * DX0 * FLOAT( IMAX-1 )
            XXC  = MRF(103,1) - 1
            YYC  = MRF(102,1) - 1
            GGC  = 1.0 / FLOAT( MIF(108,1) )
            IF ( MIF(4,1) .NE. 1 ) THEN
                WRITE( MESG, '( A, I5, 2X, A )' )
     &          'MM5 grid type', MIF(4,1), 'not yet supported'
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
            END IF

            GDTYP3D = LAMGRD3    ! from PARMS3.EXT
            XC  = MRF(3,1)       ! central longitude
            YC  = MRF(2,1)       ! central latitude
            GAM = XC             ! same as XCENT3D for MM5
            IF ( MRF(5,1) .GT. MRF(6,1) ) THEN
                BTA = MRF(5,1)   ! true latitude 1 (degrees)
                ALP = MRF(6,1)   ! true latitude 2 (degrees)
            ELSE
                ALP = MRF(5,1)   ! true latitude 2 (degrees)
                BTA = MRF(6,1)   ! true latitude 1 (degrees)
            END IF
#endif                                  /** ifndef MM5_V3:  MM5V2 **/

            RIRAX = FLOAT( IRAX )       !  float( nest ratio )

C...........   Compute and save geometry.  Do paranoia-checks.

            EFLAG = .FALSE.
            DO  N = 1, GRIDCNT     ! loop describing output grids

                XX   = 0.0  !  construct offsets for this MM5 nest
                YY   = 0.0  !  (old yistart-1, xjstart-1)
                MMN  = MM5DOMN( N )         !  subscript for "mother" domain
                M    = MMN
                DO  L = 1, LEVIDN( MMN )      !  "do 1300" in outtap()
                    K  = 1 + LEVIDN( 1 ) - LEVIDN( M )
                    GG = RIRAX**K             ! negative power of RIRAX
                    XX = XX  +  GG*FLOAT( NESTJ( M )-1 )
                    YY = YY  +  GG*FLOAT( NESTI( M )-1 )
                    M  = NUMNC( M )
                END DO

                XX = XXC  +  GGC * XX
                YY = YYC  +  GGC * YY
                K  = LEVIDN(1) - LEVIDN(MMN) ! negative exponent
                DN( N ) = DX0 * GGC * RIRAX**K
                X0( N ) = XX0  + DX0 * XX
     &                         + DN( N ) * FLOAT( J0( N ) )
                Y0( N ) = YY0  + DX0 * YY
     &                         + DN( N ) * FLOAT( I0( N ) )

                CCRO0( N ) = J0( N )    + 1     !  starting col, row
                RCRO0( N ) = I0( N )    + 1     !  for output grid within
                CDOT0( N ) = J0( N )    + 1     !  its MM5-mother (cross
                RDOT0( N ) = I0( N )    + 1     !  and dot)
                NCDOT( N ) = NCCRO( N ) + 1
                NRDOT( N ) = NRCRO( N ) + 1

                !  Paranoia:

                IF ( CCRO0( N ) .LT. 1  ) THEN
                    EFLAG = .TRUE.
                    WRITE( MESG, '( A, I4, 1X, A, A )' )
     &                  'Bad initial J =', CCRO0( N ),
     &                  'for grid ', CNAME( N )
                    CALL M3MSG2( MESG )
                END IF

                IF ( RCRO0( N ) .LT. 1  ) THEN
                    EFLAG = .TRUE.
                    WRITE( MESG, '( A, I4, 1X, A, A )' )
     &                  'Bad initial I =', RCRO0( N ),
     &                  'for grid ', CNAME( N )
                    CALL M3MSG2( MESG )
                END IF

                J = CCRO0( N )+NCCRO( N )   !  will be dot-pt dimension
                IF ( J .GT. NESTJX( MMN )  ) THEN
                    EFLAG = .TRUE.
                    WRITE( MESG, '( A, I4 )' )
     &                'MM5 J dimension: NESTJX=', NESTJX( MMN )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, 1X, A, A )' )
     &                'Bad final J =', J, 'for grid ', CNAME( N )
                    CALL M3MSG2( MESG )
                END IF

                I = RCRO0( N )+NRCRO( N )   !  will be dot-pt dimension
                IF ( I .GT. NESTIX( MMN )  ) THEN
                    EFLAG = .TRUE.
                    WRITE( MESG, '( A, I4 )' )
     &                'MM5 I dimension: NESTIX=', NESTIX( MMN )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, 1X, A, A )' )
     &                'Bad final I =', I, 'for grid ', CNAME( N )
                    CALL M3MSG2( MESG )
                END IF

            END DO

            IF ( EFLAG ) THEN
                MESG = 'Output grid specification errors'
                CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
            END IF

C...........   If GRIDDESC exists:   consistency-checks; else write it

            IF ( GRDDFLAG ) THEN

                DO  N = 1, GRIDCNT     ! loop describing grids

                    IF ( .NOT. DSCGRID( CNAME( N ), NAMBUF, GDTYP3D,
     &                    P_ALP3D, P_BET3D, P_GAM3D,
     &                    XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                    XCELL3D, YCELL3D,
     &                    NCOLS3D, NROWS3D, NTHIK( N )  ) ) THEN
                        MESG = 'No grid description for "' //
     &                  CNAME( N )( 1:TRIMLEN( CNAME( N ) ) ) // '"'
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF              !  if dscgrid() failed

                    IF ( NCOLS3D .NE. NCCRO( N ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad NCOLS3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, I7, : 2X ) )' )
     &                  'From MM5:', NCCRO( N ), '; GRIDDESC:', NCOLS3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( NROWS3D .NE. NRCRO( N ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad NROWS3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, I7, : 2X ) )' )
     &                  'From MM5:', NRCRO( N ), '; GRIDDESC:', NROWS3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( XCELL3D, DN( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad XCELL3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', DN( N ), '; GRIDDESC:', XCELL3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( YCELL3D, DN( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad YCELL3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', DN( N ), '; GRIDDESC:', YCELL3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_ALP3D, ALP ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_ALP3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', ALP, '; GRIDDESC:', P_ALP3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_BET3D, BTA ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_BET3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', BTA, '; GRIDDESC:', P_BET3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_GAM3D, GAM ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_GAM3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', GAM, '; GRIDDESC:', P_GAM3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( XORIG3D, X0( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad XORIG3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', X0( N ), '; GRIDDESC:', XORIG3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( YORIG3D, Y0( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad YORIG3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', Y0( N ), '; GRIDDESC:', YORIG3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( .NOT. DSCGRID( DNAME( N ), NAMBUF, GDTYP3D,
     &                    P_ALP3D, P_BET3D, P_GAM3D,
     &                    XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                    XCELL3D, YCELL3D,
     &                    NCOLS3D, NROWS3D, K  ) ) THEN
                        MESG = 'No grid description for "' //
     &                  DNAME( N )( 1:TRIMLEN( DNAME( N ) ) ) // '"'
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF              !  if dscgrid() failed

                    IF ( NCOLS3D .NE. NCDOT( N ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad NCOLS3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, I7, : 2X ) )' )
     &                  'From MM5:', NCDOT( N ), '; GRIDDESC:', NCOLS3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( NROWS3D .NE. NRDOT( N ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad NROWS3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, I7, : 2X ) )' )
     &                  'From MM5:', NRDOT( N ), '; GRIDDESC:', NROWS3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( XCELL3D, DN( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad XCELL3D for grid ' // DNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', DN( N ), '; GRIDDESC:', XCELL3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( YCELL3D, DN( N ) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad YCELL3D for grid ' // DNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', DN( N ), '; GRIDDESC:', YCELL3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_ALP3D, ALP ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_ALP3D for grid ' // DNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', ALP, '; GRIDDESC:', P_ALP3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_BET3D, BTA ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_BET3D for grid ' // DNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', BTA, '; GRIDDESC:', P_BET3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( P_GAM3D, GAM ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad P_GAM3D for grid ' // DNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', GAM, '; GRIDDESC:', P_GAM3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( XORIG3D+0.5D0*XCELL3D, X0(N) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad XORIG3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', X0( N ), '; GRIDDESC:', XORIG3D
                        CALL M3MSG2( MESG )
                    END IF

                    IF ( FLTERR( YORIG3D+0.5D0*YCELL3D, Y0(N) ) ) THEN
                        EFLAG = .TRUE.
                        MESG = 'Bad YORIG3D for grid ' // CNAME( N )
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 2 (A, 1PE16.6, : 2X ) )' )
     &                  'From MM5:', Y0( N ), '; GRIDDESC:', YORIG3D
                        CALL M3MSG2( MESG )
                    END IF

                END DO    !  end loop describing grids (case GRIDDESC exists)

                IF ( EFLAG ) THEN
                    MESG = 'Grid specification errors'
                    CALL M3EXIT( 'MM5:MCPL_GRID', 0, 0, MESG, 2 )
                END IF

            ELSE        !  else create GRIDDESC file and output header

                CALL ENVSTR( 'CRDNAM',
     &                       'Coordinate name for current model',
     &                       'MCPL_OUT', CRDNAM, JSTAT )

                IF ( JSTAT .GT. 0 ) THEN
                    MESG = 'Bad environment variable "CRDNAM"'
                    CALL M3EXIT( 'MM5/MCPL_GRID', 0, 0, MESG, 2 )
                END IF          !  if jstat > 0

                GUNIT = GETEFILE('GRIDDESC',.FALSE.,.TRUE.,'MM5/MCPL')
                WRITE( GUNIT, '( 2 A ) ' )
     &              '!  coords --line:  name; type, ',
     &              'P-alpha, P-beta, P-gamma, xcent, ycent'

                WRITE( GUNIT, '( 1X, 3 A ) ' )
     &              QUOTE, CRDNAM( 1:TRIMLEN( CRDNAM ) ), QUOTE

                WRITE( GUNIT, '( 1X, I5, 5( 2X, 1PD15.7 ) ) ' )
     &              GDTYP3D, ALP, BTA, GAM, XC, YC

                WRITE( GUNIT, '( 1X, 5 A ) ' )
     &              QUOTE, BLANK, QUOTE,
     &              ' !  end coords.  grids:  name; ',
     &              'xorig, yorig, xcell, ycell, ncols, nrows, nthik'


C.......   Describe dot-point output grids (subscripted by grid N) and
C.......   output descriptive lines to GRIDDESC file:

                EFLAG = .FALSE.
                DFAC  = 2.0D0 * DBLE( IRAX )**2
                DINV  = 1.0D0 / DFAC

                DO  N = 1, GRIDCNT     ! loop writing GRIDDESC

                    !  Compute double-precision enhancements of grid parms:

                    XCELL3D = DINV * DBLE( NINT( DFAC * DN( N ) ) )
                    YCELL3D = XCELL3D
                    XORIG3D = DINV * DBLE( NINT( DFAC * X0( N ) ) )
                    YORIG3D = DINV * DBLE( NINT( DFAC * Y0( N ) ) )

                    !  GRIDDESC line for cross-pt output grid:

                    GDNAM3D = CNAME( N )
                    WRITE( GUNIT, '( 1X, 3 A ) ' )
     &                  QUOTE, GDNAM3D( 1:TRIMLEN( GDNAM3D ) ), QUOTE
                    WRITE( GUNIT,
     &                     '( 1X, 3A, 4(1X, 1PE15.7), 3(1X, I5) )' )
     &                  QUOTE, CRDNAM( 1:TRIMLEN( CRDNAM ) ), QUOTE,
     &                  XORIG3D, YORIG3D,
     &                  XCELL3D, YCELL3D,
     &                  NCCRO( N ), NRCRO( N ), NTHIK( N )

                    !  GRIDDESC line for dot-pt output grid:

                    M = MIN( 6, MAX( 0, GDTYP3D ) )
                    GDNAM3D = DNAME( N )
                    WRITE( GUNIT, '( 1X, 3 A ) ' )
     &                  QUOTE, GDNAM3D( 1:TRIMLEN( GDNAM3D ) ), QUOTE
                    WRITE( GUNIT,
     &                     '(1X, 3A, 4( 1X, 1PE15.7), 3(1X, I5) )' )
     &                  QUOTE, CRDNAM( 1:TRIMLEN( CRDNAM ) ), QUOTE,
     &                  XORIG3D - 0.5D0 * XCELL3D,
     &                  YORIG3D - 0.5D0 * YCELL3D,
     &                  XCELL3D, YCELL3D,
     &                  NCDOT( N ), NRDOT( N ), NTHIK( N )

                END DO       !  end loop writing GRIDDESC

                WRITE( GUNIT, '( 1X, 5 A ) ' )
     &                  QUOTE, BLANK, QUOTE, BLANK, '!  end grids.'
                CLOSE( GUNIT )

            END IF                  !  if grddflag:  GRIDDESC exists, or not

C.......   Check & log window definitions for the output grids:

            DO  N = 1, GRIDCNT     ! loop describing output grids

                    !  Is this a "proper window" of this MM5-domain
                    !  (needed for WSPD, DIV, VOR, M3DDCPL()

                MMN  = MM5DOMN( N )
                I = RCRO0( N )+NRCRO( N )   !  will be dot-pt dimension
                J = CCRO0( N )+NCCRO( N )   !  will be dot-pt dimension
                WNDWFLAG( N ) = ( ( CCRO0( N ) .GT. 1    ) .AND.
     &                            ( RCRO0( N ) .GT. 1    ) .AND.
     &                            ( J .LT. NESTJX( MMN ) ) .AND.
     &                            ( I .LT. NESTIX( MMN ) ) )

                BNDYFLAG( N ) =
     &                    ( ( CCRO0( N ) .GT. NTHIK( N )        ) .AND.
     &                      ( RCRO0( N ) .GT. NTHIK( N )        ) .AND.
     &                      ( J .LT. NESTJX( MMN )-NTHIK( N )+1 ) .AND.
     &                      ( I .LT. NESTIX( MMN )-NTHIK( N )+1 ) )

                OBHTFLAG( N ) = ( IBLTYP( MMN ) .EQ. 5 )
#ifdef MCPL_ADDMDL3
                OBHTFLAG( N ) = ( OBHTFLAG( N ) .OR.
     &                            ( IBLTYP( MMN ) .EQ. 2 ) .OR.
     &                            ( IBLTYP( MMN ) .EQ. 6 ) )
#endif   /** ifdef MCPL_ADDMDL3 **/

                IF ( .NOT. WNDWFLAG( N ) ) THEN
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'MM5 dimensions: (I,J)=',
     &                    NESTIX( MMN ), ',', NESTJX( MMN )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'Output starting corner: (I,J)=',
     &                    RCRO0( N ), ',', CCRO0( N )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'Output final  corner: (I,J)=',
     &                    RCRO0( N ), ',', CCRO0( N )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4 )' )
     &                'Cannot decouple winds for OUTPT grid', N
                    CALL M3MSG2( MESG )
                END IF

                IF ( .NOT. BNDYFLAG( N ) ) THEN
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'MM5 dimensions: (I,J)=',
     &                    NESTIX( MMN ), ',', NESTJX( MMN )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'Output starting corner: (I,J)=',
     &                    RCRO0( N ), ',', CCRO0( N )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4, A, I4 )' )
     &                    'Output final  corner: (I,J)=',
     &                    RCRO0( N ), ',', CCRO0( N )
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( A, I4 )' )
     &                'Cannot write bndy files for OUTPT grid', N
                    CALL M3MSG2( MESG )
                END IF

                IF ( .NOT. OBHTFLAG( N ) ) THEN
                    WRITE( MESG, '( A, I4 )' )
     &                 'Cannot write observation-height variables ' //
     &                 'for OUTPT grid', N
                    CALL M3MSG2( MESG )
                END IF

                M = MIN( 6, MAX( 0, GDTYP3D ) )
                WRITE( *, '( /5X, A, I3, 2X, 3A )' )
     &              'MCPL Grid', N, 'output grid "',
     &              CNAME( N )( 1:TRIMLEN( CNAME( N ) ) ), '"'
                WRITE( *, '( 5X, A, I4, 2X, 3A )' )
     &              'Projection type', GDTYP3D, '("',
     &              PROJNAME(M)( 1:TRIMLEN( PROJNAME(M) ) ), '")'
                WRITE( *, '( 5X, 3 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'Projection parameters:  Alpha=', ALP,
     &              'Beta=', BTA,
     &              'Gamma=', GAM
                WRITE( *, '( 5X, 2 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'Origin at lon=', XC, 'lat=', YC
                WRITE( *, '( /5X, 2 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'SW corner at X=', X0( N ), 'Y=', Y0( N )
                WRITE( *, '( 5X, 2 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'Cellsize dx=', DN( N ), 'dy=', DN( N )
                WRITE( *, '( 5X, 3 ( A, I6, 2X ), A, / )' )
     &              'Dimensions:', NCCRO( N ), 'cols',
     &              NRCRO( N ), 'rows',
     &              NTHIK( N ), 'bdy-cells'

                WRITE( *, '( /5X, A, I3, 2X, 3A )' )
     &              'MCPL Grid', N, 'output grid "',
     &              DNAME( N )( 1:TRIMLEN( DNAME( N ) ) ), '"'
                XORIG3D = X0( N ) - 0.5 * DN( N )
                YORIG3D = Y0( N ) - 0.5 * DN( N )
                WRITE( *, '( 5X, 2 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'SW corner at X=', XORIG3D, 'Y=', YORIG3D
                WRITE( *, '( 5X, 2 ( A, 2X, 1PE24.17, 2X ) )' )
     &              'Cellsize dx=', DN( N ), 'dy=', DN( N )
                WRITE( *, '( 5X, 3 ( A, I6, 2X ), A, / )' )
     &              'Dimensions:', NCDOT( N ), 'cols',
     &              NRDOT( N ), 'rows',
     &              NTHIK( N ), 'bdy-cells'

            END DO  ! end loop setting [WNDW|BNDY]FLAG and logging output grids


C.......   Get variables-lists for the output files:
C.......   First, fixup of name tables for soil-layer variables
C.......   (can't be done in DATA statements since MLX is not
C.......   "nailed down" well enough).

            IF ( ISOIL .EQ. 1 ) THEN        !!  MLX soil layers for Dudhia LSM
                M = MXMC2 - 2*MLX
                DO L = 1, MLX
                    M = M + 1
                    WRITE( NAMBUF, '( A, I1 )' ) 'SOILT', L
                    SOILTNAME( L ) = NAMBUF
                    MC2LIST ( M ) = NAMBUF
                END DO
            ELSE IF ( ISOIL .EQ. 2 ) THEN   !!  MLX soil layers for OSU LSM
                M = MXMC2 - 2*MLX
                DO L = 1, MLX
                    M = M + 1
                    N = N + 1
                    NAMBUF = 'SOILT' // OSUSOILEXT( L )
                    SOILTNAME( L ) = NAMBUF
                    MC2LIST( M )   = NAMBUF
                    M = M + 1
                    NAMBUF = 'SOILM' // OSUSOILEXT( L )
                    SOILMNAME( L ) = NAMBUF
                    MC2LIST( M )   = NAMBUF
                END DO
            ELSE IF ( ISOIL .EQ. 3 ) THEN   !!  2 layers for P-X LSM
                M = MXMC2 - 2*MLX
                DO L = 1, 2
                    M = M + 1
                    WRITE( NAMBUF, '( A, I1 )' ) 'SOILT', L
                    SOILTNAME( L ) = NAMBUF
                    MC2LIST( M )   = NAMBUF
                    M = M + 1
                    WRITE( NAMBUF, '( A, I1 )' ) 'SOILM', L
                    SOILMNAME( L ) = NAMBUF
                    MC2LIST( M )   = NAMBUF
                END DO
            END IF

            CALL GETVBLES( GB2VARS, MXGB2, GB2LIST,
     &                     NGB2, GB2NAMES, GB2FLAG(1) )

            CALL GETVBLES( GB3VARS, MXGB3, GB3LIST,
     &                     NGB3, GB3NAMES, GB3FLAG(1) )

            CALL GETVBLES( GC2VARS, MXGC2, GC2LIST,
     &                     NGC2, GC2NAMES, GC2FLAG(1) )

            CALL GETVBLES( GC3VARS, MXGC3, GC3LIST,
     &                     NGC3, GC3NAMES, GC3FLAG(1) )

            CALL GETVBLES( GD2VARS, MXGD2, GD2LIST,
     &                     NGD2, GD2NAMES, GD2FLAG(1) )

            CALL GETVBLES( MB2VARS, MXMB2, MB2LIST,
     &                     NMB2, MB2NAMES, MB2FLAG(1) )

            CALL GETVBLES( MB3VARS, MXMB3, MB3LIST,
     &                     NMB3, MB3NAMES, MB3FLAG(1) )

            CALL GETVBLES( MC2VARS, MXMC2, MC2LIST,
     &                     NMC2, MC2NAMES, MC2FLAG(1) )

            CALL GETVBLES( MC3VARS, MXMC3, MC3LIST,
     &                     NMC3, MC3NAMES, MC3FLAG(1) )

            CALL GETVBLES( MD3VARS, MXMD3, MD3LIST,
     &                     NMD3, MD3NAMES, MD3FLAG(1) )

            CALL GETVBLES( PC3VARS, MXPC3, PC3LIST,
     &                     NPC3, PC3NAMES, PC3FLAG(1) )

            CALL GETVBLES( PD3VARS, MXPD3, PD3LIST,
     &                     NPD3, PD3NAMES, PD3FLAG(1) )

            CROFILES = ( GB2FLAG(1) .OR. GB3FLAG(1) .OR.
     &                   GC2FLAG(1) .OR. GC3FLAG(1) .OR.
     &                   MB2FLAG(1) .OR. MB3FLAG(1) .OR.
     &                   MC2FLAG(1) .OR. MC3FLAG(1) .OR. PC3FLAG(1) )
            DOTFILES = ( GD2FLAG(1) .OR. MD3FLAG(1) .OR. PD3FLAG(1) )

            DO  N = 2, MXFIL
                GB2FLAG(N) = GB2FLAG(1)
                GB3FLAG(N) = GB3FLAG(1)
                GC2FLAG(N) = GC2FLAG(1)
                GC3FLAG(N) = GC3FLAG(1)
                GD2FLAG(N) = GD2FLAG(1)
                MB2FLAG(N) = MB2FLAG(1)
                MB3FLAG(N) = MB3FLAG(1)
                MC2FLAG(N) = MC2FLAG(1)
                MC3FLAG(N) = MC3FLAG(1)
                MD3FLAG(N) = MD3FLAG(1)
                PC3FLAG(N) = PC3FLAG(1)
                PD3FLAG(N) = PD3FLAG(1)
                MKAFLAG(N) = .FALSE.
                MKPFLAG(N) = .FALSE.
            END DO

            KFFILES = .FALSE.

            DESC = ' turned off for this run'  !  for warning message
            DO  N = 1, GRIDCNT   !  build file names:

                GB2FILE(N) = GB2BASE( 1:TRIMLEN( GB2BASE ) )//GTAIL(N)
                IF ( GB2FLAG( N ) ) THEN
                    DESCE = 'path name for '// GB2FILE(N)
                    CALL ENVSTR( GB2FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    GB2FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. GB2FLAG( N ) ) THEN
                        MESG = GB2FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                GB3FILE(N) = GB3BASE( 1:TRIMLEN( GB3BASE ) )//GTAIL(N)
                IF ( GB3FLAG( N ) ) THEN
                    DESCE = 'path name for '// GB3FILE(N)
                    CALL ENVSTR( GB3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    GB3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. GB3FLAG( N ) ) THEN
                        MESG = GB3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                GC2FILE(N) = GC2BASE( 1:TRIMLEN( GC2BASE ) )//GTAIL(N)
                IF ( GC2FLAG( N ) ) THEN
                    DESCE = 'path name for '// GC2FILE(N)
                    CALL ENVSTR( GC2FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    GC2FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. GC2FLAG( N ) ) THEN
                        MESG = GC2FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                GC3FILE(N) = GC3BASE( 1:TRIMLEN( GC3BASE ) )//GTAIL(N)
                IF ( GC3FLAG( N ) ) THEN
                    DESCE = 'path name for '// GC3FILE(N)
                    CALL ENVSTR( GC3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    GC3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. GC3FLAG( N ) ) THEN
                        MESG = GC3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                GD2FILE(N) = GD2BASE( 1:TRIMLEN( GD2BASE ) )//GTAIL(N)
                IF ( GD2FLAG( N ) ) THEN
                    DESCE = 'path name for '// GD2FILE(N)
                    CALL ENVSTR( GD2FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    GD2FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. GD2FLAG( N ) ) THEN
                        MESG = GD2FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                MB2FILE(N) = MB2BASE( 1:TRIMLEN( MB2BASE ) )//GTAIL(N)
                IF ( MB2FLAG( N ) ) THEN
                    DESCE = 'path name for '// MB2FILE(N)
                    CALL ENVSTR( MB2FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    MB2FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. MB2FLAG( N ) ) THEN
                        MESG = MB2FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                MB3FILE(N) = MB3BASE( 1:TRIMLEN( MB3BASE ) )//GTAIL(N)
                IF ( MB3FLAG( N ) ) THEN
                    DESCE = 'path name for '// MB3FILE(N)
                    CALL ENVSTR( MB3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    MB3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. MB3FLAG( N ) ) THEN
                        MESG = MB3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                MC2FILE(N) = MC2BASE( 1:TRIMLEN( MC2BASE ) )//GTAIL(N)
                IF ( MC2FLAG( N ) ) THEN
                    DESCE = 'path name for '// MC2FILE(N)
                    CALL ENVSTR( MC2FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    MC2FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. MC2FLAG( N ) ) THEN
                        MESG = MC2FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                MC3FILE(N) = MC3BASE( 1:TRIMLEN( MC3BASE ) )//GTAIL(N)
                IF ( MC3FLAG( N ) ) THEN
                    DESCE = 'path name for '// MC3FILE(N)
                    CALL ENVSTR( MC3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    MC3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. MC3FLAG( N ) ) THEN
                        MESG = MC3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                MD3FILE(N) = MD3BASE( 1:TRIMLEN( MD3BASE ) )//GTAIL(N)
                IF ( MD3FLAG( N ) ) THEN
                    DESCE = 'path name for '// MD3FILE(N)
                    CALL ENVSTR( MD3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    MD3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. MD3FLAG( N ) ) THEN
                        MESG = MD3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                PC3FILE(N) = PC3BASE( 1:TRIMLEN( PC3BASE ) )//GTAIL(N)
                IF ( PC3FLAG( N ) ) THEN
                    DESCE = 'path name for '// PC3FILE(N)
                    CALL ENVSTR( PC3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    PC3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. PC3FLAG( N ) ) THEN
                        MESG = PC3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    ELSE
                        IF ( NCCRO(N)*NRCRO(N)*PLAYS .GT.
     &                       MIX*MJX*MKX ) THEN
                            MESG =
     &                      'Too many layers; max size exceeded for '//
     &                             PC3FILE(N) // '; file turned off'
                            CALL M3MSG2( MESG )
                            WRITE ( MESG, '(A,I9)' ) 'PLAYS = ', PLAYS
                            CALL M3MSG2( MESG )
                            PC3FLAG( N ) = .FALSE.
                        END IF
                    END IF
                END IF

                PD3FILE(N) = PD3BASE( 1:TRIMLEN( PD3BASE ) )//GTAIL(N)
                IF ( PD3FLAG( N ) ) THEN
                    DESCE = 'path name for '// PD3FILE(N)
                    CALL ENVSTR( PD3FILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                    SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    PD3FLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    IF ( .NOT. PD3FLAG( N ) ) THEN
                        MESG = PD3FILE( N ) // DESC
                        CALL M3MSG2( MESG )
                    ELSE
                        IF ( NCDOT(N)*NRDOT(N)*PLAYS .GT.
     &                       MIX*MJX*MKX ) THEN
                            MESG =
     &                      'Too many layers; max size exceeded for '//
     &                             PD3FILE(N) // '; file turned off'
                            CALL M3MSG2( MESG )
                            WRITE ( MESG, '(A,I9)' ) 'PLAYS = ', PLAYS
                            CALL M3MSG2( MESG )
                            PD3FLAG( N ) = .FALSE.
                        END IF
                    END IF
                END IF


                HTRFILE(N) = HTRBASE( 1:TRIMLEN( HTRBASE ) )//GTAIL(N)
		DESCE = 'path name for '// HTRFILE(N)
		CALL ENVSTR( HTRFILE( N ), DESCE, 'NONE',
     &                           EQNAME, JSTAT )
                SCRBUF = EQNAME
                CALL UPCASE( SCRBUF )
                HTRFLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                IF (  HTRFLAG( N ) ) THEN
                    SCFILES = .TRUE.
                ELSE
                    MESG = HTRFILE( N ) // DESC
                    CALL M3MSG2( MESG )
                END IF


                MKAFILE(N) = MKABASE( 1:TRIMLEN( MKABASE ) )//GTAIL(N)
                CALL NAMEVAL( MKAFILE( N ), EQNAME )
                SCRBUF = EQNAME
                CALL UPCASE( SCRBUF )
                MKAFLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                IF ( MKAFLAG( N ) ) THEN
                    KFFILES = .TRUE.     ! ="kffiles .or. mkaflag( n )
                ELSE
                    MESG = MKAFILE( N ) // DESC
                    CALL M3MSG2( MESG )
                END IF

                MKPFILE(N) = MKPBASE( 1:TRIMLEN( MKPBASE ) )//GTAIL(N)
                CALL NAMEVAL( MKPFILE( N ), EQNAME )
                SCRBUF = EQNAME
                CALL UPCASE( SCRBUF )
                MKPFLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                IF (  MKPFLAG( N ) ) THEN
                    KFFILES = .TRUE.     ! ="kffiles .or. mkpflag( n )
                ELSE
                    MESG = MKPFILE( N ) // DESC
                    CALL M3MSG2( MESG )
                END IF

            END DO      ! loop up to GRIDCNT building names and flags

            IF ( PC3FLAG(1) .OR. PD3FLAG(1) .OR. MC2FLAG(1) ) THEN

                IF ( REALIST( 'MCPL_PRESF',
     &               'Pressure full-levels for PC3 and PD3 files (Pa)',
     &               MXLAYS3, L, PVALS ) ) THEN
                    PLAYS = L - 1
                    DO K = 1, PLAYS
                        PVALH(K) = 0.5 * ( PVALS(K) + PVALS(K+1) )
                    END DO
                ELSE
                    PLAYS = 17
                    DO K = 1, 17
                        PVALS(K) = PVALT(K)
                        PVALH(K) = 0.5 * ( PVALT(K) + PVALT(K+1) )
                    END DO
                    PVALS(18) = PVALT(18)
                    WRITE( MESG, '( A )' )
     &              'No environment vble "MCPL_PRESF"--using defaults'
                    CALL M3WARN( 'MM5:MCPL', 0, 0,  MESG )
                END IF              !  if INTLIST failed

                CALL M3MSG2(
     &          'Pressure levels for "PRES_CRO_3D" and "PRES_DOT_3D"')
                DO  K = 1, PLAYS
                    WRITE( MESG, '( A, I4, 2X,A, 2X,1PE14.6, 2X,A )' )
     &                    'Level', K, 'at', PVALH( K ), 'Pascals'
                    CALL M3MSG2( MESG )
                END DO
                CALL M3MSG2( ' ' )

            END IF              !  if building pressure-interpolated files

            IF ( .NOT. DOTFILES  .AND.  .NOT. CROFILES ) THEN
                LNOVARS = .TRUE.
                CALL M3MSG2( 'No variables requested for MCPL...' )
            END IF  !  if not producing any (bdy, cro-pt, dot-pt, kf) file


            IF ( KFFILES ) THEN

C..............   Initialize all KF files (MKA and MKP) first...
C..............   Must build them _here_, because KFPARA may call MCPL_KF2
C..............   before MCPL_OUT is called on the relevant MM5 domain.

                IF ( INTLIST( 'MCPL_KFTHIK',
     &                        'MAX KF-cloud event per cell',
     &                        MXFIL, NNG, KFTHIK ) ) THEN
                    DO  L = NNG+1, MXFIL
                        KFTHIK( L ) = KFTHIK( NNG )
                    END DO
                ELSE
                    WRITE( MESG, '( A, I10 )' )
     &                'No environment variable "MCPL_KFTHIK"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF              !  if INTLIST failed

                DO L = 1, MXDESC3
                    FDESC3D( L ) = ' '  ! blank descrip for i/o api files
                END DO

C..............   Get vertical grid structure (recall that for MM5,
C..............   layer K goes _down_ whereas for IOAPI, layer L goes _up_)

                VGTYP3D = VGSGPN3
                VGTOP3D = VGTOP                 !  in mb.
                NLAYS   = MKX
                DO K = 1, KXP1
                    L = NLAYS - K + 2
                    VGLVS3D( L ) = SIGMA( K )   !  from /param3/
                END DO

                CALL KF_INIT( MDATE0, MTIME0, CNAME, SDATE, STIME,
     &                        CCRO0, RCRO0, NCCRO, NRCRO, KFTHIK )

            END IF              !  if kffiles


            IF ( SCFILES ) THEN

                IF ( INTLIST( 'MCPL_HTRDATE',
     &                        'HTR_CRO_2D-file start dates',
     &                        MXFIL, NNG, HTRDATE ) ) THEN
                    DO  L = NNG+1, MXFIL
                        HTRDATE( L ) = HTRDATE( NNG )
                    END DO
                ELSE
                    WRITE( MESG, '( A, I10 )' )
     &                'No environment variable "MCPL_HTRDATE"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF              !  if INTLIST failed

                IF ( INTLIST( 'MCPL_HTRTIME',
     &                        'HTR_CRO_2D-file start times',
     &                        MXFIL, NNG, HTRTIME ) ) THEN
                    DO  L = NNG+1, MXFIL
                        HTRTIME( L ) = HTRTIME( NNG )
                    END DO
                ELSE
                    WRITE( MESG, '( A, I10 )' )
     &                'No environment variable "MCPL_HTRTIME"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF              !  if INTLIST failed

                IF ( INTLIST( 'MCPL_HTRSTEP',
     &                        'HTR_CRO_2D-file output time steps',
     &                        MXFIL, NNG, HTRSTEP ) ) THEN
                    DO  L = NNG+1, MXFIL
                        HTRSTEP( L ) = HTRSTEP( NNG )
                    END DO
                ELSE
                    WRITE( MESG, '( A, I10 )' )
     &                'No environment variable "MCPL_HTRSTEP"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF              !  if INTLIST failed

                IF ( REALIST( 'MCPL_HTRPRES',
     &                       'High-Temporal-Resolution pressure ' //
     &                       'levels (mb), from ground upward',
     &                        MKX, NPHTR, PVHTR ) ) THEN
                    NPDIM = NPHTR
                    DO L = 1, NPHTR     !  convert to Pa for MM2HTR algorithm
                        PVHTR( L ) = MBTOPA * PVHTR( L )
                    END DO		    
                ELSE
                    NPHTR = 0
                    NPDIM = 1
                END IF              !  if REALIST failed		

                MESG    = 'Number of surface layers for mask algorithm'
                NEARSFC = ENVINT( 'MCPL_NEARSFC',MESG, 2, L )
                IF ( L .GT. 0 ) THEN
                    MESG = 'Bad environment variable "MCPL_NEARSFC"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF

                MESG  = 'Ice-phase threshold for mask algorithm (deg C)'
                TKICE = 273.15 + ENVREAL( 'MCPL_TICE', MESG, -1.0, L )
                IF ( L .GT. 0 ) THEN
                    MESG = 'Bad environment variable "MCPL_TICE"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF

                MESG  = 'Liquid threshold for mask algorithm (deg C)'
                TKLIQ = 273.15 + ENVREAL( 'MCPL_TLIQ', MESG, 2.0, L )
                IF ( L .GT. 0 ) THEN
                    MESG = 'Bad environment variable "MCPL_TLIQ"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF

                MESG  =
     &           'Freezing-liquid threshold for mask algorithm (deg C)'
                TKFRZ = 273.15 + ENVREAL( 'MCPL_TFRZ', MESG, -0.25, L )
                IF ( L .GT. 0 ) THEN
                    MESG = 'Bad environment variable "MCPL_TFRZ"'
                    CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
                END IF

                DO  N = 1, GRIDCNT   !  build the SC files:

                    CALL ENVSTR( HTRFILE( N ),
     &                           'HTR file logical name, or "NONE"',
     &                           'NONE',
     &                           EQNAME, JSTAT )
                    IF ( JSTAT .GT. 0 ) THEN
	                MESG = 'Bad environment variable '//HTRFILE(N)
		        CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
	            END IF
	            SCRBUF = EQNAME
                    CALL UPCASE( SCRBUF )
                    HTRFLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
                    LASTHTR( N ) = IMISS3

                END DO          !  end loop on grids

            END IF              !  if scfiles


        END IF                  !  if FIRSTIMEG:  very first call


C...........   MM5-model coupling synchronization algorithm initialization:

        EFLAG = .FALSE.		!  true iff some synch-file active
	DO  N = 1, GRIDCNT

            SYNCHFILE( N ) = 'MCPL_SYNCH' // GTAIL( N )
            SYNCHLAST( N ) = IMISS3         ! = -9999
            CALL ENVSTR( SYNCHFILE( N ),
     &                   'Nest-synch file logical name, or "NONE"',
     &                   'NONE',
     &                   EQNAME, JSTAT )
            IF ( JSTAT .GT. 0 ) THEN
	        MESG = 'Bad environment variable ' // SYNCHFILE( N )
		CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
	    END IF
	    SCRBUF = EQNAME
            CALL UPCASE( SCRBUF )
            SYNCHFLAG( N ) = ( SCRBUF(1:5) .NE. 'NONE ' )
	    EFLAG = ( EFLAG .OR. SYNCHFLAG( N ) )

	END DO		!  end loop on N evaluating synchfile(n)

	IF ( EFLAG ) THEN        ! if some synch-file active

	    MESG = 'Synchronization record-lag for ' //
     &             'MCPL output grid model couplings (time-steps)'

	    IF ( .NOT. INTLIST( 'MCPL_SYNCH_DELAY', MESG,
     &                          MXFIL, M, SYNCHDELAY ) ) THEN
                MESG = 'Bad environment variable MCPL_SYNCH_DELAY'
                CALL M3EXIT( 'MM5:MCPL', 0, 0,  MESG, 2 )
	    END IF

            DO  N = M+1, GRIDCNT
                SYNCHDELAY( N ) = SYNCHDELAY( M )
            END DO

        END IF        ! if some synch-file active


        RETURN   !!!!!!!!!!!!!!!!  end entry MCPL_GRID  !!!!!!!!!!!!!!!


C......................................................................
C...............   File processing Segment for "normal" files:

        ENTRY MCPL_OUT( NEST )

C.......   Check to see that MCPL is ready for call to MCPL_OUT:
C.......   first execution of mcpl module sets starting date and time,
C.......   checks flags and mode of operation,  reads in control flags,
C.......   variables-list files, determines appropriate mode of
C.......   operation, sets flags, and constructs list of output
C.......   file-names:

        IF ( LNOVARS ) THEN
            RETURN
        END IF

C...........   Check that MCPL initialized, and that argument NEST
C...........   is in bounds and active

        IF ( FIRSTIMEG ) THEN         !  no MCPL_GRID() call yet
            MESG = 'Must call MCPL_GRID() before MCPL_OUT()'
            CALL M3EXIT( 'MM5/MCPL', KDATE, KTIME, MESG, 2 )
        ELSE IF ( NEST .GT. MAXNES .OR. NEST .LE. 0 ) THEN   !  nest out of range
            WRITE( MESG, '(A, I5, 2X, A, I2, A )' )
     &      'Argument NEST =', NEST, 'not in range [1:', MXFIL, ']'
            CALL M3EXIT( 'MM5:MCPL', KDATE, KTIME, MESG, 2 )
        ELSE IF ( IACTIV( NEST ) .NE. 1 ) THEN  !  if nest inactive
            RETURN
        END IF                  !  if nest out of range

        WNDWERR = .FALSE.
        BNDYERR = .FALSE.

C...........   Compute current simulation date and time KDATE:KTIME;

        KDATE = MDATE0                             !  MM5 start date
        KTIME = MTIME0                             !  MM5 start time
        KSTEP = SEC2TIME( NINT( 60.0 * XTIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( KDATE, KTIME, KSTEP )        ! current simulation clock
       

C...........   If KF files are being written, call KF_CHK which calls KF_OUT()

        IF (KFFILES) CALL KF_CHK( NEST )

C...........   If writing out KFTOP, reset top of expired clouds for this nest

        IF ( MC2KFTOP .OR. MC2KFEXP ) THEN    !
            DO  J = 1, MJX       !
            DO  I = 1, MIX
                IF ( KFTIM( I,J,NEST ) .LE. XTIME ) THEN
                     KFTOP( I,J,NEST ) = 0.0
                     KFEXP( I,J,NEST ) = 0.0
                ELSE
                     KFEXP( I,J,NEST ) = 60.0 * ( KFTIM( I,J,NEST )
     &                                          - XTIME )
                END IF
            END DO
            END DO
        END IF               !  if mc2KFTOP


C.............   loop on output grids for this MM5 domain  ..............

        DO  999  N = GRIDCUM( NEST-1 )+1, GRIDCUM( NEST )

        IF ( TURNOFF( N ) )  GO TO  999  !  if output grid N turned off

C.......  Do output for this date and time?
C.......  CURREC == -1 -- before SDATE:STIME; else is logical record #
C.......  JDATE:JTIME is current time truncated to an output timestep;
C.......  if we get a new JDATE:JTIME, continue processing output grid N;
C.......  otherwise, skip this output grid.

        IREC = CURREC( KDATE, KTIME,           !  current simulation clock
     &                 SDATE( N ), STIME( N ), TSTEP( N ),
     &                 JDATE, JTIME )           !  "left edge" of timestep

        IF ( IREC .EQ. -1 )  GO TO 999          !! before start date:time for N
        IF ( IREC .EQ. LASTREC( N ) ) GO TO 999 !! already wrote JDATE:JTIME
        IF ( IREC .GT.  ENDREC( N ) ) THEN      !! past end of output period
            TURNOFF( N ) = .TRUE.
            GO TO  999
        END IF

        LASTREC( N ) = IREC

C...............   Per-Nest Initialization segment:
C.......   first execution of mcpl module for this output grid opens
C.......   the requested output files for this grid, as determined by
C.......   the control flags and variables lists:

        IF ( FIRSTIME( N ) ) THEN         !  first time for this output grid

            IF ( ICUPA(NEST) .EQ. 1 ) THEN
                CLOUDTYPE = 'NO CUM PARAM    '
            ELSE IF ( ICUPA(NEST) .EQ. 2 ) THEN
                CLOUDTYPE = 'ANTHES-KUO      '
            ELSE IF ( ICUPA(NEST) .EQ. 3 ) THEN
                CLOUDTYPE = 'GRELL           '
            ELSE IF ( ICUPA(NEST) .EQ. 4 ) THEN
                CLOUDTYPE = 'ARAKAWA-SCHUBERT'
            ELSE IF ( ICUPA(NEST) .EQ. 5 ) THEN
                CLOUDTYPE = 'FRITSCH-CHAPPELL'
            ELSE IF ( ICUPA(NEST) .EQ. 6 ) THEN
                CLOUDTYPE = 'KAIN-FRITSCH    '
            ELSE IF ( ICUPA(NEST) .EQ. 7 ) THEN
                CLOUDTYPE = 'BETTS-MILLR DEEP'
            ELSE
                CLOUDTYPE = 'UNKNOWN(not 1-7)'
            END IF
            DO L = 1, MXDESC3
                FDESC3D( L ) = ' '  ! blank descrip for i/o api files
            END DO

C.......   Get vertical grid structure (recall that for MM5,
C.......   layer K goes _down_ whereas for IOAPI, layer L goes _up_)

            VGTYP3D = VGSGPN3
            VGTOP3D = VGTOP                 !  in mb.
            NLAYS   = MKX
            DO K = 1, KXP1
                L = NLAYS - K + 2
                VGLVS3D( L ) = SIGMA( K )   !  from /param3/
            END DO

C...........   Initialization of FDESC grid description for the
C...........   cross-point grid, used by the files which "live on" it:

            IF ( CROFILES ) THEN

                IF ( .NOT. DSCGRID( CNAME( N ), NAMBUF, GDTYP3D,
     &                    P_ALP3D, P_BET3D, P_GAM3D,
     &                    XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                    XCELL3D, YCELL3D,
     &                    NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
                    MESG = 'No grid description for "' //
     &              CNAME( N )( 1:TRIMLEN( CNAME( N ) ) ) // '"'
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF          !  if dscgrid() failed

            END IF      !  if producing some bdy or cross-pt file


C..................   Cross-Point Gridded and Boundary Files files
C...........   Initialize all time-independent files
C...........   (GB2, GB3, GC2, GC3 and GD2) first...
C...........   Flag, variable and file initializations for GB2:

            NTHIK3D = NTHIK( N )      !  restore boundary thickness

            IF ( GB2FLAG( N ) .AND. BNDYFLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'LAT', GB2LAT, M3REAL, 'deg',
     &          'latitude of the grid node -90 <= LAT <= 90',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'LON', GB2LON, M3REAL, 'deg',
     &          'longitude of the grid node -180 <= LON <= 180',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'MSFX', GB2MSFX, M3REAL, 'none',
     &          'inverse of the map factor at the grid node',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'MSFX2', GB2MSFX2, M3REAL, 'none',
     &          'square of the map factor at the grid node',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'HT', GB2HT, M3REAL, 'm',
     &                       'terrain height (m)',
     &                       NGB2, GB2NAMES )

                DESC = 'non-hydrostatic reference pressure difference'
     &                 // ' (P_surface-P_top'
                CALL CKVBLE( 'PSTAR0', GB2PSTAR0, M3REAL, 'Pa',
     &                       DESC,
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'ZZERO', GB2ZZERO, M3REAL, 'm',
     &                       'roughness length',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'TSFC0', GB2TSFC0, M3REAL, 'K',
     &          'surface temperature for reference atmosphere',
     &                       NGB2, GB2NAMES )

                CALL CKVBLE( 'PSFC0', GB2PSFC0, M3REAL, 'Pa',
     &          'surface pressure for reference atmosphere',
     &                       NGB2, GB2NAMES )

                NLAYS3D = 1
                FTYPE3D = BNDARY3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = '2D grid-boundary description'
                SDATE3D = 0
                STIME3D = 0
                TSTEP3D = 0
                GDNAM3D = CNAME( N )

                IF ( .NOT. OPEN3( GB2FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // GB2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

C...................   Write contents to  file:

                IF ( GB2LAT ) THEN
                    CALL MM2BDY( 1.0, MIX, MJX, 1, XLAT,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'LAT',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write LAT to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2lat

                IF ( GB2LON ) THEN
                    CALL MM2BDY( 1.0, MIX, MJX, 1, XLONG,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'LON',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write LON to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2lon

                IF ( GB2MSFX ) THEN
                    CALL MM2BDY( 1.0, MIX, MJX, 1, MSFX,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                        IF ( .NOT. WRITE3( GB2FILE( N ), 'MSFX',
     &                                         0, 0, RBUF ) ) THEN
                        MESG = 'Could not write MSFX to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2msfx

                IF ( GB2MSFX2 ) THEN
                    CALL MMBMSQ( MIX, MJX, 1, MSFX,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'MSFX2',
     &                                         0, 0, RBUF ) ) THEN
                        MESG = 'Could not write MSFX2 to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2msfx2

                IF ( GB2HT ) THEN
                    CALL MM2BDY( 1.0/G, MIX, MJX, 1, HT,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'HT',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write HT to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2ht

                IF ( GB2PSTAR0 ) THEN
                    CALL MM2BDY( 1.0E3, MIX, MJX, 1, PSA,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'PSTAR0',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write PSTAR0 to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2pstar0

                IF ( GB2ZZERO ) THEN
                    CALL MM2BDY( 1.0E3, MIX, MJX, 1, ZNT,
     &                           NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'ZZERO',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write ZZERO to ' //
     &                         GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gb2zzero

                !!  tsfc0, psfc0 calculated by mmbref, below.

            ELSE IF ( GB2FLAG( N ) ) THEN

                BNDYERR = .TRUE.
                MESG = 'Invalid boundary geometry for ' // GB2FILE( N )
                CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                GB2FLAG( N ) = .FALSE.

            END IF      !  if gb2flag

C............  Reference atmosphere on the boundary

            IF ( ( GB3FLAG( N ) .OR.
     &            ( GB2FLAG( N ) .AND. ( GB2TSFC0 .OR. GB2PSFC0 ) ) )
     &           .AND.  BNDYFLAG( N ) ) THEN

                    !!  On output from MMBREF
                    !!     ABUF is TSFC:  ref sfc temperature
                    !!     BBUF is PSFC:  ref sfc pressure
                    !!     RBUF is DENS:  ref density
                    !!     SBUF is ZH:    ref sigh Z
                    !!     TBUF is ZF:    ref sigf Z
                    !!     UBUF is DZF:   thickness ZF(:,:,L) - ZF(:,:,L-1)
                    !!     VBUF is TH0:   ref sigh temperature
                    !!     WBUF is TF0:   ref sigf temperature
                    !!     XBUF is PRH0:  ref sigh pressure
                    !!     YBUF is PRF0:  ref sigf pressure

                CALL MMBREF( MIX, MJX, MKX,
     &                       PSA, SIGH, SIGMA, PTOP,
     &                       P0, TLP, TS0, ROVG, R,
     &                       NCCRO( N ), NRCRO( N ),
     &                       NLAYS, NTHIK( N ),
     &                       CCRO0( N ), RCRO0( N ), 1,
     &                       ABUF, BBUF,
     &                       RBUF, SBUF, TBUF, UBUF,
     &                       VBUF, WBUF, XBUF, YBUF )

                IF ( GB2FLAG( N ) .AND. GB2TSFC0 ) THEN
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'TSFC0',
     &                                 0, 0, ABUF ) ) THEN
                        MESG = 'Could not write TSFC0 to ' //
     &                  GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB2FLAG( N ) .AND. GB2PSFC0 ) THEN
                    IF ( .NOT. WRITE3( GB2FILE( N ), 'PSFC0',
     &                                 0, 0, BBUF ) ) THEN
                        MESG = 'Could not write PSFC0 to ' //
     &                  GB2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

            END IF

C...........   Flag, variable and file initializations for GB3:

            IF ( GB3FLAG( N ) .AND. BNDYFLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'DENS0', GB3DENS0, M3REAL, 'kg/M^3',
     &          'non-hydrostatic reference density',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'ZH', GB3ZH, M3REAL, 'm',
     &          'non-hydrostatic mid-layer height above ground',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'ZF', GB3ZF, M3REAL, 'm',
     &          'non-hydrostatic layer-top height above ground',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'DZZ', GB3DZZ, M3REAL, 'm',
     &          'non-hydrostatic layer thickness',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'TH0', GB3TH0, M3REAL, 'K',
     &          'non-hydrostatic ref half-level temperature',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'TF0', GB3TF0, M3REAL, 'K',
     &          'non-hydrostatic ref full-level temperature',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'PRH0', GB3PRH0, M3REAL, 'Pa',
     &          'non-hydrostatic ref half-level temperature',
     &                       NGB3, GB3NAMES )

                CALL CKVBLE( 'PRF0', GB3PRF0, M3REAL, 'Pa',
     &          'non-hydrostatic ref full-level temperature',
     &                       NGB3, GB3NAMES )

                NLAYS3D = NLAYS
                FTYPE3D = BNDARY3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = '3D grid boundary description'
                SDATE3D = 0
                STIME3D = 0
                TSTEP3D = 0
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( GB3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // GB3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                IF ( GB3DENS0 ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'DENS0',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write DENS0 to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB3ZH ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'ZH',
     &                                 0, 0, SBUF ) ) THEN
                        MESG = 'Could not write ZH to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
               END IF

                IF ( GB3ZF ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'ZF',
     &                                 0, 0, TBUF ) ) THEN
                        MESG = 'Could not write ZF to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB3DZZ ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'DZZ',
     &                                 0, 0, UBUF ) ) THEN
                        MESG = 'Could not write DZZ to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
               END IF

                IF ( GB3TH0 ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'TH0',
     &                                 0, 0, VBUF ) ) THEN
                        MESG = 'Could not write TH0 to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB3TF0 ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'TF0',
     &                                 0, 0, WBUF ) ) THEN
                        MESG = 'Could not write TF0 to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB3PRH0 ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'PRH0',
     &                                 0, 0, XBUF ) ) THEN
                        MESG = 'Could not write PRH0 to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GB3PRF0 ) THEN
                    IF ( .NOT. WRITE3( GB3FILE( N ), 'PRF0',
     &                                 0, 0, YBUF ) ) THEN
                        MESG = 'Could not write PRF0 to ' //
     &                  GB3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

            ELSE IF ( GB3FLAG( N ) ) THEN

                BNDYERR = .TRUE.
                MESG = 'Invalid boundary geometry for ' // GB3FILE( N )
                CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                GB3FLAG( N ) = .FALSE.

            END IF      !  if gb3flag


C...........   Flag, variable and file initializations for GC2:

            IF ( GC2FLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'LAT', GC2LAT, M3REAL, 'deg',
     &          'latitude of the grid node -90 <= LAT <= 90',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'LON', GC2LON, M3REAL, 'deg',
     &         'longitude of the grid node -180 <= LON <= 180',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'MSFX', GC2MSFX, M3REAL, 'none',
     &          'inverse of the map factor at the grid node',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'MSFX2', GC2MSFX2, M3REAL, 'none',
     &          'square of the map factor at the grid node',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'HT', GC2HT, M3REAL, 'm',
     &                       'terrain height (m)',
     &                       NGC2, GC2NAMES )

                DESC = 'non-hydrostatic reference pressure difference'
     &                 // ' (P_surface-P_top)'
                CALL CKVBLE( 'PSTAR0', GC2PSTAR0, M3REAL, 'Pa',
     &                       DESC,
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'ZZERO', GC2ZZERO, M3REAL, 'm',
     &                       'roughness length',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'ALB', GC2ALB, M3REAL, 'm',
     &                       'Background albedo fraction',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'TSFC0', GC2TSFC0, M3REAL, 'K',
     &          'surface temperature for reference atmosphere',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'PSFC0', GC2PSFC0, M3REAL, 'Pa',
     &          'surface pressure for reference atmosphere',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'CAPG', GC2CAPG, M3REAL, 'J/(M^2K)',
     &                       'thermal capacity of ground slab',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'CSNROT', GC2CSNROT, M3REAL, 'none',
     &          'cosine of angle between Y axis and North',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'EF', GC2EF, M3REAL, '1/s',
     &                       'component of Coriolis parameter',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'EMISS', GC2EMISS, M3REAL, 'none',
     &                       'emissivity',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'F', GC2F, M3REAL, '1/s',
     &                       'Coriolis parameter',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'MAVAIL', GC2MAVAIL, M3REAL, 'none',
     &                       'moisture availability',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'SATBRT', GC2SATBRT, M3REAL, 'none',
     &                       'dominant land use category',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'SHC', GC2SHC, M3REAL, 'J/(M^3 K)',
     &                       'dry soil heat capacity',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'SNROT', GC2SNROT, M3REAL, 'none',
     &                       'sine of angle between Y axis and North',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'SNOWC', GC2SNOWC, M3REAL, 'none',
     &                       'snow cover:  yes=1 or no=0',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'THC', GC2THC, M3REAL, 'J/(m s K)',
     &                       'thermal inertia',
     &                       NGC2, GC2NAMES )

                CALL CKVBLE( 'TMN', GC2TMN, M3REAL, 'K',
     &          'constant temperature of infinite reservoir below slab',
     &                       NGC2, GC2NAMES )

                NLAYS3D = 1
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = '2D cross-point-grid description'
                SDATE3D = 0
                STIME3D = 0
                TSTEP3D = 0
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( GC2FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // GC2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                IF ( GC2LAT ) THEN
                    CALL MM2IO( 1.0, MIX, MJX, 1, XLAT,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'LAT',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write LAT to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2lat

                IF ( GC2LON ) THEN
                    CALL MM2IO( 1.0, MIX, MJX, 1, XLONG,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'LON',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write LON to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2lon

                IF ( GC2MSFX ) THEN
                    CALL MM2IO( 1.0, MIX, MJX, 1, MSFX,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'MSFX',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write MSFX to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2msfx

                IF ( GC2MSFX2 ) THEN
                    CALL MM2MSQ( MIX, MJX, 1, MSFX,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'MSFX2',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write MSFX2 to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2msfx2

                IF ( GC2HT ) THEN
                    CALL MM2IO( 1.0/G, MIX, MJX, 1, HT,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'HT',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write HT to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2ht

                IF ( GC2PSTAR0 ) THEN
                    CALL MM2IO( 1.0E3, MIX, MJX, 1, PSA,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'PSTAR0',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write PSTAR0 to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2pstar0

                IF ( GC2ZZERO ) THEN
                    CALL MM2IO( 1.0E3, MIX, MJX, 1, ZNT,
     &                           NCCRO( N ), NRCRO( N ), 1,
     &                           CCRO0( N ), RCRO0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'ZZERO',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write ZZERO to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2zzero

                IF ( GC2ALB ) THEN
                    CALL MM2IO( 1.0, MIX, MJX, 1, ALB,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'ALB',
     &                                 0, 0, ABUF ) ) THEN
                        MESG = 'Could not write ALB to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF		!  if gc2alb

                IF ( GC2CAPG ) THEN !  CAPG from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, CAPG,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'CAPG',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write CAPG to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2capg

                IF ( GC2CSNROT ) THEN !  COSROT from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, COSROT,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'CSNROT',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write CSNROT to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2csnrot

                IF ( GC2EF ) THEN !  EF from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, EF,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'EF',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write EF to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2ef

                IF ( GC2EMISS ) THEN !  EMISS from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, EMISS,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'EMISS',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write EMISS to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2emiss

                IF ( GC2F ) THEN !  F from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, F,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'F',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write F to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2f

                IF ( GC2MAVAIL ) THEN !  MAVAIL from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, MAVAIL,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'MAVAIL',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write MAVAIL to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2MAVAIL

                IF ( GC2SATBRT ) THEN !  SATBRT from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, SATBRT,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'SATBRT',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write SATBRT to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2SATBRT

                IF ( GC2SHC ) THEN !  SHC from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, SHC,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'SHC',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write SHC to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2shc

                IF ( GC2SNROT ) THEN !  SINROT from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, SINROT,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'SNROT',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write SNROT to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2SNROT

                IF ( GC2SNOWC ) THEN !  SNOWC from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, SNOWC,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'SNOWC',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write SNOWC to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2snowc

                IF ( GC2THC ) THEN !  THC from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, THC,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'THC',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write THC to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2thc

                IF ( GC2TMN ) THEN !  TMN from /addr2/
                    CALL MM2IO( 1.0, MIX, MJX, 1, TMN,
     &                          NCCRO( N ), NRCRO( N ), 1,
     &                          CCRO0( N ), RCRO0( N ), 1, ABUF )
                    IF ( .NOT. WRITE3( GC2FILE( N ), 'TMN',
     &                                 JDATE, JTIME, ABUF ) ) THEN
                        MESG = 'Could not write TMN to ' //
     &                  GC2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if GC2tmn

                !!  tsfc0, psfc0 calculated by mm2ref, below.

            END IF      !  if gc2flag


C...........   (4) Flag, variable and file initializations for GC3:

            IF ( GC3FLAG( N ) .OR.
     &           ( GC2FLAG( N ) .AND.
     &             ( GC2TSFC0 .OR. GC2PSFC0 ) ) ) THEN

                CALL MM2REF( MIX, MJX, MKX,
     &                       PSA, SIGH, SIGMA, PTOP,
     &                       P0, TLP, TS0, ROVG, R,
     &                       NCCRO( N ), NRCRO( N ), NLAYS,
     &                       CCRO0( N ), RCRO0( N ), 1,
     &                       ABUF, BBUF,
     &                       RBUF, SBUF, TBUF, UBUF,
     &                       VBUF, WBUF, XBUF, YBUF )

                IF ( GC2FLAG( N ) ) THEN

                    IF ( GC2TSFC0 ) THEN
                        IF ( .NOT. WRITE3( GC2FILE( N ), 'TSFC0',
     &                                     0, 0, ABUF ) ) THEN
                            MESG = 'Could not write TSFC0 to ' //
     &                      GC2FILE( N )
                            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                        END IF
                    END IF

                    IF ( GC2PSFC0 ) THEN
                        IF ( .NOT. WRITE3( GC2FILE( N ), 'PSFC0',
     &                                     0, 0, BBUF ) ) THEN
                            MESG = 'Could not write PSFC0 to ' //
     &                      GC2FILE( N )
                            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                        END IF
                    END IF

                END IF  !  if gc2flag(n)

            END IF

            IF ( GC3FLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'DENS0', GC3DENS0, M3REAL, 'kg/M^3',
     &          'non-hydrostatic reference density',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'ZH', GC3ZH, M3REAL, 'm',
     &          'non-hydrostatic mid-layer height above ground',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'ZF', GC3ZF, M3REAL, 'm',
     &          'non-hydrostatic layer-top height above ground',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'DZZ', GC3DZZ, M3REAL, 'm',
     &          'non-hydrostatic layer thickness',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'TH0', GC3TH0, M3REAL, 'K',
     &          'non-hydrostatic ref half-level temperature',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'TF0', GC3TF0, M3REAL, 'K',
     &          'non-hydrostatic ref full-level temperature',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'PRH0', GC3PRH0, M3REAL, 'Pa',
     &          'non-hydrostatic ref half-level temperature',
     &                       NGC3, GC3NAMES )

                CALL CKVBLE( 'PRF0', GC3PRF0, M3REAL, 'Pa',
     &          'non-hydrostatic ref full-level temperature',
     &                       NGC3, GC3NAMES )

                NLAYS3D = NLAYS
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = '3D cross-point-grid description'
                SDATE3D = 0
                STIME3D = 0
                TSTEP3D = 0
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( GC3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // GC3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                IF ( GC3DENS0 ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'DENS0',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write DENS0 to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GC3ZH ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'ZH',
     &                                 0, 0, SBUF ) ) THEN
                        MESG = 'Could not write ZH to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
               END IF

                IF ( GC3ZF ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'ZF',
     &                                 0, 0, TBUF ) ) THEN
                        MESG = 'Could not write ZF to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GC3DZZ ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'DZZ',
     &                                 0, 0, UBUF ) ) THEN
                        MESG = 'Could not write DZZ to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
               END IF

                IF ( GC3TH0 ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'TH0',
     &                                 0, 0, VBUF ) ) THEN
                        MESG = 'Could not write TH0 to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GC3TF0 ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'TF0',
     &                                 0, 0, WBUF ) ) THEN
                        MESG = 'Could not write TF0 to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GC3PRH0 ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'PRH0',
     &                                 0, 0, XBUF ) ) THEN
                        MESG = 'Could not write PRH0 to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

                IF ( GC3PRF0 ) THEN
                    IF ( .NOT. WRITE3( GC3FILE( N ), 'PRF0',
     &                                 0, 0, YBUF ) ) THEN
                        MESG = 'Could not write PRF0 to ' //
     &                  GC3FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF

            END IF      !  if gc3flag(n)


C...................   End time-independent cross-pt file processing.
C...................   Begin time-stepped file creation.
C...........   Flag, variable and file initializations for MB2:

            IF ( MB2FLAG( N ) .AND. BNDYFLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'PSTAR', MB2PSTAR, M3REAL, 'Pa',
     &          'hydrostatic pressure difference (P_surface -P_top)',
     &                       NMB2, MB2NAMES )

                CALL CKVBLE( 'ZNT', MB2ZNT, M3REAL, 'm',
     &                       'roughness length',
     &                       NMB2, MB2NAMES )

                NLAYS3D = 1
                FTYPE3D = BNDARY3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = 'non-layered boundary meteorology'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( MB2FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // MB2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

            ELSE IF ( MB2FLAG( N ) ) THEN

                BNDYERR = .TRUE.
                MESG = 'Invalid boundary geometry for ' // MB2FILE( N )
                CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                MB2FLAG( N ) = .FALSE.

            END IF      !  if mb2flag


C...........   Flag, variable and file initializations for MB3:

            IF ( MB3FLAG( N ) .AND. BNDYFLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'DENS', MB3DENS, M3REAL, 'kg/M^3',
     &                       'air density', NMB3, MB3NAMES )
                CALL CKVBLE( 'PRES', MB3PRES, M3REAL, 'Pa',
     &                       'air pressure', NMB3, MB3NAMES )
                CALL CKVBLE( 'TV', MB3TV, M3REAL, 'K',
     &                       'virtual temperature', NMB3, MB3NAMES )
                CALL CKVBLE( 'RH', MB3RH, M3REAL, '%',
     &              'relative humidity, 2-phase water/ice formula',
     &                       NMB3, MB3NAMES )
                CALL CKVBLE( 'RHLW', MB3RHLW, M3REAL, '%',
     &              'relative humidity, 1-phase water-only formula',
     &                       NMB3, MB3NAMES )
                CALL CKVBLE( 'RHKF', MB3RHKF, M3REAL, '%',
     &              'relative humidity, mixed-phase water/ice formula',
     &                       NMB3, MB3NAMES )

                NLAYS3D = NLAYS
                FTYPE3D = BNDARY3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = 'layered boundary meteorology'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( MB3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // MB3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

            ELSE IF ( MB3FLAG( N ) ) THEN

                BNDYERR = .TRUE.
                MESG = 'Invalid boundary geometry for ' // MB3FILE( N )
                CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                MB3FLAG( N ) = .FALSE.

            END IF      !  if mb3flag


C...........   Flag, variable and file initializations for MC2:

            IF ( MC2FLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'FAKE', MC2FAKE, M3REAL, 'none',
     &                       'dummy variable filled with 1.0E3',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PBL', MC2PBL, M3REAL, 'm',
     &                       'planetary boundary layer height',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RN', MC2RN, M3REAL, 'cm',
     &                       'cumulative non-convective rainfall',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RC', MC2RC, M3REAL, 'cm',
     &                       'cumulative convective rainfall',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PSTAR', MC2PSTAR, M3REAL, 'Pa',
     &          'hydrostatic pressure difference (P_surface -P_top)',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'TGD', MC2TGD, M3REAL, 'K',
     &                       'Skin temperature',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'USTAR', MC2USTAR, M3REAL, 'M/S',
     &                       'friction velocity',
     &                       NMC2, MC2NAMES )

                IF ( MC3FLAG( N ) ) THEN
                    !! these vbles depend upon mc3rh and mc3pres...
                    CALL CKVBLE( 'CFRACH', MC2CFRACH, M3REAL, 'none',
     &                           'MM5 high-cloud fractional coverage',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'CFRACM', MC2CFRACM, M3REAL, 'none',
     &                           'MM5 medium-cloud fractional coverage',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'CFRACL', MC2CFRACL, M3REAL, 'none',
     &                           'MM5 low-cloud fractional coverage',
     &                           NMC2, MC2NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('CFRACH', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "CFRACH" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('CFRACM', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "CFRACM" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('CFRACL', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "CFRACL" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF  !! if mc3flag(n), or not

                CALL CKVBLE( 'MOL', MC2MOL, M3REAL, '1/m',
     &                       'inverse Monin-Obukhov length',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RNET', MC2RNET, M3REAL, 'W/M^2',
     &                       'net radiation at ground',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'FSOIL', MC2FSOIL, M3REAL, 'W/M^2',
     &                       'heat flux in soil layers',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'WSTAR', MC2WSTAR, M3REAL, 'M/S',
     &                       'convective velocity',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RGRND', MC2RGRND, M3REAL, 'W/M^2',
     &                       'solar radiation at ground',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'TA', MC2TA, M3REAL, 'K',
     &                       'air temperature for layer 1',
     &                       NMC2, MC2NAMES )

                IF( IMPHYS( NEST ) .GE. 2 ) THEN
                    CALL CKVBLE( 'QV', MC2QV, M3REAL,
     &                           'g-H2O/g-dryair',
     &                       'Water vapor mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                ELSE IF ( 0 .GE. INDEX1('QV', NMC2, MC2LIST ) ) THEN
                    MESG = 'Variable "QV" not available for ' //
     &                     MC2FILE( N )
                    CALL M3MSG2( MESG )
                END IF

                IF( IMPHYS( NEST ) .GE. 3 ) THEN
                    CALL CKVBLE( 'QC', MC2QC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Cloud-water mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'QR', MC2QR, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Rain mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QC', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QC" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QR', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QR" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 5 ) THEN
                    CALL CKVBLE( 'QI', MC2QI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Cloud-ice mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'QNI', MC2QNI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Snow mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QI', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QI" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNI', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QNI" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 6 ) THEN
                    CALL CKVBLE( 'QG', MC2QG, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Graupel mixing ratio for layer 1',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'QNC', MC2QNC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Ice number concentration for layer 1',
     &                           NMC2, MC2NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QG', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QG" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNC', NMC2, MC2LIST ) ) THEN
                        MESG = 'Variable "QNC" not available for ' //
     &                         MC2FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                CALL CKVBLE( 'DENS', MC2DENS, M3REAL, 'kg/M^3',
     &                       'air density for layer 1',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PRES', MC2PRES, M3REAL, 'Pa',
     &                       'pressure for layer 1',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'TV', MC2TV, M3REAL, 'K',
     &                       'virtual temperature for layer 1',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RH', MC2RH, M3REAL, '%',
     &          'relative humidity for layer 1, two-phase water/ice',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RHLW', MC2RHLW, M3REAL, '%',
     &          'relative humidity for layer 1, one-phase water-only',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RHKF', MC2RHKF, M3REAL, '%',
     &          'relative humidity for layer 1, mixed-phase water/ice',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'QFLX', MC2QFLX, M3REAL, 'M/S kg/kg',
     &                       'kinematic latent heat flux',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'HFLX', MC2HFLX, M3REAL, 'm K/s',
     &                       'kinematic sensible heat flux',
     &                       NMC2, MC2NAMES )

                IF ( WNDWFLAG( N )  ) THEN
                    CALL CKVBLE( 'UX', MC2UX, M3REAL, 'M/S',
     &                   'Cross point U wind component for layer 1',
     &                       NMC2, MC2NAMES )
                    CALL CKVBLE( 'VX', MC2VX, M3REAL, 'M/S',
     &                   'Cross point V wind component for layer 1',
     &                       NMC2, MC2NAMES )
                    CALL CKVBLE( 'WSPD', MC2WSPD, M3REAL, 'M/S',
     &                       'wind speed for layer 1',
     &                       NMC2, MC2NAMES )
                    CALL CKVBLE( 'DIV', MC2DIV, M3REAL, '1/s',
     &                      'horizontal wind divergence',
     &                       NMC2, MC2NAMES )
                    CALL CKVBLE( 'AVOR', MC2AVOR, M3REAL, '1/s',
     &                       'absolute horizontal wind vorticity',
     &                       NMC2, MC2NAMES )
                    CALL CKVBLE( 'RVOR', MC2RVOR, M3REAL, '1/s',
     &                       'relative horizontal wind vorticity',
     &                       NMC2, MC2NAMES )
                ELSE

                    MESG =
     &              'Invalid geometry to constuct ' //
     &              'UX,VX,WSPD,DIV,VOR for ' //
     &              MC2FILE( N )
                    CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                    WNDWERR = WNDWERR .OR.
     &                        ( 0.NE.INDEX1( 'UX',   NMC2, MC2NAMES )
     &                     .OR. 0.NE.INDEX1( 'VX',   NMC2, MC2NAMES )
     &                     .OR. 0.NE.INDEX1( 'WSPD', NMC2, MC2NAMES )
     &                     .OR. 0.NE.INDEX1( 'DIV',  NMC2, MC2NAMES )
     &                     .OR. 0.NE.INDEX1( 'AVOR', NMC2, MC2NAMES )
     &                     .OR. 0.NE.INDEX1( 'RVOR', NMC2, MC2NAMES ) )

                END IF  !  if speed-divergence-vorticity calc in bounds or not

                CALL CKVBLE( 'RSD', MC2RSD, M3REAL, 'W/M^2',
     &                       'downward shortwave radiation',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'ALB', MC2ALB, M3REAL, 'none',
     &                       'albedo',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'CAPG', MC2CAPG, M3REAL, 'J/(M^2K)',
     &                       'thermal capacity of ground slab',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'CSNROT', MC2CSNROT, M3REAL, 'none',
     &          'cosine of angle between Y axis and North',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'EF', MC2EF, M3REAL, '1/s',
     &                       'component of Coriolis parameter',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'EMISS', MC2EMISS, M3REAL, 'none',
     &                       'emissivity',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'F', MC2F, M3REAL, '1/s',
     &                       'Coriolis parameter',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'GLW', MC2GLW, M3REAL, 'W/M^2',
     &                       'Longwave radiation at ground',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'GSW', MC2GSW, M3REAL, 'W/M^2',
     &                       'Shortwave radiation at ground',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'HOL', MC2HOL, M3REAL, 'none',
     &                       'PBL height / Monin-Obukhov length',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'HT', MC2HT, M3REAL, 'M^2/s^2',
     &          'geopotential:  terrain height times gravity',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'MAVAIL', MC2MAVAIL, M3REAL, 'none',
     &                       'moisture availability (0-1)',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'MSFX', MC2MSFX, M3REAL, 'none',
     &                       'inverse cross point map scale factor',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PRW', MC2PRW, M3REAL, 'cm',
     &                       'precipitable water',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PSA', MC2PSA, M3REAL, 'cb',
     &                       'P-star',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'PSAINI', MC2PSAINI, M3REAL, 'cb',
     &                       'Initial value for P-star',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'QFX', MC2QFX, M3REAL, 'W/M^2',
     &                       'MM5 latent heat flux',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'HFX', MC2HFX, M3REAL, 'W/M^2',
     &                       'MM5 sensible heat flux',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RAINC', MC2RAINC, M3REAL, 'cm',
     &                       'accumulated convective rain',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'RAINNC', MC2RAINNC, M3REAL, 'cm',
     &                       'accumulated non-convective rain',
     &                       NMC2, MC2NAMES )
                MESG = 'PBL class:  1-night stable, 2-damped turb, ' //
     &                 '3-forced conv, 4-free conv'
                CALL CKVBLE( 'REGIME', MC2REGIME, M3REAL, 'none',
     &                       MESG( 1:80 ),
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'SATBRT', MC2SATBRT, M3REAL, 'none',
     &                       'dominant land use category',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'SHC', MC2SHC, M3REAL, 'J/(M^3 K)',
     &                       'dry soil heat capacity',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'SNROT', MC2SNROT, M3REAL, 'none',
     &                       'sine of angle between Y axis and North',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'SNOWC', MC2SNOWC, M3REAL, 'none',
     &                       'snow cover',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'TGA', MC2TGA, M3REAL, 'K',
     &                       'Skin temperature',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'THC', MC2THC, M3REAL, 'J/(m s K)',
     &                       'thermal inertia',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'TMN', MC2TMN, M3REAL, 'K',
     &          'constant temperature of infinite reservoir below slab',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'UST', MC2UST, M3REAL, 'M/S',
     &                       'friction velocity',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'XLAND', MC2XLAND, M3REAL, 'none',
     &          'grid point type of surface (1=land, 2=water)',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'XLAT', MC2XLAT, M3REAL, 'deg',
     &                       'latitude',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'XLONG', MC2XLONG, M3REAL, 'deg',
     &                       'longitude',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'ZNT', MC2ZNT, M3REAL, 'm',
     &                       'roughness length',
     &                       NMC2, MC2NAMES )

                CALL CKVBLE( 'KFTOP', MC2KFTOP, M3REAL, 'm',
     &                       'Convective cloud top, or 0.0',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'KFEXP', MC2KFEXP, M3REAL, 's',
     &          'Time until expiration for KF cloud, or 0.0',
     &                       NMC2, MC2NAMES )

#ifdef MM5_V3

                IF ( OBHTFLAG( N ) ) THEN

                    CALL CKVBLE( 'QV2', MC2QV2, M3REAL, 'g-H2O/g-air',
     &                           '2 meter mixing ratio',
     &                           NMC2, MC2NAMES )
                    CALL CKVBLE( 'TA2', MC2TA2, M3REAL, 'K',
     &                           '2 meter air temperature',
     &                           NMC2, MC2NAMES )

                    IF ( WNDWFLAG( N )  ) THEN
                         CALL CKVBLE( 'UX10', MC2UX10, M3REAL, 'M/S',
     &                                '10 meter U wind component',
     &                                 NMC2, MC2NAMES )
                         CALL CKVBLE( 'VX10', MC2VX10, M3REAL, 'M/S',
     &                                '10 meter V wind component',
     &                                NMC2, MC2NAMES )
                         CALL CKVBLE( 'WSPD10', MC2WSPD10, M3REAL,
     &                                'M/S',
     &                                '10 meter wind speed',
     &                                NMC2, MC2NAMES )
                    ELSE

                        MESG =
     &                  'Invalid geometry to constuct ' //
     &                  'UX10,VX10,WSPD10 for ' //
     &                  MC2FILE( N )
                        CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                        WNDWERR = WNDWERR .OR.
     &                      ( 0.NE.INDEX1( 'UX10',   NMC2, MC2NAMES )
     &                   .OR. 0.NE.INDEX1( 'UX10',   NMC2, MC2NAMES )
     &                   .OR. 0.NE.INDEX1( 'WSPD10', NMC2, MC2NAMES ) )
                    END IF          !   if wndwflag, or not


                END IF          !   if ibltyp=5

                IF ( IBLTYP( NEST ) .EQ. 7 ) THEN

                    CALL CKVBLE( 'MOLENGTH', MC2MOLENGTH, M3REAL, 'M',
     &                           'Monin-Obukhov length',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'RADNET', MC2RADNET, M3REAL, 'W/M^2',
     &                           'net radiation at ground from PX LSM',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'ALBEDO', MC2ALBEDO, M3REAL, 'none',
     &                           'albedo from PX LSM',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'GRNFLX', MC2GRNFLX, M3REAL, 'W/M^2',
     &                           'ground heat flux from PX LSM',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'RA', MC2RA, M3REAL, 'S/M',
     &                           'Aerodynamic resistance',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'RS', MC2RS, M3REAL, 'S/M',
     &                           'Surface resistance',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'TOB1', MC2TOB1, M3REAL, 'K',
     &              'Sfc-obs temperature scaled to lowest model level',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'RHOBS', MC2RHOBS, M3REAL, '%',
     &                           'OBS. relative humidity',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'LAI', MC2LAI, M3REAL, 'M^2/M^2',
     &                           'Leaf area index',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'VEGFRC', MC2VEGFRC, M3REAL, 'M^2/M^2',
     &                           'Vegetation coverage',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'ISLTYP', MC2ISLTYP, M3INT, 'none',
     &                           'USDA soil texture type category',
     &                           NMC2, MC2NAMES )

                END IF  !  if ibltyp(nest)=7:  PX:  2D variables for PX LSM

                CALL CKVBLE( 'SNOCVR', MC2SNOCVR, M3INT, 'none',
     &                       'Snow coverage:  yes=1 or no=0',
     &                        NMC2, MC2NAMES )

                CALL CKVBLE( 'TSEASFC', MC2TSEASFC, M3REAL, 'K',
     &                       'Sea surface temperature',
     &                        NMC2, MC2NAMES )

                IF ( ISOIL .EQ. 1 ) THEN   !!  MLX soil layers for Dudhia LSM

                    DO L = 1, MLX
                         WRITE( DESC, '( A, I2 )' )
     &                       'Soil temperature for soil layer ', L
                         CALL CKVBLE( SOILTNAME(L), MC2SOILT(L),
     &                             M3REAL, 'K',
     &                             DESC,
     &                             NMC2, MC2NAMES )
                    END DO

                ELSE IF ( ISOIL .EQ. 2 ) THEN   !!  MLX soil layers for OSU LSM

                    CALL CKVBLE( 'SNODPT', MC2SNODPT, M3REAL, 'mm',
     &                           'Water equivalent snow depth',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'SEAICE', MC2SEAICE, M3REAL,'M^2/M^2',
     &                           'Sea ice coverage fraction',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'SRNOFF', MC2SRNOFF, M3REAL, 'mm',
     &                           'Surface runoff',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'URNOFF', MC2URNOFF, M3REAL, 'mm',
     &                           'Underground runoff',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'CANOM', MC2CANOM, M3REAL, 'M',
     &                           'canopy moisture',
     &                           NMC2, MC2NAMES )

                    CALL CKVBLE( 'QSKIN', MC2QSKIN, M3REAL, 'KG/KG',
     &                           'SKIN mixing ratio',
     &                           NMC2, MC2NAMES )

                    DO L = 1, MLX
                         WRITE( DESC, '( A, I2 )' )
     &                       'Soil temperature for soil layer ', L
                         CALL CKVBLE( SOILTNAME(L), MC2SOILT(L),
     &                             M3REAL, 'K',
     &                             DESC,
     &                             NMC2, MC2NAMES )
                    END DO
                    DO L = 1, MLX
                         WRITE( DESC, '( A, I2 )' )
     &                       'Soil moisture for soil layer ', L
                         CALL CKVBLE( SOILMNAME(L), MC2SOILM(L),
     &                             M3REAL, 'M^3/M^3',
     &                             DESC,
     &                             NMC2, MC2NAMES )
                    END DO

                ELSE IF ( ISOIL .EQ. 3 ) THEN        !!  2 layers for P-X LSM

                    CALL CKVBLE( 'CANOM', MC2CANOM, M3REAL, 'M',
     &                           'canopy moisture',
     &                           NMC2, MC2NAMES )

                    DO L = 1, 2
                         M = M + 1
                         WRITE( DESC, '( A, I2 )' )
     &                       'Soil temperature for soil layer ', L
                         CALL CKVBLE( SOILTNAME(L), MC2SOILT(L),
     &                             M3REAL, 'K',
     &                             DESC,
     &                             NMC2, MC2NAMES )
                    END DO
                    DO L = 1, 2
                         WRITE( DESC, '( A, I2 )' )
     &                       'Soil moisture for soil layer ', L
                         CALL CKVBLE( SOILMNAME(L), MC2SOILM(L),
     &                             M3REAL, 'M^3/M^3',
     &                             DESC,
     &                             NMC2, MC2NAMES )
                    END DO

                END IF          !  if isoil = 1, 2, 3

#endif     /*  ifdef  MM5_V3  */

                CALL CKVBLE( 'SLPRS', MC2SLPRS, M3REAL, 'mb',
     &                       'Sea level pressure',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'DZ1000_850', MC2DZ1000_850, M3REAL, 'm',
     &          'Thickness between 1000mb and 850mb surfaces',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'DZ1000_500', MC2DZ1000_500, M3REAL, 'm',
     &          'Thickness between 1000mb and 500mb surfaces',
     &                       NMC2, MC2NAMES )
                CALL CKVBLE( 'DZ850_700', MC2DZ850_700, M3REAL, 'm',
     &          'Thickness between 850mb and 700mb surfaces',
     &                       NMC2, MC2NAMES )

                NLAYS3D = 1
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = 'nonlayered and layer-1 (2D) ' //
     &                        'cross-point-grid meteorology'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( MC2FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // MC2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

            END IF      !  if mc2flag


C...........   Flag, variable and file initializations for MC3:

            IF ( MC3FLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'FAKE', MC3FAKE, M3REAL, 'none',
     &                       'dummy variable filled with 1.0E3',
     &                       NMC3, MC3NAMES )

                CALL CKVBLE( 'TA', MC3TA, M3REAL, 'K',
     &                       'actual air temperature',
     &                       NMC3, MC3NAMES )


                IF( IMPHYS( NEST ) .GE. 2 ) THEN
                    CALL CKVBLE( 'QV', MC3QV, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Water vapor mixing ratio',
     &                           NMC3, MC3NAMES )
                ELSE IF ( 0 .GE. INDEX1('QV', NMC3, MC3LIST ) ) THEN
                    MESG = 'Variable "QV" not available for ' //
     &                     MC3FILE( N )
                    CALL M3MSG2( MESG )
                END IF

                IF( IMPHYS( NEST ) .GE. 3 ) THEN
                    CALL CKVBLE( 'QC', MC3QC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Cloud-water mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QR', MC3QR, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Rain mixing ratio',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QC', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QC" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QR', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QR" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 5 ) THEN
                    CALL CKVBLE( 'QI', MC3QI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Cloud-ice mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QNI', MC3QNI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Rain mixing ratio',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QI', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QI" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNI', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QNI" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 6 ) THEN
                    CALL CKVBLE( 'QG', MC3QG, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Graupel mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QNC', MC3QNC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                           'Ice number concentration',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QG', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QG" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNC', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QNC" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                CALL CKVBLE( 'DENS', MC3DENS, M3REAL, 'kg/M^3',
     &                       'air density',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'PRES', MC3PRES, M3REAL, 'Pa',
     &                       'pressure',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'TV', MC3TV, M3REAL, 'K',
     &                       'virtual temperature',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'RH', MC3RH, M3REAL, '%',
     &          'relative humidity, 2-phase water/ice formula',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'RHLW', MC3RHLW, M3REAL, '%',
     &          'relative humidity, 1-phase water-only formula',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'RHKF', MC3RHKF, M3REAL, '%',
     &          'relative humidity, mixed-phase water/ice formula',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'THETA', MC3THETA, M3REAL, 'K',
     &                       'potential temperature',
     &                       NMC3, MC3NAMES )
                CALL CKVBLE( 'WWIND', MC3WWIND, M3REAL, 'M/S',
     &          'non-hydrostatic vertical wind component at SIGMAF',
     &                       NMC3, MC3NAMES )


                IF ( WNDWFLAG( N ) ) THEN

                    CALL CKVBLE( 'UX', MC3UX, M3REAL, 'M/S',
     &                       'horizontal cross point wind U component',
     &                       NMC3, MC3NAMES )
                    CALL CKVBLE( 'VX', MC3VX, M3REAL, 'M/S',
     &                       'horizontal cross point wind V component',
     &                       NMC3, MC3NAMES )
                    CALL CKVBLE( 'WSPD', MC3WSPD, M3REAL, 'M/S',
     &                       'horizontal wind speed',
     &                       NMC3, MC3NAMES )
                    CALL CKVBLE( 'DIV', MC3DIV, M3REAL, '1/s',
     &                       'horizontal wind divergence',
     &                       NMC3, MC3NAMES )
                    CALL CKVBLE( 'AVOR', MC3AVOR, M3REAL, '1/s',
     &                      'absolute horizontal wind vorticity',
     &                       NMC3, MC3NAMES )
                    CALL CKVBLE( 'RVOR', MC3RVOR, M3REAL, '1/s',
     &                      'relative horizontal wind vorticity',
     &                       NMC3, MC3NAMES )
                ELSE

                    MESG =
     &              'Invalid geometry to constuct WSPD,DIV,VOR for ' //
     &              MC3FILE( N )
                    CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                    WNDWERR =  WNDWERR .OR.
     &                        ( 0.NE.INDEX1( 'UX',   NMC3, MC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'VX',   NMC3, MC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'WSPD', NMC3, MC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'DIV',  NMC3, MC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'AVOR', NMC3, MC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'RVOR', NMC3, MC3NAMES ) )

                END IF  !  if speed-divergence-vorticity calc in bounds, or not


                IF( IMPHYS( NEST ) .GE. 2 ) THEN
                    CALL CKVBLE( 'QVA', MC3QVA, M3REAL, 'cb kg/kg',
     &              'pstar qv -- coupled water vapor mixing ratio',
     &                       NMC3, MC3NAMES )
                ELSE IF ( 0 .GE. INDEX1('QVA', NMC3, MC3LIST ) ) THEN
                    MESG = 'Variable "QVA" not available for ' //
     &                     MC3FILE( N )
                    CALL M3MSG2( MESG )
                END IF

                IF( IMPHYS( NEST ) .GE. 3 ) THEN
                    CALL CKVBLE( 'QCA', MC3QCA, M3REAL,
     &                           'cb kg/kg',
     &                 'pstar qc -- coupled cloud-water mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QRA', MC3QRA, M3REAL,
     &                           'cb kg/kg',
     &                    'pstar qc -- coupled rain mixing ratio',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QCA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QCA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QRA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QRA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 5 ) THEN
                    CALL CKVBLE( 'QIA', MC3QIA, M3REAL,
     &                           'cb kg/kg',
     &                   'pstar qi -- coupled cloud-ice mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QNIA', MC3QNIA, M3REAL,
     &                           'cb kg/kg',
     &                   'pstar qni -- coupled snow mixing ratio',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QIA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QIA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNIA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QNIA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 6 ) THEN
                    CALL CKVBLE( 'QGA', MC3QGA, M3REAL,
     &                           'cb kg/kg',
     &                   'pstar qg -- coupled graupel mixing ratio',
     &                           NMC3, MC3NAMES )
                    CALL CKVBLE( 'QNCA', MC3QNCA, M3REAL,
     &                           'cb/kg',
     &                   'pstar qnc -- coupled ice number conc',
     &                           NMC3, MC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QGA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QGA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNCA', NMC3, MC3LIST ) ) THEN
                        MESG = 'Variable "QNCA" not available for ' //
     &                         MC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF
	
                NLAYS3D = NLAYS
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) =
     &          'layered (3D) cross-point-grid MM5 meteorology'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = CNAME( N )
                IF ( .NOT. OPEN3( MC3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // MC3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

            END IF      !  if mc3flag


C...........   Flag, variable and file initializations for PC3:

            IF ( PC3FLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'PDEX', PC3PDEX, M3INT, 'none',
     &          'Index table for pressure-level interpolation',
     &                       NPC3, PC3NAMES )

                CALL CKVBLE( 'CLIN', PC3CLIN, M3REAL, 'none',
     &          'Coefficient table for p-level linear interpolation',
     &                       NPC3, PC3NAMES )

                CALL CKVBLE( 'CLOG', PC3CLOG, M3REAL, 'none',
     &          'Coefficient table for p-level log interpolation',
     &                       NPC3, PC3NAMES )

                CALL CKVBLE( 'CPOW', PC3CPOW, M3REAL, 'none',
     &          'Coefficient table for p-level power interpolation',
     &                       NPC3, PC3NAMES )

                CALL CKVBLE( 'Z', PC3Z, M3REAL, 'm',
     &                       'Elevation above mean sea level',
     &                       NPC3, PC3NAMES )

                IF( IMPHYS( NEST ) .GE. 2 ) THEN
                    CALL CKVBLE( 'QV', PC3QV, M3REAL,
     &                           'g-H2O/g-dryair',
     &               'Water vapor mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                ELSE IF ( 0 .GE. INDEX1('QV', NPC3, PC3NAMES ) ) THEN
                    MESG = 'Variable "QV" not available for ' //
     &                     PC3FILE( N )
                    CALL M3MSG2( MESG )
                END IF

                IF( IMPHYS( NEST ) .GE. 3 ) THEN
                    CALL CKVBLE( 'QC', PC3QC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                  'Cloud-water mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                    CALL CKVBLE( 'QR', PC3QR, M3REAL,
     &                           'g-H2O/g-dryair',
     &                   'Rain mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QC', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QC" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QR', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QR" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 5 ) THEN
                    CALL CKVBLE( 'QI', PC3QI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                   'Cloud-ice mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                    CALL CKVBLE( 'QNI', PC3QNI, M3REAL,
     &                           'g-H2O/g-dryair',
     &                  'Snow mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QI', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QI" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNI', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QNI" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                IF( IMPHYS( NEST ) .GE. 6 ) THEN
                    CALL CKVBLE( 'QG', PC3QG, M3REAL,
     &                           'g-H2O/g-dryair',
     &                   'Graupel mixing ratio , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                    CALL CKVBLE( 'QNC', PC3QNC, M3REAL,
     &                           'g-H2O/g-dryair',
     &                  'Ice number concentration , or MISSING=-9.999',
     &                           NPC3, PC3NAMES )
                ELSE
                    IF ( 0 .GE. INDEX1('QG', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QG" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                    IF ( 0 .GE. INDEX1('QNC', NPC3, PC3NAMES ) ) THEN
                        MESG = 'Variable "QNC" not available for ' //
     &                         PC3FILE( N )
                        CALL M3MSG2( MESG )
                    END IF
                END IF

                CALL CKVBLE( 'TA', PC3TA, M3REAL, 'K',
     &                       'actual air temperature',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'DENS', PC3DENS, M3REAL, 'kg/M^3',
     &                       'air density',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'TV', PC3TV, M3REAL, 'K',
     &                       'virtual temperature',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'RH', PC3RH, M3REAL, '%',
     &               'relative humidity, 2-phase water.ice formula',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'RHLW', PC3RHLW, M3REAL, '%',
     &               'relative humidity, 1-phase water-only formula',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'RHKF', PC3RHKF, M3REAL, '%',
     &              'relative humidity, mixed-phase water.ice formula',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'THETA', PC3THETA, M3REAL, 'K',
     &                       'potential temperature',
     &                       NPC3, PC3NAMES )
                CALL CKVBLE( 'WWIND', PC3WWIND, M3REAL, 'M/S',
     &                       'non-hydrostatic vertical wind component',
     &                       NPC3, PC3NAMES )

                IF ( WNDWFLAG( N ) ) THEN

                    CALL CKVBLE( 'WSPD', PC3WSPD, M3REAL, 'M/S',
     &                       'horizontal wind speed',
     &                       NPC3, PC3NAMES )
                    CALL CKVBLE( 'DIV', PC3DIV, M3REAL, '1/s',
     &                       'horizontal wind divergence',
     &                       NPC3, PC3NAMES )
                    CALL CKVBLE( 'AVOR', PC3AVOR, M3REAL, '1/s',
     &                      'absolute horizontal wind vorticity',
     &                       NPC3, PC3NAMES )
                    CALL CKVBLE( 'RVOR', PC3RVOR, M3REAL, '1/s',
     &                      'relative horizontal wind vorticity',
     &                       NPC3, PC3NAMES )
                    CALL CKVBLE( 'UWIND', PC3UWIND, M3REAL, 'M/S',
     &             'horizontal Easting wind component at cross points',
     &                       NPC3, PC3NAMES )
                    CALL CKVBLE( 'VWIND', PC3VWIND, M3REAL, 'M/S',
     &             'horizontal Northing wind component at cross points',
     &                       NPC3, PC3NAMES )
                ELSE

                    MESG =
     &              'Invalid geometry to constuct WSPD,DIV,VOR for ' //
     &              PC3FILE( N )
                    CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                    WNDWERR = WNDWERR .OR.
     &                        ( 0.NE.INDEX1( 'WSPD', NPC3, PC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'DIV',  NPC3, PC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'AVOR', NPC3, PC3NAMES )
     &                     .OR. 0.NE.INDEX1( 'RVOR', NPC3, PC3NAMES ) )

                END IF  !  if speed-divergence-vorticity calc in bounds, or not


                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) =
     &          'layered (3D) cross-point-grid MM5 meteorology ' //
     &          'at std pressure levels'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = CNAME( N )
                VGTYP3D = VGPRES3     !  set up pressure vertical coordinate

                NLAYS3D = PLAYS
                DO L = 1, PLAYS+1
                    VGLVS3D( L ) = PVALS( L )
                END DO
                IF ( .NOT. OPEN3( PC3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // PC3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                VGTYP3D = VGSGPN3       !  restore vertical coordinates
                VGTOP3D = VGTOP         !  in mb.
                NLAYS3D = MKX
                DO K = 1, KXP1
                    L = NLAYS - K + 2
                    VGLVS3D( L ) = SIGMA( K )   !  from /param3/
                END DO

            END IF      !  if PC3flag


C...........   Dot-point file initialization:
C...........   Set FDESC dot-point grid descriptions:

            IF ( DOTFILES ) THEN

                IF ( .NOT. DSCGRID( DNAME( N ), NAMBUF, GDTYP3D,
     &                    P_ALP3D, P_BET3D, P_GAM3D,
     &                    XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                    XCELL3D, YCELL3D,
     &                    NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
                    MESG = 'No grid description for "' //
     &              DNAME( N )( 1:TRIMLEN( DNAME( N ) ) ) // '"'
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF          !  if dscgrid() failed

            END IF      !  if dotfiles

C...........   Flag, variable and file initializations for GD2:

            IF ( GD2FLAG( N ) ) THEN

                NVARS3D = 0
                CALL CKVBLE( 'LAT', GD2LAT, M3REAL, 'deg',
     &          'latitude of the grid node -90 <= LAT <= 90',
     &                       NGD2, GD2NAMES )

                CALL CKVBLE( 'LON', GD2LON, M3REAL, 'deg',
     &          'longitude of the grid node -180 <= LON <= 180',
     &                       NGD2, GD2NAMES )

                CALL CKVBLE( 'MSFD2', GD2MSFD2, M3REAL, 'none',
     &          'SQUARE of the dot point map factor at the grid node',
     &                       NGD2, GD2NAMES )

                CALL CKVBLE( 'MSFD', GD2MSFD, M3REAL, 'none',
     &          'INVERSE of the dot point map factor at the grid node',
     &                       NGD2, GD2NAMES )

                NLAYS3D = 1
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) = 'nonlayered and layer-1 (2D) ' //
     &                        'dot-point-grid description'
                SDATE3D = 0
                STIME3D = 0
                TSTEP3D = 0
                GDNAM3D = DNAME( N )
                IF ( .NOT. OPEN3( GD2FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // GD2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                IF ( GD2LAT ) THEN !  XLAT from /addr2/
                    CALL MMX2D( 1.0, MIX, MJX, 1, XLAT,
     &                          NCDOT( N ), NRDOT( N ), 1,
     &                          CDOT0( N ), RDOT0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GD2FILE( N ), 'LAT',
     &                         0,0, RBUF ) ) THEN
                      MESG = 'Could not write LAT to ' //
     &                  GD2FILE( N )
                      CALL M3EXIT( 'MM5/MCPL', 0,0, MESG, 2 )
                    END IF
                END IF               !  if gd2lat

                IF ( GD2LON ) THEN !  XLONG from /addr2/
                    CALL MMX2D( 1.0, MIX, MJX, 1, XLONG,
     &                          NCDOT( N ), NRDOT( N ), 1,
     &                          CDOT0( N ), RDOT0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GD2FILE( N ), 'LON',
     &                         0,0, RBUF ) ) THEN
                        MESG = 'Could not write LON to ' //
     &                  GD2FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0,0, MESG, 2 )
                    END IF
                END IF               !  if gd2lon

                IF ( GD2MSFD2 ) THEN !  compute square of
                    CALL MM2MSQ( MIX, MJX, 1, MSFD,
     &                           NCDOT( N ), NRDOT( N ), 1,
     &                           CDOT0( N ), RDOT0( N ), 1, RBUF )
                    IF ( .NOT. WRITE3( GD2FILE( N ), 'MSFD2',
     &                                 0, 0, RBUF ) ) THEN
                        MESG = 'Could not write MSFD2 to ' //
     &                  GD2FILE( N )
                        CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                    END IF
                END IF               !  if gd2msfd2

              IF ( GD2MSFD ) THEN !  MSFD from /addr1/
                  CALL MM2IO( 1.0, MIX, MJX, 1,  MSFD,
     &                        NCDOT( N ), NRDOT( N ), 1,
     &                        CDOT0( N ), RDOT0( N ), 1, RBUF )
                  IF ( .NOT. WRITE3( GD2FILE( N ), 'MSFD',
     &                               0, 0, RBUF ) ) THEN
                      MESG = 'Could not write MSFD to ' //
     &                  GD2FILE( N )
                      CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                  END IF
              END IF               !  if gd2msfd

            END IF      !  if gd2flag


C...........   Flag, variable and file initializations for MD3:

            IF ( MD3FLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'FAKE', MD3FAKE, M3REAL, 'none',
     &                       'dummy variable filled with 1.0E3',
     &                       NMD3, MD3NAMES )

                IF ( WNDWFLAG( N ) ) THEN

                    CALL CKVBLE( 'UWIND', MD3UWIND, M3REAL, 'M/S',
     &              'Easting wind component, in map-scaled meters',
     &              NMD3, MD3NAMES )
                    CALL CKVBLE( 'VWIND', MD3VWIND, M3REAL, 'M/S',
     &              'Northing wind component, in map-scaled meters',
     &              NMD3, MD3NAMES )

                ELSE

                    MESG =
     &              'Invalid geometry to constuct UWIND,VWIND for ' //
     &              MD3FILE( N )
                    CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                    WNDWERR = WNDWERR .OR.
     &                        ( 0.NE.INDEX1( 'UWIND', NMD3, MD3NAMES )
     &                     .OR. 0.NE.INDEX1( 'VWIND', NMD3, MD3NAMES ) )

                END IF           !  if wndwflag( N ), or not

                CALL CKVBLE( 'UA', MD3UA, M3REAL, 'cb M/S',
     &          'pstar U -- coupled Easting wind component',
     &                       NMC3, MD3NAMES )
                CALL CKVBLE( 'VA', MD3VA, M3REAL, 'cb M/S',
     &          'pstar V -- coupled Northing wind component',
     &                       NMC3, MD3NAMES )

                NLAYS3D = NLAYS
                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) =
     &          'layered (3D) cross-point-grid meteorology'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                GDNAM3D = DNAME( N )
                IF ( .NOT. OPEN3( MD3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // MD3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

            END IF      !  if md3flag


C...........   Flag, variable and file initializations for PD3:

            IF ( PD3FLAG( N ) .AND. WNDWFLAG( N ) ) THEN

                NVARS3D = 0

                CALL CKVBLE( 'PDEX', PD3PDEX, M3INT, 'none',
     &          'Index table for pressure-level interpolation',
     &                       NPD3, PD3NAMES )

                CALL CKVBLE( 'CLIN', PD3CLIN, M3REAL, 'none',
     &          'Coefficient table for p-level linear interpolation',
     &                       NPD3, PD3NAMES )

                CALL CKVBLE( 'CLOG', PD3CLOG, M3REAL, 'none',
     &          'Coefficient table for p-level log interpolation',
     &                       NPD3, PD3NAMES )

                CALL CKVBLE( 'CPOW', PD3CPOW, M3REAL, 'none',
     &          'Coefficient table for p-level power interpolation',
     &                       NPD3, PD3NAMES )

                CALL CKVBLE( 'UWIND', PD3UWIND, M3REAL, 'M/S',
     &              'Easting wind component, in map-scaled meters',
     &              NPD3, PD3NAMES )

                CALL CKVBLE( 'VWIND', PD3VWIND, M3REAL, 'M/S',
     &              'Northing wind component, in map-scaled meters',
     &              NPD3, PD3NAMES )

                FTYPE3D = GRDDED3
                FDESC3D(1) = CLOUDTYPE //
     &                           'CLOUD PARAMETERIZATION IN MM5'
                FDESC3D( 2 ) =
     &          'layered (3D) dot-point-grid meteorology ' //
     &          'at std pressure levels'
                SDATE3D = SDATE( N )
                STIME3D = STIME( N )
                TSTEP3D = TSTEP( N )
                VGTYP3D = VGPRES3

                NLAYS3D = PLAYS         !  set up pressure vertical coordinate
                DO L = 1, PLAYS+1
                    VGLVS3D( L ) = PVALS( L )   !  from /param3/
                END DO
                GDNAM3D = DNAME( N )
                IF ( .NOT. OPEN3( PD3FILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                    MESG = 'Could not open ' // PD3FILE( N )
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF

                VGTYP3D = VGSGPN3       !  restore vertical coordinates
                VGTOP3D = VGTOP         !  in mb.
                NLAYS3D = MKX
                DO K = 1, KXP1
                    L = NLAYS - K + 2
                    VGLVS3D( L ) = SIGMA( K )   !  from /param3/
                END DO

            ELSE IF ( PD3FLAG( N ) ) THEN

                MESG = 'Bad window; cannot construct ' //
     &                  PD3FILE( N ) // ' for ' // GDNAM3D
                CALL M3WARN( 'MM5/MCPL', 0, 0, MESG )
                WNDWERR  = .TRUE.
                PD3FLAG( N ) = .FALSE.

            END IF      !  if PD3flag

            BDPTFLAG  = ( MB3DENS .OR. MB3PRES .OR. MB3TV .OR. MB3RH
     &               .OR. MB3RHLW .OR. MB3RHKF )
            C2DPTFLAG = ( MC2DENS .OR. MC2PRES .OR. MC2TV .OR. MC2RH
     &               .OR. MC2RHLW .OR. MC2RHKF .OR. MC2HFLX
     &               .OR. MC2QFLX )
            MC3DPTFLAG= ( MC3DENS .OR. MC3PRES .OR. MC3TV .OR. MC3RH
     &               .OR. MC3RHLW .OR. MC3RHKF .OR. MC3THETA
     &               .OR. MC2CFRACH .OR. MC2CFRACM .OR. MC2CFRACL )
            PC3DPTFLAG= ( PC3DENS .OR. PC3TA   .OR. PC3TV .OR. PC3RH
     &               .OR. PC3RHLW .OR. PC3RHKF .OR. PC3THETA )

            SDV2FLAG = ( MC2UX   .OR. MC2VX    .OR. MC2WSPD .OR.
     &                   MC2DIV  .OR. MC2AVOR  .OR. MC2RVOR )
            SDV3FLAG = ( MC3UX   .OR. MC3VX    .OR. MC3WSPD .OR.
     &                   MC3DIV  .OR. MC3AVOR  .OR. MC3RVOR )
            SDVPFLAG = ( PC3WSPD .OR. PC3DIV   .OR. PC3AVOR
     &              .OR. PC3RVOR .OR. PC3UWIND .OR. PC3VWIND )

            ZDZPFLAG = ( PC3Z           .OR. MC2DZ1000_850 .OR.
     &                   MC2DZ1000_500  .OR. MC2DZ850_700 )

        END IF          !  if firstime( N )


C........................................................................
C...............   Time-stepped File Output:
C...................   Met-Boundary-2D file:

#ifdef MCPL_OMP
!$OMP PARALLEL SECTIONS
!$OMP& DEFAULT( SHARED )

!$OMP     SECTION

#endif  /** MCPL_OMP  **/
      IF ( MB2FLAG( N ) ) THEN

        CALL MCPL_MB2( MB2FILE( N ), JDATE, JTIME, NEST, N,
     &                 NCCRO( N ), NRCRO( N ), 1, NTHIK( N ),
     &                 CCRO0( N ), RCRO0( N ) )

      END IF    !  if mb2flag(n)

C...................   Met-Boundary-3D file:

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( MB3FLAG( N ) ) THEN

        CALL MCPL_MB3( MB3FILE( N ), JDATE, JTIME, NEST, N,
     &                 NCCRO( N ), NRCRO( N ), NLAYS, NTHIK( N ),
     &                 CCRO0( N ), RCRO0( N ), SIGH )

      END IF            !  if mb3flag(n)

C...................   Met-Cross-2D file:

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( MC2FLAG( N ) ) THEN

        CALL MCPL_MC2( MC2FILE( N ), JDATE, JTIME, NEST, N,
     &                 NCCRO( N ), NRCRO( N ), 1,
     &                 CCRO0( N ), RCRO0( N ), SIGH, PLAYS, PVALH )


      END IF            !   if mc2flag(n)

C...................   Met-Cross-3D file:

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( MC3FLAG( N ) ) THEN

        CALL MCPL_MC3( MC3FILE( N ), MC2FILE( N ),
     &                 JDATE, JTIME, NEST, N,
     &                 NCCRO( N ), NRCRO( N ), NLAYS,
     &                 CCRO0( N ), RCRO0( N ), SIGH )

      END IF                    !  if mc3flag(n)

C...................   Met-Dot-3D file:

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( MD3FLAG( N ) ) THEN

        CALL MCPL_MD3( MD3FILE( N ), JDATE, JTIME, NEST, N,
     &                 NCDOT( N ), NRDOT( N ), NLAYS,
     &                 CDOT0( N ), RDOT0( N ) )

      END IF            !  if md3flag(n)


C...................   Pres-Cross-3D file + DZ*_* variables for MC2:
C...................   ZDZPFLAG says "either PC3 Z or MC2 DZ_*"

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( PC3FLAG( N ) ) THEN

        CALL MCPL_PC3( PC3FILE( N ), JDATE, JTIME, NEST, N,
     &                 NCCRO( N ), NRCRO( N ), PLAYS,
     &                 CCRO0( N ), RCRO0( N ), SIGH, PVALH )

      END IF            !  if pc3flag(n)


C...................   Pres-Dot-3D file, Pres-Cross-3 variables coming
C...................   from MM5 dot point variables:

#ifdef MCPL_OMP
!$OMP     SECTION
#endif  /** MCPL_OMP  **/

      IF ( WNDWFLAG( N ) .AND. PD3FLAG( N ) ) THEN

          CALL MCPL_PD3( PD3FILE( N ), JDATE, JTIME, NEST, N,
     &                   NCDOT( N ), NRDOT( N ), PLAYS,
     &                   CDOT0( N ), RDOT0( N ), SIGH, PVALH )

        END IF		!  if wndwflag(n) and pd3flag(n)


#ifdef MCPL_OMP
!$OMP   END PARALLEL SECTIONS
#endif  /** MCPL_OMP  **/

C...................   Processing for this output grid complete

        FIRSTIME( N ) = .FALSE.

999     CONTINUE                !  end loop on output grids for this MM5 nest

C...........   Check for output grid or boundary geometry errors:

        IF ( WNDWERR .OR. BNDYERR ) THEN
            WRITE( MESG, '(A, I3 )' )
     &       'Errors in geometry specification for grid', NEST
            CALL M3EXIT( 'MM5/MCPL', JDATE, JTIME, MESG, 2 )
        END IF


        RETURN  !!!!!!!!!!!!!!!!  end ENTRY MCPL_OUT  !!!!!!!!!!!!!!!!!


C.........................................................................
C...............   Get I/O API-convention date:time for current MM5 XTIME:

        ENTRY MCPL_TIME( IDATE, ITIME )

        IF ( FIRSTIMEG ) THEN         !  no MCPL_GRID() call yet
            MESG = 'Must call MCPL_GRID() before MCPL_TIME()'
            CALL M3EXIT( 'MM5/MCPL/MCPL_TIME', JDATE, JTIME, MESG, 2 )
        END IF

        IDATE = MDATE0                             !  MM5 start date
        ITIME = MTIME0                             !  MM5 start time
        KSTEP = SEC2TIME( NINT( 60.0 * XTIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( IDATE, ITIME, KSTEP )        ! current simulation clock

        RETURN !!!!!!!!!!!!!!!!  end ENTRY MCPL_TIME !!!!!!!!!!!!!!!!!


C.........................................................................
C...............   Get I/O API-convention date:time for current MM5 XTIME:

        ENTRY MCPL_ATIME( ATIME, IDATE, ITIME )

        IF ( FIRSTIMEG ) THEN         !  no MCPL_GRID() call yet
            MESG = 'Must call MCPL_GRID() before MCPL_ATIME()'
            CALL M3EXIT( 'MM5/MCPL/MCPL_ATIME', JDATE, JTIME, MESG, 2 )
        END IF

        IDATE = MDATE0                             !  MM5 start date
        ITIME = MTIME0                             !  MM5 start time
        KSTEP = SEC2TIME( NINT( 60.0 * ATIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( IDATE, ITIME, KSTEP )        ! current simulation clock

        RETURN !!!!!!!!!!!!!!!!  end ENTRY MCPL_TIME !!!!!!!!!!!!!!!!!


C............................................................................
C......   File processing Segment for "High-Temporal-Resolution" (HTR) files:

        ENTRY MCPL_HTR( NEST )

        IF ( .NOT. SCFILES ) THEN
            RETURN
        END IF


C...........   Check that MCPL initialized, and that argument NEST
C...........   is in bounds and active

        IF ( FIRSTIMEG ) THEN         !  no MCPL_GRID() call yet
            MESG = 'Must call MCPL_GRID() before MCPL_OUT()'
            CALL M3EXIT( 'MM5/MCPL', KDATE, KTIME, MESG, 2 )
        ELSE IF ( NEST .GT. MAXNES .OR. NEST .LE. 0 ) THEN   !  nest out of range
            WRITE( MESG, '(A, I5, 2X, A, I2, A )' )
     &      'Argument NEST =', NEST, 'not in range [1:', MXFIL, ']'
            CALL M3EXIT( 'MM5:MCPL', KDATE, KTIME, MESG, 2 )
        ELSE IF ( IACTIV( NEST ) .NE. 1 ) THEN  !  if nest inactive
            RETURN
        END IF                  !  if nest out of range

C...........   Compute current simulation date and time KDATE:KTIME;

        KDATE = MDATE0                             !  MM5 start date
        KTIME = MTIME0                             !  MM5 start time
        KSTEP = SEC2TIME( NINT( 60.0 * XTIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( KDATE, KTIME, KSTEP )        ! current simulation clock

        DO  1222  N = GRIDCUM( NEST-1 )+1, GRIDCUM( NEST )

            IF ( .NOT.HTRFLAG( N ) ) GO TO 1222

            IREC = CURREC( KDATE, KTIME,           !  current simulation clock
     &                     HTRDATE( N ), HTRTIME( N ), HTRSTEP( N ),
     &                     JDATE, JTIME )          !  "left edge" of timestep

            IF ( IREC .EQ. -1         ) GO TO 1222  !! before start date:time for N
            IF ( IREC .EQ. LASTHTR(N) ) GO TO 1222  !! already wrote JDATE:JTIME

            LASTHTR( N ) = IREC

            IF ( FIRSTSC( N ) ) THEN

                IF ( .NOT. DSCGRID( CNAME(N), NAMBUF, GDTYP3D,
     &                              P_ALP3D, P_BET3D, P_GAM3D,
     &                              XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                              XCELL3D, YCELL3D,
     &                              NCOLS3D, NROWS3D, NTHIK3D ) ) THEN
                    MESG = 'No grid description for "' //
     &                      CNAME( N )( 1:TRIMLEN( CNAME(N) ) ) // '"'
                    CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
                END IF          !  if dscgrid() failed

                GDNAM3D = CNAME( N )
                SDATE3D = HTRDATE( N )
                STIME3D = HTRTIME( N )
                TSTEP3D = HTRSTEP( N )

                FTYPE3D = GRDDED3
                VGTYP3D = IMISS3
                VGTOP3D = 0.0
                NLAYS3D = 1

                DO K = 1, 2
                    VGLVS3D( K ) = FLOAT( K - 1 )
                END DO

                V = 0
                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TWET1'
                UNITS3D(V) = 'K'
                VDESC3D(V) = 'Layer-1 wet-bulb air temperature'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TWMEAN'
                UNITS3D(V) = 'K'
                VDESC3D(V) =
     &          'Integrated near-surface wet-bulb air temperature'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'MASKFRAC'
                UNITS3D(V) = 'none'
                VDESC3D(V) =
     &          'Precip frac: freezing <= 0 < mixed < 1 <= liquid'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'QRLAY1'
                UNITS3D(V) = 'g-H2O-liq-ppt/g-dryair'
                VDESC3D(V) = 'layer-1 rain mixing ratio at each cell'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'QNILAY1'
                UNITS3D(V) = 'g-H2O-ice-ppt/g-dryair'
                VDESC3D(V) = 'layer-1 ice mixing ratio at each cell'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'RAIN'
                UNITS3D(V) = 'cm'
                VDESC3D(V) =
     &          'accumulated total (convective + non-convective) rain'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TALAY1'
                UNITS3D(V) = 'K'
                VDESC3D(V) = 'layer-1 half-sigma air temperature'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TNSFC'
                UNITS3D(V) = 'K'
                VDESC3D(V) = 'mean near-sfc half-sigma air temp'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TA2'
                UNITS3D(V) = 'K'
                VDESC3D(V) = '2 meter air temperature'
		
                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'TDEW2'
                UNITS3D(V) = 'K'
                VDESC3D(V) = '2-Meter Dew Point'		

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'UX10'
                UNITS3D(V) = 'M/S'
                VDESC3D(V) = '10 meter U wind component'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'VX10'
                UNITS3D(V) = 'M/S'
                VDESC3D(V) = '10 meter V wind component'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'CFRACL'
                UNITS3D(V) = 'none'
                VDESC3D(V) = 'MM5 low-cloud fractional coverage'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'CFRACM'
                UNITS3D(V) = 'none'
                VDESC3D(V) = 'MM5 medium-cloud fractional coverage'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'CFRACH'
                UNITS3D(V) = 'none'
                VDESC3D(V) = 'MM5 high-cloud fractional coverage'

                V = V + 1
                VTYPE3D(V) = M3REAL
                VNAME3D(V) = 'SLPRS'
                UNITS3D(V) = 'mb'
                VDESC3D(V) = 'MM5 Mean-sea-level pressure'

                DO L = 1, NPHTR

                    V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( UNAMHTR(L), '( A, I4.4 )' )
     &                    'UWIND', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = UNAMHTR( L )
                    UNITS3D(V) = 'M/S'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &                    'cross point horizontal U wind component at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                    V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( VNAMHTR(L), '( A, I4.4 )' )
     &                    'VWIND', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = VNAMHTR(L)
                    UNITS3D(V) = 'M/S'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &                    'cross point horizontal V wind component at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                    V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( ZNAMHTR(L), '( A, I4.4 )' )
     &                    'Z', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = ZNAMHTR(L)
                    UNITS3D(V) = 'M'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &                    'Elevation above mean sea level at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                     V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( HNAMHTR(L), '( A, I4.4 )' )
     &                    'RH', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = HNAMHTR(L)
                    UNITS3D(V) = '%'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &              'relative humidity, 1-phase water-only formula, at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                    V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( TNAMHTR(L), '( A, I4.4 )' )
     &                    'TA', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = TNAMHTR(L)
                    UNITS3D(V) = 'K'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &              'Temperature, at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                    V = V + 1
                    VTYPE3D(V) = M3REAL
                    WRITE( ANAMHTR(L), '( A, I4.4 )' )
     &                    'AVOR', NINT( PATOMB*PVHTR( L ) )
                    VNAME3D(V) = ANAMHTR(L)
                    UNITS3D(V) = 'none'
                    WRITE( VDESC3D(V), '(A, I4, 1X, A )' )
     &              'Absolute vorticity, at',
     &                    NINT( PATOMB*PVHTR( L ) ), 'mb'

                END DO  !  end loop on pressure levels L

                NVARS3D = V

                FDESC3D( 1 ) =
     &          'High-Temporal-Resolution 2D file:'
                FDESC3D( 2 ) =
     &          'Ice/Mixed/Liquid-precipitation mask, '
                FDESC3D( 3 ) =
     &          'Wet-bulb-feezing height, total rain, etc.'

                DO  L = 4, MXDESC3
                    FDESC3D( L ) = ' '
                END DO

                IF ( .NOT. OPEN3( HTRFILE( N ), FSUNKN3,
     &                            'MM5/MCPL' ) ) THEN
                   MESG = 'Could not open ' // HTRFILE( N )
                   CALL M3EXIT( 'MM5/MCPL_HTR', 0, 0, MESG, 2 )
                END IF

            END IF

            !!  mm2mask() produces the following:
            !!  RBUF(1, 1) = TWET1  = modeled layer-1 wet-bulb temperature (K)
            !!  RBUF(1, 2) = TWMEAN = modeled integrated near-surface temperature (K)
            !!  RBUF(1, 3) = MASKFRAC = floating-point sliding mask
            !!  RBUF(1, 4) = QRLAY1   = layer-1 QR
            !!  RBUF(1, 5) = QNILAY1  = layer-1 QNI
            !!  RBUF(1, 6) = RAIN     = total (cumulative) precip
	    !!  RBUF(1, 7) = TALAY1   = layer-1 half-sigma air temp
            !!  RBUF(1, 8) = TNSFC    = mean near-sfc half-sigma air temp	    
            !!  RBUF(1, 9) = TA2      = 2-meter temperature
            !!  RBUF(1,10) = TDEW2    = 2-meter dew point (K)	    
            !!  RBUF(1,11) = UX10     = 10-meter U-component wind
            !!  RBUF(1,12) = VX10     = 10-meter V-component wind
            !!  RBUF(1,13) = CFRACL   = low    cloud fraction
            !!  RBUF(1,14) = CFRACM   = middle cloud fraction
            !!  RBUF(1,15) = CFRACH   = high   cloud fraction
	    !!  RBUF(1,16) = SLPRS    = mean-sea-level pressure (mb)
            !!  UBUF(*)    = UWIND    = pressure-interpolated cross pt U wind
            !!  VBUF(*)    = VWIND    = pressure-interpolated cross pt V wind
            !!  SBUF(*)    = Z        = pressure-surface elevations
            !!  TBUF(*)    = RH       = pressure-surface relative humidity
            !!  WBUF(*)    = TA       = pressure-surface temperature	
            !!  XBUF(*)    = AVOR     = pressure-surface absolute vorticity            
            	    
            CALL MM2HTR9( DX, MIX, MJX, MKX, NEARSFC,
     &                  TKICE, TKLIQ, TKFRZ, XLAND,
     &                  TA, QVA, QRA, QNIA, PPA, UA, VA, PSA, RAINC,
     &                  RAINNC, TA2, QA2, UA10, VA10, PBL, HT, F,
     &                  SIGH, PTOP, P0, TLP, TS0, ROVG,
     &                  NCCRO( N ), NRCRO( N ), NLAYS,
     &                  NPDIM, NPHTR, PVHTR,
     &                  CCRO0( N ), RCRO0( N ), 1,
     &                  RBUF(1, 1), RBUF(1, 2), RBUF(1, 3), RBUF(1, 4),
     &                  RBUF(1, 5), RBUF(1, 6), RBUF(1, 7), RBUF(1, 8),
     &                  RBUF(1, 9), RBUF(1,10), RBUF(1,11), RBUF(1,12),
     &                  RBUF(1,13), RBUF(1,14), RBUF(1,15), RBUF(1,16),
     &                  UBUF, VBUF, SBUF, TBUF, WBUF, XBUF )

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TWET1',
     &                         JDATE, JTIME, RBUF(1,1) ) ) THEN
                MESG = 'Could not write TWET1 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TWMEAN',
     &                         JDATE, JTIME, RBUF(1,2) ) ) THEN
                MESG = 'Could not write TWMEAN to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'MASKFRAC',
     &                         JDATE, JTIME, RBUF(1,3) ) ) THEN
                MESG = 'Could not write MASKFRAC to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'QRLAY1',
     &                         JDATE, JTIME, RBUF(1,4) ) ) THEN
                MESG = 'Could not write QRLAY1 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'QNILAY1',
     &                         JDATE, JTIME, RBUF(1,5) ) ) THEN
                MESG = 'Could not write QNILAY1 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'RAIN',
     &                         JDATE, JTIME, RBUF(1,6) ) ) THEN
                MESG = 'Could not write RAIN to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TALAY1',
     &                         JDATE, JTIME, RBUF(1,7) ) ) THEN
                MESG = 'Could not write TALAY1 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TNSFC',
     &                         JDATE, JTIME, RBUF(1,8) ) ) THEN
                MESG = 'Could not write TNSFC to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TA2',
     &                         JDATE, JTIME, RBUF(1,9) ) ) THEN
                MESG = 'Could not write TA2 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'TDEW2',
     &                         JDATE, JTIME, RBUF(1,10) ) ) THEN
                MESG = 'Could not write TDEW2 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'UX10',
     &                         JDATE, JTIME, RBUF(1,11) ) ) THEN
                MESG = 'Could not write UX10 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'VX10',
     &                         JDATE, JTIME, RBUF(1,12) ) ) THEN
                MESG = 'Could not write VX10 to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'CFRACL',
     &                         JDATE, JTIME, RBUF(1,13) ) ) THEN
                MESG = 'Could not write CFRACL to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'CFRACM',
     &                         JDATE, JTIME, RBUF(1,14) ) ) THEN
                MESG = 'Could not write CFRACM to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'CFRACH',
     &                         JDATE, JTIME, RBUF(1,15) ) ) THEN
                MESG = 'Could not write CFRACH to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            IF ( .NOT. WRITE3( HTRFILE( N ), 'SLPRS',
     &                         JDATE, JTIME, RBUF(1,16) ) ) THEN
                MESG = 'Could not write SLPRS to ' // HTRFILE( N )
                CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
            END IF

            DO L = 1, NPHTR     !  loop on interpolated variables

                M = 1 + ( L - 1 ) * NCCRO( N ) * NRCRO( N )
                IF ( .NOT. WRITE3( HTRFILE( N ), UNAMHTR(L),
     &                             JDATE, JTIME, UBUF(M) ) ) THEN
                    MESG = 'Could not write "' // UNAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF

                IF ( .NOT. WRITE3( HTRFILE( N ), VNAMHTR(L),
     &                             JDATE, JTIME, VBUF(M) ) ) THEN
                    MESG = 'Could not write "' // VNAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF

                IF ( .NOT. WRITE3( HTRFILE( N ), ZNAMHTR(L),
     &                             JDATE, JTIME, SBUF(M) ) ) THEN
                    MESG = 'Could not write "' // ZNAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF

                 IF ( .NOT. WRITE3( HTRFILE( N ), HNAMHTR(L),
     &                             JDATE, JTIME, TBUF(M) ) ) THEN
                    MESG = 'Could not write "' // HNAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF

                 IF ( .NOT. WRITE3( HTRFILE( N ), TNAMHTR(L),
     &                             JDATE, JTIME, WBUF(M) ) ) THEN
                    MESG = 'Could not write "' // TNAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF

                 IF ( .NOT. WRITE3( HTRFILE( N ), ANAMHTR(L),
     &                             JDATE, JTIME, XBUF(M) ) ) THEN
                    MESG = 'Could not write "' // ANAMHTR(L) //
     &                     '" to ' // HTRFILE( N )
                    CALL M3EXIT('MM5/MCPL_HTR', JDATE,JTIME,MESG, 2)
                END IF


           END DO

1222    CONTINUE     !  end loop on grids in this MM5 nest


        RETURN  !!!!!!!!!!!!!!!!  end ENTRY MCPL_HTR  !!!!!!!!!!!!!!!!!

	END

